"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
const path = __importStar(require("path"));
const parser_1 = require("./parser");
const fileWriter = __importStar(require("./file-writer"));
const declarations_1 = require("./declarations");
const object_converter_1 = require("./object-converter");
const events_1 = require("./events");
const common_properties_1 = require("./common-properties");
const ts_parser_1 = require("./ts-parser");
const debug_patch_1 = require("./debug-patch");
const logger_1 = require("./logger");
function writeToFile(outputDir, contents, fileName, emitProgressMessage) {
    const json = JSON.stringify(contents);
    const outputFile = path.resolve(outputDir, `${fileName}.json`);
    logger_1.logMessage(`...writing ${outputFile}`, !emitProgressMessage);
    return fileWriter.writeFile(outputFile, json);
}
exports.writeToFile = writeToFile;
async function getResolvedDeclaration(sourceDirs, excludedDirs, options) {
    logger_1.logMessage('...extracting', options.silenceOutput);
    const parser = new parser_1.Parser(sourceDirs, excludedDirs, options.applyEndpoints, options.lowerCaseEvents);
    let declarations = await parser.extractDeclarations();
    declarations = declarations_1.resolveDeclarations(declarations, options.verbose);
    return declarations;
}
exports.getResolvedDeclaration = getResolvedDeclaration;
async function extractAndResolveDeclarations(sourceSpecs) {
    try {
        const allDeclarations = { events: new events_1.Events(), commonProperties: new common_properties_1.CommonProperties() };
        const allTypeScriptDeclarations = Object.create(null);
        for (const spec of sourceSpecs) {
            const declarations = await getResolvedDeclaration(spec.sourceDirs, spec.excludedDirs, spec.parserOptions);
            let typescriptDeclarations = Object.create(null);
            // The parser does not know how to handle multiple source directories due to different TS configs, so we manually have to parse each source dir
            spec.sourceDirs.forEach((dir) => {
                Object.assign(typescriptDeclarations, new ts_parser_1.TsParser(dir, spec.excludedDirs, spec.parserOptions.applyEndpoints, spec.parserOptions.lowerCaseEvents).parseFiles());
            });
            if (spec.parserOptions.eventPrefix !== '') {
                declarations.events.dataPoints = declarations.events.dataPoints.map((event) => {
                    event.name = spec.parserOptions.eventPrefix + event.name;
                    return event;
                });
                const modifiedDeclartions = Object.create(null);
                // Modify the object keys to be prefixed with the specified prefix
                for (const key in typescriptDeclarations) {
                    modifiedDeclartions[spec.parserOptions.eventPrefix + key] = typescriptDeclarations[key];
                }
                typescriptDeclarations = modifiedDeclartions;
            }
            if (spec.parserOptions.patchDebugEvents) {
                debug_patch_1.patchDebugEvents(declarations.events, spec.parserOptions.eventPrefix);
            }
            // We concatenate each extensions properties into a central one
            // Throwing out fragments as they have already been used to resolve that extensions declarations
            allDeclarations.commonProperties.properties = allDeclarations.commonProperties.properties.concat(declarations.commonProperties.properties);
            allDeclarations.events.dataPoints = allDeclarations.events.dataPoints.concat(declarations.events.dataPoints);
            Object.assign(allTypeScriptDeclarations, typescriptDeclarations);
        }
        const formattedDeclarations = await object_converter_1.transformOutput(allDeclarations);
        for (const dec in allTypeScriptDeclarations) {
            formattedDeclarations.events[dec] = allTypeScriptDeclarations[dec];
        }
        return Promise.resolve(formattedDeclarations);
    }
    catch (error) {
        console.error(`Error: ${error}`);
        return Promise.reject(error);
    }
}
exports.extractAndResolveDeclarations = extractAndResolveDeclarations;
//# sourceMappingURL=save-declarations.js.map