"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const workbench_web_api_1 = require("vs/workbench/workbench.web.api");
const uuid_1 = require("vs/base/common/uuid");
const cancellation_1 = require("vs/base/common/cancellation");
const buffer_1 = require("vs/base/common/buffer");
const lifecycle_1 = require("vs/base/common/lifecycle");
const request_1 = require("vs/base/parts/request/browser/request");
class LocalStorageCredentialsProvider {
    get credentials() {
        if (!this._credentials) {
            try {
                const serializedCredentials = window.localStorage.getItem(LocalStorageCredentialsProvider.CREDENTIALS_OPENED_KEY);
                if (serializedCredentials) {
                    this._credentials = JSON.parse(serializedCredentials);
                }
            }
            catch (error) {
                // ignore
            }
            if (!Array.isArray(this._credentials)) {
                this._credentials = [];
            }
        }
        return this._credentials;
    }
    save() {
        window.localStorage.setItem(LocalStorageCredentialsProvider.CREDENTIALS_OPENED_KEY, JSON.stringify(this.credentials));
    }
    getPassword(service, account) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.doGetPassword(service, account);
        });
    }
    doGetPassword(service, account) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const credential of this.credentials) {
                if (credential.service === service) {
                    if (typeof account !== 'string' || account === credential.account) {
                        return credential.password;
                    }
                }
            }
            return null;
        });
    }
    setPassword(service, account, password) {
        return __awaiter(this, void 0, void 0, function* () {
            this.deletePassword(service, account);
            this.credentials.push({ service, account, password });
            this.save();
        });
    }
    deletePassword(service, account) {
        return __awaiter(this, void 0, void 0, function* () {
            let found = false;
            this._credentials = this.credentials.filter(credential => {
                if (credential.service === service && credential.account === account) {
                    found = true;
                    return false;
                }
                return true;
            });
            if (found) {
                this.save();
            }
            return found;
        });
    }
    findPassword(service) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.doGetPassword(service);
        });
    }
    findCredentials(service) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.credentials
                .filter(credential => credential.service === service)
                .map(({ account, password }) => ({ account, password }));
        });
    }
}
LocalStorageCredentialsProvider.CREDENTIALS_OPENED_KEY = 'credentials.provider';
class PollingURLCallbackProvider extends lifecycle_1.Disposable {
    constructor() {
        super(...arguments);
        this._onCallback = this._register(new workbench_web_api_1.Emitter());
        this.onCallback = this._onCallback.event;
    }
    create(options) {
        const queryValues = new Map();
        const requestId = uuid_1.generateUuid();
        queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.REQUEST_ID, requestId);
        const { scheme, authority, path, query, fragment } = options ? options : { scheme: undefined, authority: undefined, path: undefined, query: undefined, fragment: undefined };
        if (scheme) {
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.SCHEME, scheme);
        }
        if (authority) {
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.AUTHORITY, authority);
        }
        if (path) {
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.PATH, path);
        }
        if (query) {
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.QUERY, query);
        }
        if (fragment) {
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.FRAGMENT, fragment);
        }
        // Start to poll on the callback being fired
        this.periodicFetchCallback(requestId, Date.now());
        return this.doCreateUri('/callback', queryValues);
    }
    periodicFetchCallback(requestId, startTime) {
        return __awaiter(this, void 0, void 0, function* () {
            // Ask server for callback results
            const queryValues = new Map();
            queryValues.set(PollingURLCallbackProvider.QUERY_KEYS.REQUEST_ID, requestId);
            const result = yield request_1.request({
                url: this.doCreateUri('/fetch-callback', queryValues).toString(true)
            }, cancellation_1.CancellationToken.None);
            // Check for callback results
            const content = yield buffer_1.streamToBuffer(result.stream);
            if (content.byteLength > 0) {
                try {
                    this._onCallback.fire(workbench_web_api_1.URI.revive(JSON.parse(content.toString())));
                }
                catch (error) {
                    console.error(error);
                }
                return; // done
            }
            // Continue fetching unless we hit the timeout
            if (Date.now() - startTime < PollingURLCallbackProvider.FETCH_TIMEOUT) {
                setTimeout(() => this.periodicFetchCallback(requestId, startTime), PollingURLCallbackProvider.FETCH_INTERVAL);
            }
        });
    }
    doCreateUri(path, queryValues) {
        let query = undefined;
        if (queryValues) {
            let index = 0;
            queryValues.forEach((value, key) => {
                if (!query) {
                    query = '';
                }
                const prefix = (index++ === 0) ? '' : '&';
                query += `${prefix}${key}=${encodeURIComponent(value)}`;
            });
        }
        return workbench_web_api_1.URI.parse(window.location.href).with({ path, query });
    }
}
PollingURLCallbackProvider.FETCH_INTERVAL = 500; // fetch every 500ms
PollingURLCallbackProvider.FETCH_TIMEOUT = 5 * 60 * 1000; // ...but stop after 5min
PollingURLCallbackProvider.QUERY_KEYS = {
    REQUEST_ID: 'vscode-requestId',
    SCHEME: 'vscode-scheme',
    AUTHORITY: 'vscode-authority',
    PATH: 'vscode-path',
    QUERY: 'vscode-query',
    FRAGMENT: 'vscode-fragment'
};
const options = JSON.parse(document.getElementById('vscode-workbench-web-configuration').getAttribute('data-settings'));
options.urlCallbackProvider = new PollingURLCallbackProvider();
options.credentialsProvider = new LocalStorageCredentialsProvider();
if (options.folderUri) {
    options.folderUri = workbench_web_api_1.URI.revive(options.folderUri);
}
if (options.workspaceUri) {
    options.workspaceUri = workbench_web_api_1.URI.revive(options.workspaceUri);
}
if (Array.isArray(options.staticExtensions)) {
    options.staticExtensions.forEach(extension => {
        extension.extensionLocation = workbench_web_api_1.URI.revive(extension.extensionLocation);
    });
}
workbench_web_api_1.create(document.body, options);
//# sourceMappingURL=workbench.js.map