"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/platform/update/common/update.config.contribution");
const electron_1 = require("electron");
const objects_1 = require("vs/base/common/objects");
const platform = require("vs/base/common/platform");
const product_1 = require("vs/platform/product/common/product");
const argvHelper_1 = require("vs/platform/environment/node/argvHelper");
const argv_1 = require("vs/platform/environment/node/argv");
const pfs_1 = require("vs/base/node/pfs");
const paths_1 = require("vs/code/node/paths");
const lifecycleMainService_1 = require("vs/platform/lifecycle/electron-main/lifecycleMainService");
const ipc_net_1 = require("vs/base/parts/ipc/node/ipc.net");
const launchMainService_1 = require("vs/platform/launch/electron-main/launchMainService");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const descriptors_1 = require("vs/platform/instantiation/common/descriptors");
const log_1 = require("vs/platform/log/common/log");
const stateService_1 = require("vs/platform/state/node/stateService");
const state_1 = require("vs/platform/state/common/state");
const environment_1 = require("vs/platform/environment/common/environment");
const environmentService_1 = require("vs/platform/environment/node/environmentService");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const configurationService_1 = require("vs/platform/configuration/node/configurationService");
const request_1 = require("vs/platform/request/common/request");
const requestMainService_1 = require("vs/platform/request/electron-main/requestMainService");
const fs = require("fs");
const app_1 = require("vs/code/electron-main/app");
const nls_1 = require("vs/nls");
const labels_1 = require("vs/base/common/labels");
const spdlogService_1 = require("vs/platform/log/node/spdlogService");
const bufferLog_1 = require("vs/platform/log/common/bufferLog");
const errors_1 = require("vs/base/common/errors");
const themeMainService_1 = require("vs/platform/theme/electron-main/themeMainService");
const functional_1 = require("vs/base/common/functional");
const sign_1 = require("vs/platform/sign/common/sign");
const signService_1 = require("vs/platform/sign/node/signService");
const diagnosticsIpc_1 = require("vs/platform/diagnostics/node/diagnosticsIpc");
class ExpectedError extends Error {
    constructor() {
        super(...arguments);
        this.isExpected = true;
    }
}
class CodeMain {
    main() {
        // Set the error handler early enough so that we are not getting the
        // default electron error dialog popping up
        errors_1.setUnexpectedErrorHandler(err => console.error(err));
        // Parse arguments
        let args;
        try {
            args = argvHelper_1.parseMainProcessArgv(process.argv);
            args = paths_1.validatePaths(args);
        }
        catch (err) {
            console.error(err.message);
            electron_1.app.exit(1);
            return;
        }
        // If we are started with --wait create a random temporary file
        // and pass it over to the starting instance. We can use this file
        // to wait for it to be deleted to monitor that the edited file
        // is closed and then exit the waiting process.
        //
        // Note: we are not doing this if the wait marker has been already
        // added as argument. This can happen if Code was started from CLI.
        if (args.wait && !args.waitMarkerFilePath) {
            const waitMarkerFilePath = argv_1.createWaitMarkerFile(args.verbose);
            if (waitMarkerFilePath) {
                argv_1.addArg(process.argv, '--waitMarkerFilePath', waitMarkerFilePath);
                args.waitMarkerFilePath = waitMarkerFilePath;
            }
        }
        // Launch
        this.startup(args);
    }
    startup(args) {
        return __awaiter(this, void 0, void 0, function* () {
            // We need to buffer the spdlog logs until we are sure
            // we are the only instance running, otherwise we'll have concurrent
            // log file access on Windows (https://github.com/Microsoft/vscode/issues/41218)
            const bufferLogService = new bufferLog_1.BufferLogService();
            const [instantiationService, instanceEnvironment] = this.createServices(args, bufferLogService);
            try {
                // Init services
                yield instantiationService.invokeFunction((accessor) => __awaiter(this, void 0, void 0, function* () {
                    const environmentService = accessor.get(environment_1.IEnvironmentService);
                    const configurationService = accessor.get(configuration_1.IConfigurationService);
                    const stateService = accessor.get(state_1.IStateService);
                    try {
                        yield this.initServices(environmentService, configurationService, stateService);
                    }
                    catch (error) {
                        // Show a dialog for errors that can be resolved by the user
                        this.handleStartupDataDirError(environmentService, error);
                        throw error;
                    }
                }));
                // Startup
                yield instantiationService.invokeFunction((accessor) => __awaiter(this, void 0, void 0, function* () {
                    const environmentService = accessor.get(environment_1.IEnvironmentService);
                    const logService = accessor.get(log_1.ILogService);
                    const lifecycleMainService = accessor.get(lifecycleMainService_1.ILifecycleMainService);
                    const configurationService = accessor.get(configuration_1.IConfigurationService);
                    const mainIpcServer = yield this.doStartup(logService, environmentService, lifecycleMainService, instantiationService, true);
                    bufferLogService.logger = new spdlogService_1.SpdLogService('main', environmentService.logsPath, bufferLogService.getLevel());
                    functional_1.once(lifecycleMainService.onWillShutdown)(() => configurationService.dispose());
                    return instantiationService.createInstance(app_1.CodeApplication, mainIpcServer, instanceEnvironment).startup();
                }));
            }
            catch (error) {
                instantiationService.invokeFunction(this.quit, error);
            }
        });
    }
    createServices(args, bufferLogService) {
        const services = new serviceCollection_1.ServiceCollection();
        const environmentService = new environmentService_1.EnvironmentService(args, process.execPath);
        const instanceEnvironment = this.patchEnvironment(environmentService); // Patch `process.env` with the instance's environment
        services.set(environment_1.IEnvironmentService, environmentService);
        const logService = new log_1.MultiplexLogService([new log_1.ConsoleLogMainService(log_1.getLogLevel(environmentService)), bufferLogService]);
        process.once('exit', () => logService.dispose());
        services.set(log_1.ILogService, logService);
        services.set(configuration_1.IConfigurationService, new configurationService_1.ConfigurationService(environmentService.settingsResource));
        services.set(lifecycleMainService_1.ILifecycleMainService, new descriptors_1.SyncDescriptor(lifecycleMainService_1.LifecycleMainService));
        services.set(state_1.IStateService, new descriptors_1.SyncDescriptor(stateService_1.StateService));
        services.set(request_1.IRequestService, new descriptors_1.SyncDescriptor(requestMainService_1.RequestMainService));
        services.set(themeMainService_1.IThemeMainService, new descriptors_1.SyncDescriptor(themeMainService_1.ThemeMainService));
        services.set(sign_1.ISignService, new descriptors_1.SyncDescriptor(signService_1.SignService));
        return [new instantiationService_1.InstantiationService(services, true), instanceEnvironment];
    }
    initServices(environmentService, configurationService, stateService) {
        // Environment service (paths)
        const environmentServiceInitialization = Promise.all([
            environmentService.extensionsPath,
            environmentService.nodeCachedDataDir,
            environmentService.logsPath,
            environmentService.globalStorageHome,
            environmentService.workspaceStorageHome,
            environmentService.backupHome.fsPath
        ].map((path) => path ? pfs_1.mkdirp(path) : undefined));
        // Configuration service
        const configurationServiceInitialization = configurationService.initialize();
        // State service
        const stateServiceInitialization = stateService.init();
        return Promise.all([environmentServiceInitialization, configurationServiceInitialization, stateServiceInitialization]);
    }
    patchEnvironment(environmentService) {
        const instanceEnvironment = {
            VSCODE_IPC_HOOK: environmentService.mainIPCHandle,
            VSCODE_NLS_CONFIG: process.env['VSCODE_NLS_CONFIG'],
            VSCODE_LOGS: process.env['VSCODE_LOGS']
        };
        if (process.env['VSCODE_PORTABLE']) {
            instanceEnvironment['VSCODE_PORTABLE'] = process.env['VSCODE_PORTABLE'];
        }
        objects_1.assign(process.env, instanceEnvironment);
        return instanceEnvironment;
    }
    doStartup(logService, environmentService, lifecycleMainService, instantiationService, retry) {
        return __awaiter(this, void 0, void 0, function* () {
            // Try to setup a server for running. If that succeeds it means
            // we are the first instance to startup. Otherwise it is likely
            // that another instance is already running.
            let server;
            try {
                server = yield ipc_net_1.serve(environmentService.mainIPCHandle);
                functional_1.once(lifecycleMainService.onWillShutdown)(() => server.dispose());
            }
            catch (error) {
                // Handle unexpected errors (the only expected error is EADDRINUSE that
                // indicates a second instance of Code is running)
                if (error.code !== 'EADDRINUSE') {
                    // Show a dialog for errors that can be resolved by the user
                    this.handleStartupDataDirError(environmentService, error);
                    // Any other runtime error is just printed to the console
                    throw error;
                }
                // Since we are the second instance, we do not want to show the dock
                if (platform.isMacintosh) {
                    electron_1.app.dock.hide();
                }
                // there's a running instance, let's connect to it
                let client;
                try {
                    client = yield ipc_net_1.connect(environmentService.mainIPCHandle, 'main');
                }
                catch (error) {
                    // Handle unexpected connection errors by showing a dialog to the user
                    if (!retry || platform.isWindows || error.code !== 'ECONNREFUSED') {
                        if (error.code === 'EPERM') {
                            this.showStartupWarningDialog(nls_1.localize('secondInstanceAdmin', "A second instance of {0} is already running as administrator.", product_1.default.nameShort), nls_1.localize('secondInstanceAdminDetail', "Please close the other instance and try again."));
                        }
                        throw error;
                    }
                    // it happens on Linux and OS X that the pipe is left behind
                    // let's delete it, since we can't connect to it and then
                    // retry the whole thing
                    try {
                        fs.unlinkSync(environmentService.mainIPCHandle);
                    }
                    catch (error) {
                        logService.warn('Could not delete obsolete instance handle', error);
                        throw error;
                    }
                    return this.doStartup(logService, environmentService, lifecycleMainService, instantiationService, false);
                }
                // Tests from CLI require to be the only instance currently
                if (environmentService.extensionTestsLocationURI && !environmentService.debugExtensionHost.break) {
                    const msg = 'Running extension tests from the command line is currently only supported if no other instance of Code is running.';
                    logService.error(msg);
                    client.dispose();
                    throw new Error(msg);
                }
                // Show a warning dialog after some timeout if it takes long to talk to the other instance
                // Skip this if we are running with --wait where it is expected that we wait for a while.
                // Also skip when gathering diagnostics (--status) which can take a longer time.
                let startupWarningDialogHandle = undefined;
                if (!environmentService.wait && !environmentService.status) {
                    startupWarningDialogHandle = setTimeout(() => {
                        this.showStartupWarningDialog(nls_1.localize('secondInstanceNoResponse', "Another instance of {0} is running but not responding", product_1.default.nameShort), nls_1.localize('secondInstanceNoResponseDetail', "Please close all other instances and try again."));
                    }, 10000);
                }
                const channel = client.getChannel('launch');
                const launchClient = new launchMainService_1.LaunchChannelClient(channel);
                // Process Info
                if (environmentService.args.status) {
                    return instantiationService.invokeFunction((accessor) => __awaiter(this, void 0, void 0, function* () {
                        // Create a diagnostic service connected to the existing shared process
                        const sharedProcessClient = yield ipc_net_1.connect(environmentService.sharedIPCHandle, 'main');
                        const diagnosticsChannel = sharedProcessClient.getChannel('diagnostics');
                        const diagnosticsService = new diagnosticsIpc_1.DiagnosticsService(diagnosticsChannel);
                        const mainProcessInfo = yield launchClient.getMainProcessInfo();
                        const remoteDiagnostics = yield launchClient.getRemoteDiagnostics({ includeProcesses: true, includeWorkspaceMetadata: true });
                        const diagnostics = yield diagnosticsService.getDiagnostics(mainProcessInfo, remoteDiagnostics);
                        console.log(diagnostics);
                        throw new ExpectedError();
                    }));
                }
                // Windows: allow to set foreground
                if (platform.isWindows) {
                    yield this.windowsAllowSetForegroundWindow(launchClient, logService);
                }
                // Send environment over...
                logService.trace('Sending env to running instance...');
                yield launchClient.start(environmentService.args, process.env);
                // Cleanup
                yield client.dispose();
                // Now that we started, make sure the warning dialog is prevented
                if (startupWarningDialogHandle) {
                    clearTimeout(startupWarningDialogHandle);
                }
                throw new ExpectedError('Sent env to running instance. Terminating...');
            }
            // Print --status usage info
            if (environmentService.args.status) {
                logService.warn('Warning: The --status argument can only be used if Code is already running. Please run it again after Code has started.');
                throw new ExpectedError('Terminating...');
            }
            // dock might be hidden at this case due to a retry
            if (platform.isMacintosh) {
                electron_1.app.dock.show();
            }
            // Set the VSCODE_PID variable here when we are sure we are the first
            // instance to startup. Otherwise we would wrongly overwrite the PID
            process.env['VSCODE_PID'] = String(process.pid);
            return server;
        });
    }
    handleStartupDataDirError(environmentService, error) {
        if (error.code === 'EACCES' || error.code === 'EPERM') {
            const directories = [environmentService.userDataPath];
            if (environmentService.extensionsPath) {
                directories.push(environmentService.extensionsPath);
            }
            if (environmentService_1.xdgRuntimeDir) {
                directories.push(environmentService_1.xdgRuntimeDir);
            }
            this.showStartupWarningDialog(nls_1.localize('startupDataDirError', "Unable to write program user data."), nls_1.localize('startupUserDataAndExtensionsDirErrorDetail', "Please make sure the following directories are writeable:\n\n{0}", directories.join('\n')));
        }
    }
    showStartupWarningDialog(message, detail) {
        electron_1.dialog.showMessageBox({
            title: product_1.default.nameLong,
            type: 'warning',
            buttons: [labels_1.mnemonicButtonLabel(nls_1.localize({ key: 'close', comment: ['&& denotes a mnemonic'] }, "&&Close"))],
            message,
            detail,
            noLink: true
        });
    }
    windowsAllowSetForegroundWindow(client, logService) {
        return __awaiter(this, void 0, void 0, function* () {
            if (platform.isWindows) {
                const processId = yield client.getMainProcessId();
                logService.trace('Sending some foreground love to the running instance:', processId);
                try {
                    (yield Promise.resolve().then(() => require('windows-foreground-love'))).allowSetForegroundWindow(processId);
                }
                catch (error) {
                    logService.error(error);
                }
            }
        });
    }
    quit(accessor, reason) {
        const logService = accessor.get(log_1.ILogService);
        const lifecycleMainService = accessor.get(lifecycleMainService_1.ILifecycleMainService);
        let exitCode = 0;
        if (reason) {
            if (reason.isExpected) {
                if (reason.message) {
                    logService.trace(reason.message);
                }
            }
            else {
                exitCode = 1; // signal error to the outside
                if (reason.stack) {
                    logService.error(reason.stack);
                }
                else {
                    logService.error(`Startup error: ${reason.toString()}`);
                }
            }
        }
        lifecycleMainService.kill(exitCode);
    }
}
exports.CodeMain = CodeMain;
// Main Startup
// const code = new CodeMain();
// code.main();
//# sourceMappingURL=main.js.map