"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dom_1 = require("vs/base/browser/dom");
const actions_1 = require("vs/base/common/actions");
const errors_1 = require("vs/base/common/errors");
const position_1 = require("vs/editor/common/core/position");
const lifecycle_1 = require("vs/base/common/lifecycle");
class CodeActionWidget extends lifecycle_1.Disposable {
    constructor(_editor, _contextMenuService, _delegate) {
        super();
        this._editor = _editor;
        this._contextMenuService = _contextMenuService;
        this._delegate = _delegate;
        this._showingActions = this._register(new lifecycle_1.MutableDisposable());
        this._visible = false;
    }
    show(codeActions, at) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!codeActions.actions.length) {
                this._visible = false;
                return;
            }
            if (!this._editor.getDomNode()) {
                // cancel when editor went off-dom
                this._visible = false;
                return Promise.reject(errors_1.canceled());
            }
            this._visible = true;
            const actions = codeActions.actions.map(action => this.codeActionToAction(action));
            this._showingActions.value = codeActions;
            this._contextMenuService.showContextMenu({
                getAnchor: () => {
                    if (position_1.Position.isIPosition(at)) {
                        at = this._toCoords(at);
                    }
                    return at || { x: 0, y: 0 };
                },
                getActions: () => actions,
                onHide: () => {
                    this._visible = false;
                    this._editor.focus();
                },
                autoSelectFirstItem: true
            });
        });
    }
    codeActionToAction(action) {
        const id = action.command ? action.command.id : action.title;
        const title = action.title;
        return new actions_1.Action(id, title, undefined, true, () => this._delegate.onSelectCodeAction(action));
    }
    get isVisible() {
        return this._visible;
    }
    _toCoords(position) {
        if (!this._editor.hasModel()) {
            return { x: 0, y: 0 };
        }
        this._editor.revealPosition(position, 1 /* Immediate */);
        this._editor.render();
        // Translate to absolute editor position
        const cursorCoords = this._editor.getScrolledVisiblePosition(position);
        const editorCoords = dom_1.getDomNodePagePosition(this._editor.getDomNode());
        const x = editorCoords.left + cursorCoords.left;
        const y = editorCoords.top + cursorCoords.top + cursorCoords.height;
        return { x, y };
    }
}
exports.CodeActionWidget = CodeActionWidget;
//# sourceMappingURL=codeActionWidget.js.map