"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const arrays_1 = require("vs/base/common/arrays");
const cancellation_1 = require("vs/base/common/cancellation");
const errors_1 = require("vs/base/common/errors");
const uri_1 = require("vs/base/common/uri");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const modes_1 = require("vs/editor/common/modes");
const modelService_1 = require("vs/editor/common/services/modelService");
const lifecycle_1 = require("vs/base/common/lifecycle");
class CodeLensModel {
    constructor() {
        this.lenses = [];
        this._disposables = new lifecycle_1.DisposableStore();
    }
    dispose() {
        this._disposables.dispose();
    }
    add(list, provider) {
        this._disposables.add(list);
        for (const symbol of list.lenses) {
            this.lenses.push({ symbol, provider });
        }
    }
}
exports.CodeLensModel = CodeLensModel;
function getCodeLensData(model, token) {
    const provider = modes_1.CodeLensProviderRegistry.ordered(model);
    const providerRanks = new Map();
    const result = new CodeLensModel();
    const promises = provider.map((provider, i) => {
        providerRanks.set(provider, i);
        return Promise.resolve(provider.provideCodeLenses(model, token))
            .then(list => list && result.add(list, provider))
            .catch(errors_1.onUnexpectedExternalError);
    });
    return Promise.all(promises).then(() => {
        result.lenses = arrays_1.mergeSort(result.lenses, (a, b) => {
            // sort by lineNumber, provider-rank, and column
            if (a.symbol.range.startLineNumber < b.symbol.range.startLineNumber) {
                return -1;
            }
            else if (a.symbol.range.startLineNumber > b.symbol.range.startLineNumber) {
                return 1;
            }
            else if (providerRanks.get(a.provider) < providerRanks.get(b.provider)) {
                return -1;
            }
            else if (providerRanks.get(a.provider) > providerRanks.get(b.provider)) {
                return 1;
            }
            else if (a.symbol.range.startColumn < b.symbol.range.startColumn) {
                return -1;
            }
            else if (a.symbol.range.startColumn > b.symbol.range.startColumn) {
                return 1;
            }
            else {
                return 0;
            }
        });
        return result;
    });
}
exports.getCodeLensData = getCodeLensData;
editorExtensions_1.registerLanguageCommand('_executeCodeLensProvider', function (accessor, args) {
    let { resource, itemResolveCount } = args;
    if (!(resource instanceof uri_1.URI)) {
        throw errors_1.illegalArgument();
    }
    const model = accessor.get(modelService_1.IModelService).getModel(resource);
    if (!model) {
        throw errors_1.illegalArgument();
    }
    const result = [];
    const disposables = new lifecycle_1.DisposableStore();
    return getCodeLensData(model, cancellation_1.CancellationToken.None).then(value => {
        disposables.add(value);
        let resolve = [];
        for (const item of value.lenses) {
            if (typeof itemResolveCount === 'undefined' || Boolean(item.symbol.command)) {
                result.push(item.symbol);
            }
            else if (itemResolveCount-- > 0 && item.provider.resolveCodeLens) {
                resolve.push(Promise.resolve(item.provider.resolveCodeLens(model, item.symbol, cancellation_1.CancellationToken.None)).then(symbol => result.push(symbol || item.symbol)));
            }
        }
        return Promise.all(resolve);
    }).then(() => {
        return result;
    }).finally(() => {
        // make sure to return results, then (on next tick)
        // dispose the results
        setTimeout(() => disposables.dispose(), 100);
    });
});
//# sourceMappingURL=codelens.js.map