"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const bulkEditService_1 = require("vs/editor/browser/services/bulkEditService");
const codeEditorService_1 = require("vs/editor/browser/services/codeEditorService");
const editorWorkerService_1 = require("vs/editor/common/services/editorWorkerService");
const editorWorkerServiceImpl_1 = require("vs/editor/common/services/editorWorkerServiceImpl");
const modeService_1 = require("vs/editor/common/services/modeService");
const modeServiceImpl_1 = require("vs/editor/common/services/modeServiceImpl");
const modelService_1 = require("vs/editor/common/services/modelService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const resourceConfiguration_1 = require("vs/editor/common/services/resourceConfiguration");
const simpleServices_1 = require("vs/editor/standalone/browser/simpleServices");
const standaloneCodeServiceImpl_1 = require("vs/editor/standalone/browser/standaloneCodeServiceImpl");
const standaloneThemeServiceImpl_1 = require("vs/editor/standalone/browser/standaloneThemeServiceImpl");
const standaloneThemeService_1 = require("vs/editor/standalone/common/standaloneThemeService");
const actions_1 = require("vs/platform/actions/common/actions");
const commands_1 = require("vs/platform/commands/common/commands");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const contextKeyService_1 = require("vs/platform/contextkey/browser/contextKeyService");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const contextMenuService_1 = require("vs/platform/contextview/browser/contextMenuService");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const contextViewService_1 = require("vs/platform/contextview/browser/contextViewService");
const dialogs_1 = require("vs/platform/dialogs/common/dialogs");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const label_1 = require("vs/platform/label/common/label");
const listService_1 = require("vs/platform/list/browser/listService");
const log_1 = require("vs/platform/log/common/log");
const markerService_1 = require("vs/platform/markers/common/markerService");
const markers_1 = require("vs/platform/markers/common/markers");
const notification_1 = require("vs/platform/notification/common/notification");
const progress_1 = require("vs/platform/progress/common/progress");
const storage_1 = require("vs/platform/storage/common/storage");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const themeService_1 = require("vs/platform/theme/common/themeService");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const menuService_1 = require("vs/platform/actions/common/menuService");
const markersDecorationService_1 = require("vs/editor/common/services/markersDecorationService");
const markerDecorationsServiceImpl_1 = require("vs/editor/common/services/markerDecorationsServiceImpl");
const accessibility_1 = require("vs/platform/accessibility/common/accessibility");
const accessibilityService_1 = require("vs/platform/accessibility/common/accessibilityService");
const layoutService_1 = require("vs/platform/layout/browser/layoutService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
var StaticServices;
(function (StaticServices) {
    const _serviceCollection = new serviceCollection_1.ServiceCollection();
    class LazyStaticService {
        constructor(serviceId, factory) {
            this._serviceId = serviceId;
            this._factory = factory;
            this._value = null;
        }
        get id() { return this._serviceId; }
        get(overrides) {
            if (!this._value) {
                if (overrides) {
                    this._value = overrides[this._serviceId.toString()];
                }
                if (!this._value) {
                    this._value = this._factory(overrides);
                }
                if (!this._value) {
                    throw new Error('Service ' + this._serviceId + ' is missing!');
                }
                _serviceCollection.set(this._serviceId, this._value);
            }
            return this._value;
        }
    }
    StaticServices.LazyStaticService = LazyStaticService;
    let _all = [];
    function define(serviceId, factory) {
        let r = new LazyStaticService(serviceId, factory);
        _all.push(r);
        return r;
    }
    function init(overrides) {
        // Create a fresh service collection
        let result = new serviceCollection_1.ServiceCollection();
        // make sure to add all services that use `registerSingleton`
        for (const [id, descriptor] of extensions_1.getSingletonServiceDescriptors()) {
            result.set(id, descriptor);
        }
        // Initialize the service collection with the overrides
        for (let serviceId in overrides) {
            if (overrides.hasOwnProperty(serviceId)) {
                result.set(instantiation_1.createDecorator(serviceId), overrides[serviceId]);
            }
        }
        // Make sure the same static services are present in all service collections
        _all.forEach(service => result.set(service.id, service.get(overrides)));
        // Ensure the collection gets the correct instantiation service
        let instantiationService = new instantiationService_1.InstantiationService(result, true);
        result.set(instantiation_1.IInstantiationService, instantiationService);
        return [result, instantiationService];
    }
    StaticServices.init = init;
    StaticServices.instantiationService = define(instantiation_1.IInstantiationService, () => new instantiationService_1.InstantiationService(_serviceCollection, true));
    const configurationServiceImpl = new simpleServices_1.SimpleConfigurationService();
    StaticServices.configurationService = define(configuration_1.IConfigurationService, () => configurationServiceImpl);
    StaticServices.resourceConfigurationService = define(resourceConfiguration_1.ITextResourceConfigurationService, () => new simpleServices_1.SimpleResourceConfigurationService(configurationServiceImpl));
    StaticServices.resourcePropertiesService = define(resourceConfiguration_1.ITextResourcePropertiesService, () => new simpleServices_1.SimpleResourcePropertiesService(configurationServiceImpl));
    StaticServices.contextService = define(workspace_1.IWorkspaceContextService, () => new simpleServices_1.SimpleWorkspaceContextService());
    StaticServices.labelService = define(label_1.ILabelService, () => new simpleServices_1.SimpleUriLabelService());
    StaticServices.telemetryService = define(telemetry_1.ITelemetryService, () => new simpleServices_1.StandaloneTelemetryService());
    StaticServices.dialogService = define(dialogs_1.IDialogService, () => new simpleServices_1.SimpleDialogService());
    StaticServices.notificationService = define(notification_1.INotificationService, () => new simpleServices_1.SimpleNotificationService());
    StaticServices.markerService = define(markers_1.IMarkerService, () => new markerService_1.MarkerService());
    StaticServices.modeService = define(modeService_1.IModeService, (o) => new modeServiceImpl_1.ModeServiceImpl());
    StaticServices.modelService = define(modelService_1.IModelService, (o) => new modelServiceImpl_1.ModelServiceImpl(StaticServices.configurationService.get(o), StaticServices.resourcePropertiesService.get(o)));
    StaticServices.markerDecorationsService = define(markersDecorationService_1.IMarkerDecorationsService, (o) => new markerDecorationsServiceImpl_1.MarkerDecorationsService(StaticServices.modelService.get(o), StaticServices.markerService.get(o)));
    StaticServices.standaloneThemeService = define(standaloneThemeService_1.IStandaloneThemeService, () => new standaloneThemeServiceImpl_1.StandaloneThemeServiceImpl());
    StaticServices.codeEditorService = define(codeEditorService_1.ICodeEditorService, (o) => new standaloneCodeServiceImpl_1.StandaloneCodeEditorServiceImpl(StaticServices.standaloneThemeService.get(o)));
    StaticServices.editorProgressService = define(progress_1.IEditorProgressService, () => new simpleServices_1.SimpleEditorProgressService());
    StaticServices.storageService = define(storage_1.IStorageService, () => new storage_1.InMemoryStorageService());
    StaticServices.logService = define(log_1.ILogService, () => new log_1.NullLogService());
    StaticServices.editorWorkerService = define(editorWorkerService_1.IEditorWorkerService, (o) => new editorWorkerServiceImpl_1.EditorWorkerServiceImpl(StaticServices.modelService.get(o), StaticServices.resourceConfigurationService.get(o), StaticServices.logService.get(o)));
})(StaticServices = exports.StaticServices || (exports.StaticServices = {}));
class DynamicStandaloneServices extends lifecycle_1.Disposable {
    constructor(domElement, overrides) {
        super();
        const [_serviceCollection, _instantiationService] = StaticServices.init(overrides);
        this._serviceCollection = _serviceCollection;
        this._instantiationService = _instantiationService;
        const configurationService = this.get(configuration_1.IConfigurationService);
        const notificationService = this.get(notification_1.INotificationService);
        const telemetryService = this.get(telemetry_1.ITelemetryService);
        const themeService = this.get(themeService_1.IThemeService);
        let ensure = (serviceId, factory) => {
            let value = null;
            if (overrides) {
                value = overrides[serviceId.toString()];
            }
            if (!value) {
                value = factory();
            }
            this._serviceCollection.set(serviceId, value);
            return value;
        };
        let contextKeyService = ensure(contextkey_1.IContextKeyService, () => this._register(new contextKeyService_1.ContextKeyService(configurationService)));
        ensure(accessibility_1.IAccessibilityService, () => new accessibilityService_1.BrowserAccessibilityService(contextKeyService, configurationService));
        ensure(listService_1.IListService, () => new listService_1.ListService(contextKeyService));
        let commandService = ensure(commands_1.ICommandService, () => new simpleServices_1.StandaloneCommandService(this._instantiationService));
        let keybindingService = ensure(keybinding_1.IKeybindingService, () => this._register(new simpleServices_1.StandaloneKeybindingService(contextKeyService, commandService, telemetryService, notificationService, domElement)));
        let layoutService = ensure(layoutService_1.ILayoutService, () => new simpleServices_1.SimpleLayoutService(domElement));
        let contextViewService = ensure(contextView_1.IContextViewService, () => this._register(new contextViewService_1.ContextViewService(layoutService)));
        ensure(contextView_1.IContextMenuService, () => {
            const contextMenuService = new contextMenuService_1.ContextMenuService(telemetryService, notificationService, contextViewService, keybindingService, themeService);
            contextMenuService.configure({ blockMouse: false }); // we do not want that in the standalone editor
            return this._register(contextMenuService);
        });
        ensure(actions_1.IMenuService, () => new menuService_1.MenuService(commandService));
        ensure(bulkEditService_1.IBulkEditService, () => new simpleServices_1.SimpleBulkEditService(StaticServices.modelService.get(modelService_1.IModelService)));
    }
    get(serviceId) {
        let r = this._serviceCollection.get(serviceId);
        if (!r) {
            throw new Error('Missing service ' + serviceId);
        }
        return r;
    }
    set(serviceId, instance) {
        this._serviceCollection.set(serviceId, instance);
    }
    has(serviceId) {
        return this._serviceCollection.has(serviceId);
    }
}
exports.DynamicStandaloneServices = DynamicStandaloneServices;
//# sourceMappingURL=standaloneServices.js.map