"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codeEditorService_1 = require("vs/editor/browser/services/codeEditorService");
const codeEditorWidget_1 = require("vs/editor/browser/widget/codeEditorWidget");
const textModel_1 = require("vs/editor/common/model/textModel");
const editorTestServices_1 = require("vs/editor/test/browser/editorTestServices");
const testConfiguration_1 = require("vs/editor/test/common/mocks/testConfiguration");
const commands_1 = require("vs/platform/commands/common/commands");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const mockKeybindingService_1 = require("vs/platform/keybinding/test/common/mockKeybindingService");
const notification_1 = require("vs/platform/notification/common/notification");
const testNotificationService_1 = require("vs/platform/notification/test/common/testNotificationService");
const themeService_1 = require("vs/platform/theme/common/themeService");
const testThemeService_1 = require("vs/platform/theme/test/common/testThemeService");
class TestCodeEditor extends codeEditorWidget_1.CodeEditorWidget {
    //#region testing overrides
    _createConfiguration(options) {
        return new testConfiguration_1.TestConfiguration(options);
    }
    _createView(viewModel, cursor) {
        // Never create a view
        return [null, false];
    }
    //#endregion
    //#region Testing utils
    getCursor() {
        return this._modelData ? this._modelData.cursor : undefined;
    }
    registerAndInstantiateContribution(ctor) {
        let r = this._instantiationService.createInstance(ctor, this);
        this._contributions[r.getId()] = r;
        return r;
    }
    dispose() {
        super.dispose();
        if (this._modelData) {
            this._modelData.model.dispose();
        }
    }
}
exports.TestCodeEditor = TestCodeEditor;
class TestEditorDomElement {
    constructor() {
        this.parentElement = null;
    }
    setAttribute(attr, value) { }
    removeAttribute(attr) { }
    hasAttribute(attr) { return false; }
    getAttribute(attr) { return undefined; }
    addEventListener(event) { }
    removeEventListener(event) { }
}
function withTestCodeEditor(text, options, callback) {
    // create a model if necessary and remember it in order to dispose it.
    if (!options.model) {
        if (typeof text === 'string') {
            options.model = textModel_1.TextModel.createFromString(text);
        }
        else if (text) {
            options.model = textModel_1.TextModel.createFromString(text.join('\n'));
        }
    }
    let editor = createTestCodeEditor(options);
    callback(editor, editor.getCursor());
    editor.dispose();
}
exports.withTestCodeEditor = withTestCodeEditor;
function createTestCodeEditor(options) {
    const model = options.model;
    delete options.model;
    const services = options.serviceCollection || new serviceCollection_1.ServiceCollection();
    delete options.serviceCollection;
    const instantiationService = new instantiationService_1.InstantiationService(services);
    if (!services.has(codeEditorService_1.ICodeEditorService)) {
        services.set(codeEditorService_1.ICodeEditorService, new editorTestServices_1.TestCodeEditorService());
    }
    if (!services.has(contextkey_1.IContextKeyService)) {
        services.set(contextkey_1.IContextKeyService, new mockKeybindingService_1.MockContextKeyService());
    }
    if (!services.has(notification_1.INotificationService)) {
        services.set(notification_1.INotificationService, new testNotificationService_1.TestNotificationService());
    }
    if (!services.has(commands_1.ICommandService)) {
        services.set(commands_1.ICommandService, new editorTestServices_1.TestCommandService(instantiationService));
    }
    if (!services.has(themeService_1.IThemeService)) {
        services.set(themeService_1.IThemeService, new testThemeService_1.TestThemeService());
    }
    const codeEditorWidgetOptions = {
        contributions: []
    };
    const editor = instantiationService.createInstance(TestCodeEditor, new TestEditorDomElement(), options, codeEditorWidgetOptions);
    editor.setModel(model);
    return editor;
}
exports.createTestCodeEditor = createTestCodeEditor;
//# sourceMappingURL=testCodeEditor.js.map