"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
const languageConfiguration_1 = require("vs/editor/common/modes/languageConfiguration");
const onEnter_1 = require("vs/editor/common/modes/supports/onEnter");
const javascriptOnEnterRules_1 = require("vs/editor/test/common/modes/supports/javascriptOnEnterRules");
suite('OnEnter', () => {
    test('uses brackets', () => {
        let brackets = [
            ['(', ')'],
            ['begin', 'end']
        ];
        let support = new onEnter_1.OnEnterSupport({
            brackets: brackets
        });
        let testIndentAction = (beforeText, afterText, expected) => {
            let actual = support.onEnter('', beforeText, afterText);
            if (expected === languageConfiguration_1.IndentAction.None) {
                assert.equal(actual, null);
            }
            else {
                assert.equal(actual.indentAction, expected);
            }
        };
        testIndentAction('a', '', languageConfiguration_1.IndentAction.None);
        testIndentAction('', 'b', languageConfiguration_1.IndentAction.None);
        testIndentAction('(', 'b', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('a', ')', languageConfiguration_1.IndentAction.None);
        testIndentAction('begin', 'ending', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('abegin', 'end', languageConfiguration_1.IndentAction.None);
        testIndentAction('begin', ')', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('begin', 'end', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction('begin ', ' end', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction(' begin', 'end//as', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction('(', ')', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction('( ', ')', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction('a(', ')b', languageConfiguration_1.IndentAction.IndentOutdent);
        testIndentAction('(', '', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('(', 'foo', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('begin', 'foo', languageConfiguration_1.IndentAction.Indent);
        testIndentAction('begin', '', languageConfiguration_1.IndentAction.Indent);
    });
    test('uses regExpRules', () => {
        let support = new onEnter_1.OnEnterSupport({
            regExpRules: javascriptOnEnterRules_1.javascriptOnEnterRules
        });
        let testIndentAction = (oneLineAboveText, beforeText, afterText, expectedIndentAction, expectedAppendText, removeText = 0) => {
            let actual = support.onEnter(oneLineAboveText, beforeText, afterText);
            if (expectedIndentAction === null) {
                assert.equal(actual, null, 'isNull:' + beforeText);
            }
            else {
                assert.equal(actual !== null, true, 'isNotNull:' + beforeText);
                assert.equal(actual.indentAction, expectedIndentAction, 'indentAction:' + beforeText);
                if (expectedAppendText !== null) {
                    assert.equal(actual.appendText, expectedAppendText, 'appendText:' + beforeText);
                }
                if (removeText !== 0) {
                    assert.equal(actual.removeText, removeText, 'removeText:' + beforeText);
                }
            }
        };
        testIndentAction('', '\t/**', ' */', languageConfiguration_1.IndentAction.IndentOutdent, ' * ');
        testIndentAction('', '\t/**', '', languageConfiguration_1.IndentAction.None, ' * ');
        testIndentAction('', '\t/** * / * / * /', '', languageConfiguration_1.IndentAction.None, ' * ');
        testIndentAction('', '\t/** /*', '', languageConfiguration_1.IndentAction.None, ' * ');
        testIndentAction('', '/**', '', languageConfiguration_1.IndentAction.None, ' * ');
        testIndentAction('', '\t/**/', '', null, null);
        testIndentAction('', '\t/***/', '', null, null);
        testIndentAction('', '\t/*******/', '', null, null);
        testIndentAction('', '\t/** * * * * */', '', null, null);
        testIndentAction('', '\t/** */', '', null, null);
        testIndentAction('', '\t/** asdfg */', '', null, null);
        testIndentAction('', '\t/* asdfg */', '', null, null);
        testIndentAction('', '\t/* asdfg */', '', null, null);
        testIndentAction('', '\t/** asdfg */', '', null, null);
        testIndentAction('', '*/', '', null, null);
        testIndentAction('', '\t/*', '', null, null);
        testIndentAction('', '\t*', '', null, null);
        testIndentAction('\t/**', '\t *', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('\t * something', '\t *', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('\t *', '\t *', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('', '\t */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '\t * */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '\t * * / * / * / */', '', null, null);
        testIndentAction('\t/**', '\t * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('\t * something', '\t * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('\t *', '\t * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('/**', ' * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' * something', ' * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' *', ' * asdfsfagadfg', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('/**', ' * asdfsfagadfg * * * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' * something', ' * asdfsfagadfg * * * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' *', ' * asdfsfagadfg * * * ', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('/**', ' * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' * something', ' * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' *', ' * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('/**', ' * asdfsfagadfg * / * / * /', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' * something', ' * asdfsfagadfg * / * / * /', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' *', ' * asdfsfagadfg * / * / * /', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('/**', ' * asdfsfagadfg * / * / * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' * something', ' * asdfsfagadfg * / * / * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction(' *', ' * asdfsfagadfg * / * / * /*', '', languageConfiguration_1.IndentAction.None, '* ');
        testIndentAction('', ' */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '\t */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '\t\t */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '   */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '     */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', '\t     */', '', languageConfiguration_1.IndentAction.None, null, 1);
        testIndentAction('', ' *--------------------------------------------------------------------------------------------*/', '', languageConfiguration_1.IndentAction.None, null, 1);
        // issue #43469
        testIndentAction('class A {', '    * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
        testIndentAction('', '    * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
        testIndentAction('    ', '    * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
        testIndentAction('class A {', '  * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
        testIndentAction('', '  * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
        testIndentAction('  ', '  * test() {', '', languageConfiguration_1.IndentAction.Indent, null, 0);
    });
});
//# sourceMappingURL=onEnter.test.js.map