"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const arrays = require("vs/base/common/arrays");
const state_1 = require("vs/platform/state/common/state");
const electron_1 = require("electron");
const log_1 = require("vs/platform/log/common/log");
const labels_1 = require("vs/base/common/labels");
const event_1 = require("vs/base/common/event");
const platform_1 = require("vs/base/common/platform");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const workspacesMainService_1 = require("vs/platform/workspaces/electron-main/workspacesMainService");
const history_1 = require("vs/platform/history/common/history");
const async_1 = require("vs/base/common/async");
const resources_1 = require("vs/base/common/resources");
const uri_1 = require("vs/base/common/uri");
const network_1 = require("vs/base/common/network");
const environment_1 = require("vs/platform/environment/common/environment");
const label_1 = require("vs/platform/label/common/label");
const historyStorage_1 = require("vs/platform/history/common/historyStorage");
const pfs_1 = require("vs/base/node/pfs");
const lifecycleMainService_1 = require("vs/platform/lifecycle/electron-main/lifecycleMainService");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
exports.IHistoryMainService = instantiation_1.createDecorator('historyMainService');
let HistoryMainService = class HistoryMainService {
    constructor(stateService, logService, workspacesMainService, environmentService, lifecycleMainService) {
        this.stateService = stateService;
        this.logService = logService;
        this.workspacesMainService = workspacesMainService;
        this.environmentService = environmentService;
        this._onRecentlyOpenedChange = new event_1.Emitter();
        this.onRecentlyOpenedChange = this._onRecentlyOpenedChange.event;
        this.macOSRecentDocumentsUpdater = new async_1.ThrottledDelayer(800);
        lifecycleMainService.when(3 /* AfterWindowOpen */).then(() => this.handleWindowsJumpList());
    }
    handleWindowsJumpList() {
        if (!platform_1.isWindows) {
            return; // only on windows
        }
        this.updateWindowsJumpList();
        this.onRecentlyOpenedChange(() => this.updateWindowsJumpList());
    }
    addRecentlyOpened(newlyAdded) {
        const workspaces = [];
        const files = [];
        for (let curr of newlyAdded) {
            // Workspace
            if (history_1.isRecentWorkspace(curr)) {
                if (!this.workspacesMainService.isUntitledWorkspace(curr.workspace) && indexOfWorkspace(workspaces, curr.workspace) === -1) {
                    workspaces.push(curr);
                }
            }
            // Folder
            else if (history_1.isRecentFolder(curr)) {
                if (indexOfFolder(workspaces, curr.folderUri) === -1) {
                    workspaces.push(curr);
                }
            }
            // File
            else {
                const alreadyExistsInHistory = indexOfFile(files, curr.fileUri) >= 0;
                const shouldBeFiltered = curr.fileUri.scheme === network_1.Schemas.file && HistoryMainService.COMMON_FILES_FILTER.indexOf(resources_1.basename(curr.fileUri)) >= 0;
                if (!alreadyExistsInHistory && !shouldBeFiltered) {
                    files.push(curr);
                    // Add to recent documents (Windows only, macOS later)
                    if (platform_1.isWindows && curr.fileUri.scheme === network_1.Schemas.file) {
                        electron_1.app.addRecentDocument(curr.fileUri.fsPath);
                    }
                }
            }
        }
        this.addEntriesFromStorage(workspaces, files);
        if (workspaces.length > HistoryMainService.MAX_TOTAL_RECENT_ENTRIES) {
            workspaces.length = HistoryMainService.MAX_TOTAL_RECENT_ENTRIES;
        }
        if (files.length > HistoryMainService.MAX_TOTAL_RECENT_ENTRIES) {
            files.length = HistoryMainService.MAX_TOTAL_RECENT_ENTRIES;
        }
        this.saveRecentlyOpened({ workspaces, files });
        this._onRecentlyOpenedChange.fire();
        // Schedule update to recent documents on macOS dock
        if (platform_1.isMacintosh) {
            this.macOSRecentDocumentsUpdater.trigger(() => this.updateMacOSRecentDocuments());
        }
    }
    removeFromRecentlyOpened(toRemove) {
        const keep = (recent) => {
            const uri = location(recent);
            for (const r of toRemove) {
                if (resources_1.isEqual(r, uri)) {
                    return false;
                }
            }
            return true;
        };
        const mru = this.getRecentlyOpened();
        const workspaces = mru.workspaces.filter(keep);
        const files = mru.files.filter(keep);
        if (workspaces.length !== mru.workspaces.length || files.length !== mru.files.length) {
            this.saveRecentlyOpened({ files, workspaces });
            this._onRecentlyOpenedChange.fire();
            // Schedule update to recent documents on macOS dock
            if (platform_1.isMacintosh) {
                this.macOSRecentDocumentsUpdater.trigger(() => this.updateMacOSRecentDocuments());
            }
        }
    }
    updateMacOSRecentDocuments() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!platform_1.isMacintosh) {
                return;
            }
            // We clear all documents first to ensure an up-to-date view on the set. Since entries
            // can get deleted on disk, this ensures that the list is always valid
            electron_1.app.clearRecentDocuments();
            const mru = this.getRecentlyOpened();
            // Collect max-N recent workspaces that are known to exist
            const workspaceEntries = [];
            let entries = 0;
            for (let i = 0; i < mru.workspaces.length && entries < HistoryMainService.MAX_MACOS_DOCK_RECENT_WORKSPACES; i++) {
                const loc = location(mru.workspaces[i]);
                if (loc.scheme === network_1.Schemas.file) {
                    const workspacePath = resources_1.originalFSPath(loc);
                    if (yield pfs_1.exists(workspacePath)) {
                        workspaceEntries.push(workspacePath);
                        entries++;
                    }
                }
            }
            // Collect max-N recent files that are known to exist
            const fileEntries = [];
            for (let i = 0; i < mru.files.length && entries < HistoryMainService.MAX_MACOS_DOCK_RECENT_ENTRIES_TOTAL; i++) {
                const loc = location(mru.files[i]);
                if (loc.scheme === network_1.Schemas.file) {
                    const filePath = resources_1.originalFSPath(loc);
                    if (HistoryMainService.COMMON_FILES_FILTER.indexOf(resources_1.basename(loc)) !== -1 || // skip some well known file entries
                        workspaceEntries.indexOf(filePath) !== -1 // prefer a workspace entry over a file entry (e.g. for .code-workspace)
                    ) {
                        continue;
                    }
                    if (yield pfs_1.exists(filePath)) {
                        fileEntries.push(filePath);
                        entries++;
                    }
                }
            }
            // The apple guidelines (https://developer.apple.com/design/human-interface-guidelines/macos/menus/menu-anatomy/)
            // explain that most recent entries should appear close to the interaction by the user (e.g. close to the
            // mouse click). Most native macOS applications that add recent documents to the dock, show the most recent document
            // to the bottom (because the dock menu is not appearing from top to bottom, but from the bottom to the top). As such
            // we fill in the entries in reverse order so that the most recent shows up at the bottom of the menu.
            //
            // On top of that, the maximum number of documents can be configured by the user (defaults to 10). To ensure that
            // we are not failing to show the most recent entries, we start by adding files first (in reverse order of recency)
            // and then add folders (in reverse order of recency). Given that strategy, we can ensure that the most recent
            // N folders are always appearing, even if the limit is low (https://github.com/microsoft/vscode/issues/74788)
            fileEntries.reverse().forEach(fileEntry => electron_1.app.addRecentDocument(fileEntry));
            workspaceEntries.reverse().forEach(workspaceEntry => electron_1.app.addRecentDocument(workspaceEntry));
        });
    }
    clearRecentlyOpened() {
        this.saveRecentlyOpened({ workspaces: [], files: [] });
        electron_1.app.clearRecentDocuments();
        // Event
        this._onRecentlyOpenedChange.fire();
    }
    getRecentlyOpened(currentWorkspace, currentFolder, currentFiles) {
        const workspaces = [];
        const files = [];
        // Add current workspace to beginning if set
        if (currentWorkspace && !this.workspacesMainService.isUntitledWorkspace(currentWorkspace)) {
            workspaces.push({ workspace: currentWorkspace });
        }
        if (currentFolder) {
            workspaces.push({ folderUri: currentFolder });
        }
        // Add currently files to open to the beginning if any
        if (currentFiles) {
            for (let currentFile of currentFiles) {
                const fileUri = currentFile.fileUri;
                if (fileUri && indexOfFile(files, fileUri) === -1) {
                    files.push({ fileUri });
                }
            }
        }
        this.addEntriesFromStorage(workspaces, files);
        return { workspaces, files };
    }
    addEntriesFromStorage(workspaces, files) {
        // Get from storage
        let recents = this.getRecentlyOpenedFromStorage();
        for (let recent of recents.workspaces) {
            let index = history_1.isRecentFolder(recent) ? indexOfFolder(workspaces, recent.folderUri) : indexOfWorkspace(workspaces, recent.workspace);
            if (index >= 0) {
                workspaces[index].label = workspaces[index].label || recent.label;
            }
            else {
                workspaces.push(recent);
            }
        }
        for (let recent of recents.files) {
            let index = indexOfFile(files, recent.fileUri);
            if (index >= 0) {
                files[index].label = files[index].label || recent.label;
            }
            else {
                files.push(recent);
            }
        }
    }
    getRecentlyOpenedFromStorage() {
        const storedRecents = this.stateService.getItem(HistoryMainService.recentlyOpenedStorageKey);
        return historyStorage_1.restoreRecentlyOpened(storedRecents, this.logService);
    }
    saveRecentlyOpened(recent) {
        const serialized = historyStorage_1.toStoreData(recent);
        this.stateService.setItem(HistoryMainService.recentlyOpenedStorageKey, serialized);
    }
    updateWindowsJumpList() {
        if (!platform_1.isWindows) {
            return; // only on windows
        }
        const jumpList = [];
        // Tasks
        jumpList.push({
            type: 'tasks',
            items: [
                {
                    type: 'task',
                    title: nls.localize('newWindow', "New Window"),
                    description: nls.localize('newWindowDesc', "Opens a new window"),
                    program: process.execPath,
                    args: '-n',
                    iconPath: process.execPath,
                    iconIndex: 0
                }
            ]
        });
        // Recent Workspaces
        if (this.getRecentlyOpened().workspaces.length > 0) {
            // The user might have meanwhile removed items from the jump list and we have to respect that
            // so we need to update our list of recent paths with the choice of the user to not add them again
            // Also: Windows will not show our custom category at all if there is any entry which was removed
            // by the user! See https://github.com/Microsoft/vscode/issues/15052
            let toRemove = [];
            for (let item of electron_1.app.getJumpListSettings().removedItems) {
                const args = item.args;
                if (args) {
                    const match = /^--(folder|file)-uri\s+"([^"]+)"$/.exec(args);
                    if (match) {
                        toRemove.push(uri_1.URI.parse(match[2]));
                    }
                }
            }
            this.removeFromRecentlyOpened(toRemove);
            // Add entries
            jumpList.push({
                type: 'custom',
                name: nls.localize('recentFolders', "Recent Workspaces"),
                items: arrays.coalesce(this.getRecentlyOpened().workspaces.slice(0, 7 /* limit number of entries here */).map(recent => {
                    const workspace = history_1.isRecentWorkspace(recent) ? recent.workspace : recent.folderUri;
                    const title = recent.label || label_1.getSimpleWorkspaceLabel(workspace, this.environmentService.untitledWorkspacesHome);
                    let description;
                    let args;
                    if (workspaces_1.isSingleFolderWorkspaceIdentifier(workspace)) {
                        description = nls.localize('folderDesc', "{0} {1}", labels_1.getBaseLabel(workspace), labels_1.getPathLabel(resources_1.dirname(workspace), this.environmentService));
                        args = `--folder-uri "${workspace.toString()}"`;
                    }
                    else {
                        description = nls.localize('workspaceDesc', "{0} {1}", labels_1.getBaseLabel(workspace.configPath), labels_1.getPathLabel(resources_1.dirname(workspace.configPath), this.environmentService));
                        args = `--file-uri "${workspace.configPath.toString()}"`;
                    }
                    return {
                        type: 'task',
                        title,
                        description,
                        program: process.execPath,
                        args,
                        iconPath: 'explorer.exe',
                        iconIndex: 0
                    };
                }))
            });
        }
        // Recent
        jumpList.push({
            type: 'recent' // this enables to show files in the "recent" category
        });
        try {
            electron_1.app.setJumpList(jumpList);
        }
        catch (error) {
            this.logService.warn('#setJumpList', error); // since setJumpList is relatively new API, make sure to guard for errors
        }
    }
};
HistoryMainService.MAX_TOTAL_RECENT_ENTRIES = 100;
HistoryMainService.MAX_MACOS_DOCK_RECENT_WORKSPACES = 7; // prefer more workspaces...
HistoryMainService.MAX_MACOS_DOCK_RECENT_ENTRIES_TOTAL = 10; // ...compared to files
// Exclude some very common files from the dock/taskbar
HistoryMainService.COMMON_FILES_FILTER = [
    'COMMIT_EDITMSG',
    'MERGE_MSG'
];
HistoryMainService.recentlyOpenedStorageKey = 'openedPathsList';
HistoryMainService = __decorate([
    __param(0, state_1.IStateService),
    __param(1, log_1.ILogService),
    __param(2, workspacesMainService_1.IWorkspacesMainService),
    __param(3, environment_1.IEnvironmentService),
    __param(4, lifecycleMainService_1.ILifecycleMainService)
], HistoryMainService);
exports.HistoryMainService = HistoryMainService;
function location(recent) {
    if (history_1.isRecentFolder(recent)) {
        return recent.folderUri;
    }
    if (history_1.isRecentFile(recent)) {
        return recent.fileUri;
    }
    return recent.workspace.configPath;
}
function indexOfWorkspace(arr, workspace) {
    return arrays.firstIndex(arr, w => history_1.isRecentWorkspace(w) && w.workspace.id === workspace.id);
}
function indexOfFolder(arr, folderURI) {
    return arrays.firstIndex(arr, f => history_1.isRecentFolder(f) && resources_1.isEqual(f.folderUri, folderURI));
}
function indexOfFile(arr, fileURI) {
    return arrays.firstIndex(arr, f => resources_1.isEqual(f.fileUri, fileURI));
}
//# sourceMappingURL=historyMainService.js.map