"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("vs/base/common/path");
const fs = require("fs");
const environment_1 = require("vs/platform/environment/common/environment");
const pfs_1 = require("vs/base/node/pfs");
const types_1 = require("vs/base/common/types");
const log_1 = require("vs/platform/log/common/log");
class FileStorage {
    constructor(dbPath, onError) {
        this.dbPath = dbPath;
        this.onError = onError;
        this._database = null;
        this.lastFlushedSerializedDatabase = null;
    }
    get database() {
        if (!this._database) {
            this._database = this.loadSync();
        }
        return this._database;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._database) {
                return; // return if database was already loaded
            }
            const database = yield this.loadAsync();
            if (this._database) {
                return; // return if database was already loaded
            }
            this._database = database;
        });
    }
    loadSync() {
        try {
            this.lastFlushedSerializedDatabase = fs.readFileSync(this.dbPath).toString();
            return JSON.parse(this.lastFlushedSerializedDatabase);
        }
        catch (error) {
            if (error.code !== 'ENOENT') {
                this.onError(error);
            }
            return {};
        }
    }
    loadAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.lastFlushedSerializedDatabase = (yield pfs_1.readFile(this.dbPath)).toString();
                return JSON.parse(this.lastFlushedSerializedDatabase);
            }
            catch (error) {
                if (error.code !== 'ENOENT') {
                    this.onError(error);
                }
                return {};
            }
        });
    }
    getItem(key, defaultValue) {
        const res = this.database[key];
        if (types_1.isUndefinedOrNull(res)) {
            return defaultValue;
        }
        return res;
    }
    setItem(key, data) {
        // Remove an item when it is undefined or null
        if (types_1.isUndefinedOrNull(data)) {
            return this.removeItem(key);
        }
        // Shortcut for primitives that did not change
        if (typeof data === 'string' || typeof data === 'number' || typeof data === 'boolean') {
            if (this.database[key] === data) {
                return;
            }
        }
        this.database[key] = data;
        this.saveSync();
    }
    removeItem(key) {
        // Only update if the key is actually present (not undefined)
        if (!types_1.isUndefined(this.database[key])) {
            this.database[key] = undefined;
            this.saveSync();
        }
    }
    saveSync() {
        const serializedDatabase = JSON.stringify(this.database, null, 4);
        if (serializedDatabase === this.lastFlushedSerializedDatabase) {
            return; // return early if the database has not changed
        }
        try {
            pfs_1.writeFileSync(this.dbPath, serializedDatabase); // permission issue can happen here
            this.lastFlushedSerializedDatabase = serializedDatabase;
        }
        catch (error) {
            this.onError(error);
        }
    }
}
exports.FileStorage = FileStorage;
let StateService = class StateService {
    constructor(environmentService, logService) {
        this.fileStorage = new FileStorage(path.join(environmentService.userDataPath, StateService.STATE_FILE), error => logService.error(error));
    }
    init() {
        return this.fileStorage.init();
    }
    getItem(key, defaultValue) {
        return this.fileStorage.getItem(key, defaultValue);
    }
    setItem(key, data) {
        this.fileStorage.setItem(key, data);
    }
    removeItem(key) {
        this.fileStorage.removeItem(key);
    }
};
StateService.STATE_FILE = 'storage.json';
StateService = __decorate([
    __param(0, environment_1.IEnvironmentService),
    __param(1, log_1.ILogService)
], StateService);
exports.StateService = StateService;
//# sourceMappingURL=stateService.js.map