"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const files_1 = require("vs/platform/files/common/files");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const buffer_1 = require("vs/base/common/buffer");
const event_1 = require("vs/base/common/event");
const log_1 = require("vs/platform/log/common/log");
const async_1 = require("vs/base/common/async");
const environment_1 = require("vs/platform/environment/common/environment");
const resources_1 = require("vs/base/common/resources");
const extensionManagement_1 = require("vs/platform/extensionManagement/common/extensionManagement");
const extensionManagementUtil_1 = require("vs/platform/extensionManagement/common/extensionManagementUtil");
let ExtensionsSynchroniser = class ExtensionsSynchroniser extends lifecycle_1.Disposable {
    constructor(fileService, environmentService, userDataSyncStoreService, extensionManagementService, logService) {
        super();
        this.fileService = fileService;
        this.environmentService = environmentService;
        this.userDataSyncStoreService = userDataSyncStoreService;
        this.extensionManagementService = extensionManagementService;
        this.logService = logService;
        this._status = userDataSync_1.SyncStatus.Idle;
        this._onDidChangStatus = this._register(new event_1.Emitter());
        this.onDidChangeStatus = this._onDidChangStatus.event;
        this._onDidChangeLocal = this._register(new event_1.Emitter());
        this.onDidChangeLocal = this._onDidChangeLocal.event;
        this.lastSyncExtensionsResource = resources_1.joinPath(this.environmentService.userRoamingDataHome, '.lastSyncExtensions');
        this.throttledDelayer = this._register(new async_1.ThrottledDelayer(500));
        this._register(event_1.Event.filter(this.fileService.onFileChanges, e => e.contains(this.environmentService.settingsResource))(() => this.throttledDelayer.trigger(() => this.onDidChangeSettings())));
    }
    get status() { return this._status; }
    onDidChangeSettings() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    setStatus(status) {
        if (this._status !== status) {
            this._status = status;
            this._onDidChangStatus.fire(status);
        }
    }
    sync() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.status !== userDataSync_1.SyncStatus.Idle) {
                return false;
            }
            this.setStatus(userDataSync_1.SyncStatus.Syncing);
            try {
                yield this.doSync();
            }
            catch (e) {
                this.setStatus(userDataSync_1.SyncStatus.Idle);
                if (e instanceof userDataSync_1.UserDataSyncStoreError && e.code === userDataSync_1.UserDataSyncStoreErrorCode.Rejected) {
                    // Rejected as there is a new remote version. Syncing again,
                    this.logService.info('Failed to Synchronise settings as there is a new remote version available. Synchronising again...');
                    return this.sync();
                }
                if (e instanceof files_1.FileSystemProviderError && e.code === files_1.FileSystemProviderErrorCode.FileExists) {
                    // Rejected as there is a new local version. Syncing again.
                    this.logService.info('Failed to Synchronise settings as there is a new local version available. Synchronising again...');
                    return this.sync();
                }
                throw e;
            }
            return true;
        });
    }
    doSync() {
        return __awaiter(this, void 0, void 0, function* () {
            const lastSyncData = yield this.getLastSyncUserData();
            const remoteData = yield this.userDataSyncStoreService.read(ExtensionsSynchroniser.EXTERNAL_USER_DATA_EXTENSIONS_KEY, lastSyncData);
            const lastSyncExtensions = lastSyncData ? JSON.parse(lastSyncData.content) : null;
            const remoteExtensions = remoteData.content ? JSON.parse(remoteData.content) : null;
            const localExtensions = yield this.getLocalExtensions();
            // First time sync to remote
            if (localExtensions && !remoteExtensions) {
                this.logService.trace('Settings Sync: Remote contents does not exist. So sync with settings file.');
                // Return local extensions
                return;
            }
            if (localExtensions && remoteExtensions) {
                const localToRemote = this.compare(localExtensions, remoteExtensions);
                if (localToRemote.added.length === 0 && localToRemote.removed.length === 0 && localToRemote.updated.length === 0) {
                    // No changes found between local and remote.
                    return;
                }
                const baseToLocal = lastSyncExtensions ? this.compare(lastSyncExtensions, localExtensions) : { added: localExtensions.map(({ identifier }) => identifier), removed: [], updated: [] };
                const baseToRemote = lastSyncExtensions ? this.compare(lastSyncExtensions, remoteExtensions) : { added: remoteExtensions.map(({ identifier }) => identifier), removed: [], updated: [] };
                // Locally added extensions
                for (const localAdded of baseToLocal.added) {
                    // Got added in remote
                    if (baseToRemote.added.some(added => extensionManagementUtil_1.areSameExtensions(added, localAdded))) {
                        // Is different from local to remote
                        if (localToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, localAdded))) {
                            // update it in local
                        }
                    }
                    else {
                        // add to remote
                    }
                }
                // Remotely added extension
                for (const remoteAdded of baseToRemote.added) {
                    // Got added in local
                    if (baseToLocal.added.some(added => extensionManagementUtil_1.areSameExtensions(added, remoteAdded))) {
                        // Is different from local to remote
                        if (localToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, remoteAdded))) {
                            // update it in local
                        }
                    }
                    else {
                        // add to local
                    }
                }
                // Locally updated extensions
                for (const localUpdated of baseToLocal.updated) {
                    // If updated in remote
                    if (baseToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, localUpdated))) {
                        // Is different from local to remote
                        if (localToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, localUpdated))) {
                            // update it in local
                        }
                    }
                }
                // Remotely updated extensions
                for (const remoteUpdated of baseToRemote.updated) {
                    // If updated in local
                    if (baseToLocal.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, remoteUpdated))) {
                        // Is different from local to remote
                        if (localToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, remoteUpdated))) {
                            // update it in local
                        }
                    }
                }
                // Locally removed extensions
                for (const localRemoved of baseToLocal.removed) {
                    // If not updated in remote
                    if (!baseToRemote.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, localRemoved))) {
                        // remove it from remote
                    }
                }
                // Remote removed extensions
                for (const remoteRemoved of baseToRemote.removed) {
                    // If not updated in local
                    if (!baseToLocal.updated.some(updated => extensionManagementUtil_1.areSameExtensions(updated, remoteRemoved))) {
                        // remove it from local
                    }
                }
            }
        });
    }
    compare(from, to) {
        const added = to.filter(toExtension => from.every(fromExtension => !extensionManagementUtil_1.areSameExtensions(fromExtension.identifier, toExtension.identifier))).map(({ identifier }) => identifier);
        const removed = from.filter(fromExtension => to.every(toExtension => !extensionManagementUtil_1.areSameExtensions(toExtension.identifier, fromExtension.identifier))).map(({ identifier }) => identifier);
        const updated = [];
        for (const fromExtension of from) {
            if (removed.some(identifier => extensionManagementUtil_1.areSameExtensions(identifier, fromExtension.identifier))) {
                continue;
            }
            const toExtension = to.filter(e => extensionManagementUtil_1.areSameExtensions(e.identifier, fromExtension.identifier))[0];
            if (fromExtension.enabled !== toExtension.enabled
                || fromExtension.version !== toExtension.version) {
                updated.push(fromExtension.identifier);
            }
        }
        return { added, removed, updated };
    }
    getLocalExtensions() {
        return __awaiter(this, void 0, void 0, function* () {
            const installedExtensions = yield this.extensionManagementService.getInstalled(1 /* User */);
            return installedExtensions.map(({ identifier }) => ({ identifier, enabled: true }));
        });
    }
    getLastSyncUserData() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const content = yield this.fileService.readFile(this.lastSyncExtensionsResource);
                return JSON.parse(content.value.toString());
            }
            catch (error) {
                return null;
            }
        });
    }
    writeToRemote(content, ref) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.userDataSyncStoreService.write(ExtensionsSynchroniser.EXTERNAL_USER_DATA_EXTENSIONS_KEY, content, ref);
        });
    }
    updateLastSyncValue(remoteUserData) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fileService.writeFile(this.lastSyncExtensionsResource, buffer_1.VSBuffer.fromString(JSON.stringify(remoteUserData)));
        });
    }
};
ExtensionsSynchroniser.EXTERNAL_USER_DATA_EXTENSIONS_KEY = 'extensions';
ExtensionsSynchroniser = __decorate([
    __param(0, files_1.IFileService),
    __param(1, environment_1.IEnvironmentService),
    __param(2, userDataSync_1.IUserDataSyncStoreService),
    __param(3, extensionManagement_1.IExtensionManagementService),
    __param(4, log_1.ILogService)
], ExtensionsSynchroniser);
exports.ExtensionsSynchroniser = ExtensionsSynchroniser;
//# sourceMappingURL=extensionsSync.js.map