"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const files_1 = require("vs/platform/files/common/files");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const buffer_1 = require("vs/base/common/buffer");
const json_1 = require("vs/base/common/json");
const nls_1 = require("vs/nls");
const event_1 = require("vs/base/common/event");
const log_1 = require("vs/platform/log/common/log");
const async_1 = require("vs/base/common/async");
const environment_1 = require("vs/platform/environment/common/environment");
const resources_1 = require("vs/base/common/resources");
let SettingsSynchroniser = class SettingsSynchroniser extends lifecycle_1.Disposable {
    constructor(fileService, environmentService, userDataSyncStoreService, settingsMergeService, logService) {
        super();
        this.fileService = fileService;
        this.environmentService = environmentService;
        this.userDataSyncStoreService = userDataSyncStoreService;
        this.settingsMergeService = settingsMergeService;
        this.logService = logService;
        this.syncPreviewResultPromise = null;
        this._status = userDataSync_1.SyncStatus.Idle;
        this._onDidChangStatus = this._register(new event_1.Emitter());
        this.onDidChangeStatus = this._onDidChangStatus.event;
        this._onDidChangeLocal = this._register(new event_1.Emitter());
        this.onDidChangeLocal = this._onDidChangeLocal.event;
        this.lastSyncSettingsResource = resources_1.joinPath(this.environmentService.userRoamingDataHome, '.lastSyncSettings.json');
        this.throttledDelayer = this._register(new async_1.ThrottledDelayer(500));
        this._register(event_1.Event.filter(this.fileService.onFileChanges, e => e.contains(this.environmentService.settingsResource))(() => this.throttledDelayer.trigger(() => this.onDidChangeSettings())));
    }
    get status() { return this._status; }
    onDidChangeSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const localFileContent = yield this.getLocalFileContent();
            const lastSyncData = yield this.getLastSyncUserData();
            if (localFileContent && lastSyncData) {
                if (localFileContent.value.toString() !== lastSyncData.content) {
                    this._onDidChangeLocal.fire();
                    return;
                }
            }
            if (!localFileContent || !lastSyncData) {
                this._onDidChangeLocal.fire();
                return;
            }
        });
    }
    setStatus(status) {
        if (this._status !== status) {
            this._status = status;
            this._onDidChangStatus.fire(status);
        }
    }
    sync(_continue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (_continue) {
                return this.continueSync();
            }
            if (this.status !== userDataSync_1.SyncStatus.Idle) {
                return false;
            }
            this.setStatus(userDataSync_1.SyncStatus.Syncing);
            try {
                const result = yield this.getPreview();
                if (result.hasConflicts) {
                    this.setStatus(userDataSync_1.SyncStatus.HasConflicts);
                    return false;
                }
                yield this.apply();
                return true;
            }
            catch (e) {
                this.syncPreviewResultPromise = null;
                this.setStatus(userDataSync_1.SyncStatus.Idle);
                if (e instanceof userDataSync_1.UserDataSyncStoreError && e.code === userDataSync_1.UserDataSyncStoreErrorCode.Rejected) {
                    // Rejected as there is a new remote version. Syncing again,
                    this.logService.info('Failed to Synchronise settings as there is a new remote version available. Synchronising again...');
                    return this.sync();
                }
                if (e instanceof files_1.FileSystemProviderError && e.code === files_1.FileSystemProviderErrorCode.FileExists) {
                    // Rejected as there is a new local version. Syncing again.
                    this.logService.info('Failed to Synchronise settings as there is a new local version available. Synchronising again...');
                    return this.sync();
                }
                throw e;
            }
        });
    }
    continueSync() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.status !== userDataSync_1.SyncStatus.HasConflicts) {
                return false;
            }
            yield this.apply();
            return true;
        });
    }
    apply() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.syncPreviewResultPromise) {
                return;
            }
            if (yield this.fileService.exists(this.environmentService.settingsSyncPreviewResource)) {
                const settingsPreivew = yield this.fileService.readFile(this.environmentService.settingsSyncPreviewResource);
                const content = settingsPreivew.value.toString();
                if (this.hasErrors(content)) {
                    return Promise.reject(nls_1.localize('errorInvalidSettings', "Unable to sync settings. Please resolve conflicts without any errors/warnings and try again."));
                }
                let { fileContent, remoteUserData, hasLocalChanged, hasRemoteChanged } = yield this.syncPreviewResultPromise;
                if (hasRemoteChanged) {
                    const ref = yield this.writeToRemote(content, remoteUserData.ref);
                    remoteUserData = { ref, content };
                }
                if (hasLocalChanged) {
                    yield this.writeToLocal(content, fileContent);
                }
                if (remoteUserData.content) {
                    yield this.updateLastSyncValue(remoteUserData);
                }
                // Delete the preview
                yield this.fileService.del(this.environmentService.settingsSyncPreviewResource);
            }
            this.syncPreviewResultPromise = null;
            this.setStatus(userDataSync_1.SyncStatus.Idle);
        });
    }
    hasErrors(content) {
        const parseErrors = [];
        json_1.parse(content, parseErrors);
        return parseErrors.length > 0;
    }
    getPreview() {
        if (!this.syncPreviewResultPromise) {
            this.syncPreviewResultPromise = async_1.createCancelablePromise(token => this.generatePreview());
        }
        return this.syncPreviewResultPromise;
    }
    generatePreview() {
        return __awaiter(this, void 0, void 0, function* () {
            const lastSyncData = yield this.getLastSyncUserData();
            const remoteUserData = yield this.userDataSyncStoreService.read(SettingsSynchroniser.EXTERNAL_USER_DATA_SETTINGS_KEY, lastSyncData);
            const remoteContent = remoteUserData.content;
            // Get file content last to get the latest
            const fileContent = yield this.getLocalFileContent();
            let hasLocalChanged = false;
            let hasRemoteChanged = false;
            let hasConflicts = false;
            // First time sync to remote
            if (fileContent && !remoteContent) {
                this.logService.trace('Settings Sync: Remote contents does not exist. So sync with settings file.');
                hasRemoteChanged = true;
                yield this.fileService.writeFile(this.environmentService.settingsSyncPreviewResource, buffer_1.VSBuffer.fromString(fileContent.value.toString()));
                return { fileContent, remoteUserData, hasLocalChanged, hasRemoteChanged, hasConflicts };
            }
            // Settings file does not exist, so sync with remote contents.
            if (remoteContent && !fileContent) {
                this.logService.trace('Settings Sync: Settings file does not exist. So sync with remote contents');
                hasLocalChanged = true;
                yield this.fileService.writeFile(this.environmentService.settingsSyncPreviewResource, buffer_1.VSBuffer.fromString(remoteContent));
                return { fileContent, remoteUserData, hasLocalChanged, hasRemoteChanged, hasConflicts };
            }
            if (fileContent && remoteContent) {
                const localContent = fileContent.value.toString();
                if (!lastSyncData // First time sync
                    || lastSyncData.content !== localContent // Local has moved forwarded
                    || lastSyncData.content !== remoteContent // Remote has moved forwarded
                ) {
                    this.logService.trace('Settings Sync: Merging remote contents with settings file.');
                    const result = yield this.settingsMergeService.merge(localContent, remoteContent, lastSyncData ? lastSyncData.content : null);
                    // Sync only if there are changes
                    if (result.hasChanges) {
                        hasLocalChanged = result.mergeContent !== localContent;
                        hasRemoteChanged = result.mergeContent !== remoteContent;
                        hasConflicts = result.hasConflicts;
                        yield this.fileService.writeFile(this.environmentService.settingsSyncPreviewResource, buffer_1.VSBuffer.fromString(result.mergeContent));
                        return { fileContent, remoteUserData, hasLocalChanged, hasRemoteChanged, hasConflicts };
                    }
                }
            }
            this.logService.trace('Settings Sync: No changes.');
            return { fileContent, remoteUserData, hasLocalChanged, hasRemoteChanged, hasConflicts };
        });
    }
    getLastSyncUserData() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const content = yield this.fileService.readFile(this.lastSyncSettingsResource);
                return JSON.parse(content.value.toString());
            }
            catch (error) {
                return null;
            }
        });
    }
    getLocalFileContent() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.fileService.readFile(this.environmentService.settingsResource);
            }
            catch (error) {
                return null;
            }
        });
    }
    writeToRemote(content, ref) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.userDataSyncStoreService.write(SettingsSynchroniser.EXTERNAL_USER_DATA_SETTINGS_KEY, content, ref);
        });
    }
    writeToLocal(newContent, oldContent) {
        return __awaiter(this, void 0, void 0, function* () {
            if (oldContent) {
                // file exists already
                yield this.fileService.writeFile(this.environmentService.settingsResource, buffer_1.VSBuffer.fromString(newContent), oldContent);
            }
            else {
                // file does not exist
                yield this.fileService.createFile(this.environmentService.settingsResource, buffer_1.VSBuffer.fromString(newContent), { overwrite: false });
            }
        });
    }
    updateLastSyncValue(remoteUserData) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.fileService.writeFile(this.lastSyncSettingsResource, buffer_1.VSBuffer.fromString(JSON.stringify(remoteUserData)));
        });
    }
};
SettingsSynchroniser.EXTERNAL_USER_DATA_SETTINGS_KEY = 'settings';
SettingsSynchroniser = __decorate([
    __param(0, files_1.IFileService),
    __param(1, environment_1.IEnvironmentService),
    __param(2, userDataSync_1.IUserDataSyncStoreService),
    __param(3, userDataSync_1.ISettingsMergeService),
    __param(4, log_1.ILogService)
], SettingsSynchroniser);
exports.SettingsSynchroniser = SettingsSynchroniser;
//# sourceMappingURL=settingsSync.js.map