"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const event_1 = require("vs/base/common/event");
const windows_1 = require("vs/platform/windows/common/windows");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const uri_1 = require("vs/base/common/uri");
const history_1 = require("vs/platform/history/common/history");
class WindowsChannel {
    constructor(service) {
        this.service = service;
        this.onWindowOpen = event_1.Event.buffer(service.onWindowOpen, true);
        this.onWindowFocus = event_1.Event.buffer(service.onWindowFocus, true);
        this.onWindowBlur = event_1.Event.buffer(service.onWindowBlur, true);
        this.onWindowMaximize = event_1.Event.buffer(service.onWindowMaximize, true);
        this.onWindowUnmaximize = event_1.Event.buffer(service.onWindowUnmaximize, true);
        this.onRecentlyOpenedChange = event_1.Event.buffer(service.onRecentlyOpenedChange, true);
    }
    listen(_, event) {
        switch (event) {
            case 'onWindowOpen': return this.onWindowOpen;
            case 'onWindowFocus': return this.onWindowFocus;
            case 'onWindowBlur': return this.onWindowBlur;
            case 'onWindowMaximize': return this.onWindowMaximize;
            case 'onWindowUnmaximize': return this.onWindowUnmaximize;
            case 'onRecentlyOpenedChange': return this.onRecentlyOpenedChange;
        }
        throw new Error(`Event not found: ${event}`);
    }
    call(_, command, arg) {
        switch (command) {
            case 'closeWorkspace': return this.service.closeWorkspace(arg);
            case 'enterWorkspace': return this.service.enterWorkspace(arg[0], uri_1.URI.revive(arg[1]));
            case 'addRecentlyOpened': return this.service.addRecentlyOpened(arg.map((recent) => {
                if (history_1.isRecentFile(recent)) {
                    recent.fileUri = uri_1.URI.revive(recent.fileUri);
                }
                else if (history_1.isRecentFolder(recent)) {
                    recent.folderUri = uri_1.URI.revive(recent.folderUri);
                }
                else {
                    recent.workspace = workspaces_1.reviveWorkspaceIdentifier(recent.workspace);
                }
                return recent;
            }));
            case 'removeFromRecentlyOpened': return this.service.removeFromRecentlyOpened(arg.map(uri_1.URI.revive));
            case 'clearRecentlyOpened': return this.service.clearRecentlyOpened();
            case 'newWindowTab': return this.service.newWindowTab();
            case 'showPreviousWindowTab': return this.service.showPreviousWindowTab();
            case 'showNextWindowTab': return this.service.showNextWindowTab();
            case 'moveWindowTabToNewWindow': return this.service.moveWindowTabToNewWindow();
            case 'mergeAllWindowTabs': return this.service.mergeAllWindowTabs();
            case 'toggleWindowTabsBar': return this.service.toggleWindowTabsBar();
            case 'updateTouchBar': return this.service.updateTouchBar(arg[0], arg[1]);
            case 'getRecentlyOpened': return this.service.getRecentlyOpened(arg);
            case 'focusWindow': return this.service.focusWindow(arg);
            case 'closeWindow': return this.service.closeWindow(arg);
            case 'isFocused': return this.service.isFocused(arg);
            case 'isMaximized': return this.service.isMaximized(arg);
            case 'maximizeWindow': return this.service.maximizeWindow(arg);
            case 'unmaximizeWindow': return this.service.unmaximizeWindow(arg);
            case 'minimizeWindow': return this.service.minimizeWindow(arg);
            case 'onWindowTitleDoubleClick': return this.service.onWindowTitleDoubleClick(arg);
            case 'openWindow': {
                const urisToOpen = arg[1];
                const options = arg[2];
                urisToOpen.forEach(r => {
                    if (windows_1.isWorkspaceToOpen(r)) {
                        r.workspaceUri = uri_1.URI.revive(r.workspaceUri);
                    }
                    else if (windows_1.isFolderToOpen(r)) {
                        r.folderUri = uri_1.URI.revive(r.folderUri);
                    }
                    else {
                        r.fileUri = uri_1.URI.revive(r.fileUri);
                    }
                });
                options.waitMarkerFileURI = options.waitMarkerFileURI && uri_1.URI.revive(options.waitMarkerFileURI);
                return this.service.openWindow(arg[0], urisToOpen, options);
            }
            case 'openExtensionDevelopmentHostWindow': return this.service.openExtensionDevelopmentHostWindow(arg[0], arg[1]);
            case 'getWindows': return this.service.getWindows();
            case 'whenSharedProcessReady': return this.service.whenSharedProcessReady();
            case 'toggleSharedProcess': return this.service.toggleSharedProcess();
            case 'quit': return this.service.quit();
            case 'getActiveWindowId': return this.service.getActiveWindowId();
            case 'openExternal': return this.service.openExternal(arg);
            case 'startCrashReporter': return this.service.startCrashReporter(arg);
        }
        throw new Error(`Call not found: ${command}`);
    }
}
exports.WindowsChannel = WindowsChannel;
//# sourceMappingURL=windowsIpc.js.map