"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls_1 = require("vs/nls");
const errors_1 = require("vs/base/common/errors");
const lifecycle_1 = require("vs/base/common/lifecycle");
const objects_1 = require("vs/base/common/objects");
const uri_1 = require("vs/base/common/uri");
const bulkEditService_1 = require("vs/editor/browser/services/bulkEditService");
const codeEditorService_1 = require("vs/editor/browser/services/codeEditorService");
const commands_1 = require("vs/platform/commands/common/commands");
const editor_1 = require("vs/platform/editor/common/editor");
const opener_1 = require("vs/platform/opener/common/opener");
const extHost_protocol_1 = require("vs/workbench/api/common/extHost.protocol");
const editor_2 = require("vs/workbench/api/common/shared/editor");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
let MainThreadTextEditors = class MainThreadTextEditors {
    constructor(documentsAndEditors, extHostContext, _codeEditorService, _bulkEditService, _editorService, _editorGroupService) {
        this._codeEditorService = _codeEditorService;
        this._bulkEditService = _bulkEditService;
        this._editorService = _editorService;
        this._editorGroupService = _editorGroupService;
        this._toDispose = new lifecycle_1.DisposableStore();
        this._instanceId = String(++MainThreadTextEditors.INSTANCE_COUNT);
        this._proxy = extHostContext.getProxy(extHost_protocol_1.ExtHostContext.ExtHostEditors);
        this._documentsAndEditors = documentsAndEditors;
        this._textEditorsListenersMap = Object.create(null);
        this._editorPositionData = null;
        this._toDispose.add(documentsAndEditors.onTextEditorAdd(editors => editors.forEach(this._onTextEditorAdd, this)));
        this._toDispose.add(documentsAndEditors.onTextEditorRemove(editors => editors.forEach(this._onTextEditorRemove, this)));
        this._toDispose.add(this._editorService.onDidVisibleEditorsChange(() => this._updateActiveAndVisibleTextEditors()));
        this._toDispose.add(this._editorGroupService.onDidRemoveGroup(() => this._updateActiveAndVisibleTextEditors()));
        this._toDispose.add(this._editorGroupService.onDidMoveGroup(() => this._updateActiveAndVisibleTextEditors()));
        this._registeredDecorationTypes = Object.create(null);
    }
    dispose() {
        Object.keys(this._textEditorsListenersMap).forEach((editorId) => {
            lifecycle_1.dispose(this._textEditorsListenersMap[editorId]);
        });
        this._textEditorsListenersMap = Object.create(null);
        this._toDispose.dispose();
        for (let decorationType in this._registeredDecorationTypes) {
            this._codeEditorService.removeDecorationType(decorationType);
        }
        this._registeredDecorationTypes = Object.create(null);
    }
    _onTextEditorAdd(textEditor) {
        const id = textEditor.getId();
        const toDispose = [];
        toDispose.push(textEditor.onPropertiesChanged((data) => {
            this._proxy.$acceptEditorPropertiesChanged(id, data);
        }));
        this._textEditorsListenersMap[id] = toDispose;
    }
    _onTextEditorRemove(id) {
        lifecycle_1.dispose(this._textEditorsListenersMap[id]);
        delete this._textEditorsListenersMap[id];
    }
    _updateActiveAndVisibleTextEditors() {
        // editor columns
        const editorPositionData = this._getTextEditorPositionData();
        if (!objects_1.equals(this._editorPositionData, editorPositionData)) {
            this._editorPositionData = editorPositionData;
            this._proxy.$acceptEditorPositionData(this._editorPositionData);
        }
    }
    _getTextEditorPositionData() {
        const result = Object.create(null);
        for (let workbenchEditor of this._editorService.visibleControls) {
            const id = this._documentsAndEditors.findTextEditorIdFor(workbenchEditor);
            if (id) {
                result[id] = editor_2.editorGroupToViewColumn(this._editorGroupService, workbenchEditor.group);
            }
        }
        return result;
    }
    // --- from extension host process
    $tryShowTextDocument(resource, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = uri_1.URI.revive(resource);
            const editorOptions = {
                preserveFocus: options.preserveFocus,
                pinned: options.pinned,
                selection: options.selection,
                // preserve pre 1.38 behaviour to not make group active when preserveFocus: true
                // but make sure to restore the editor to fix https://github.com/microsoft/vscode/issues/79633
                activation: options.preserveFocus ? editor_1.EditorActivation.RESTORE : undefined
            };
            const input = {
                resource: uri,
                options: editorOptions
            };
            const editor = yield this._editorService.openEditor(input, editor_2.viewColumnToEditorGroup(this._editorGroupService, options.position));
            if (!editor) {
                return undefined;
            }
            return this._documentsAndEditors.findTextEditorIdFor(editor);
        });
    }
    $tryShowEditor(id, position) {
        return __awaiter(this, void 0, void 0, function* () {
            const mainThreadEditor = this._documentsAndEditors.getEditor(id);
            if (mainThreadEditor) {
                const model = mainThreadEditor.getModel();
                yield this._editorService.openEditor({
                    resource: model.uri,
                    options: { preserveFocus: false }
                }, editor_2.viewColumnToEditorGroup(this._editorGroupService, position));
                return;
            }
        });
    }
    $tryHideEditor(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const mainThreadEditor = this._documentsAndEditors.getEditor(id);
            if (mainThreadEditor) {
                const editors = this._editorService.visibleControls;
                for (let editor of editors) {
                    if (mainThreadEditor.matches(editor)) {
                        return editor.group.closeEditor(editor.input);
                    }
                }
            }
        });
    }
    $trySetSelections(id, selections) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        editor.setSelections(selections);
        return Promise.resolve(undefined);
    }
    $trySetDecorations(id, key, ranges) {
        key = `${this._instanceId}-${key}`;
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        editor.setDecorations(key, ranges);
        return Promise.resolve(undefined);
    }
    $trySetDecorationsFast(id, key, ranges) {
        key = `${this._instanceId}-${key}`;
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        editor.setDecorationsFast(key, ranges);
        return Promise.resolve(undefined);
    }
    $tryRevealRange(id, range, revealType) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        editor.revealRange(range, revealType);
        return Promise.resolve();
    }
    $trySetOptions(id, options) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        editor.setConfiguration(options);
        return Promise.resolve(undefined);
    }
    $tryApplyEdits(id, modelVersionId, edits, opts) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        return Promise.resolve(editor.applyEdits(modelVersionId, edits, opts));
    }
    $tryApplyWorkspaceEdit(dto) {
        const { edits } = extHost_protocol_1.reviveWorkspaceEditDto(dto);
        return this._bulkEditService.apply({ edits }, undefined).then(() => true, err => false);
    }
    $tryInsertSnippet(id, template, ranges, opts) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(errors_1.disposed(`TextEditor(${id})`));
        }
        return Promise.resolve(editor.insertSnippet(template, ranges, opts));
    }
    $registerTextEditorDecorationType(key, options) {
        key = `${this._instanceId}-${key}`;
        this._registeredDecorationTypes[key] = true;
        this._codeEditorService.registerDecorationType(key, options);
    }
    $removeTextEditorDecorationType(key) {
        key = `${this._instanceId}-${key}`;
        delete this._registeredDecorationTypes[key];
        this._codeEditorService.removeDecorationType(key);
    }
    $getDiffInformation(id) {
        const editor = this._documentsAndEditors.getEditor(id);
        if (!editor) {
            return Promise.reject(new Error('No such TextEditor'));
        }
        const codeEditor = editor.getCodeEditor();
        if (!codeEditor) {
            return Promise.reject(new Error('No such CodeEditor'));
        }
        const codeEditorId = codeEditor.getId();
        const diffEditors = this._codeEditorService.listDiffEditors();
        const [diffEditor] = diffEditors.filter(d => d.getOriginalEditor().getId() === codeEditorId || d.getModifiedEditor().getId() === codeEditorId);
        if (diffEditor) {
            return Promise.resolve(diffEditor.getLineChanges() || []);
        }
        const dirtyDiffContribution = codeEditor.getContribution('editor.contrib.dirtydiff');
        if (dirtyDiffContribution) {
            return Promise.resolve(dirtyDiffContribution.getChanges());
        }
        return Promise.resolve([]);
    }
};
MainThreadTextEditors.INSTANCE_COUNT = 0;
MainThreadTextEditors = __decorate([
    __param(2, codeEditorService_1.ICodeEditorService),
    __param(3, bulkEditService_1.IBulkEditService),
    __param(4, editorService_1.IEditorService),
    __param(5, editorGroupsService_1.IEditorGroupsService)
], MainThreadTextEditors);
exports.MainThreadTextEditors = MainThreadTextEditors;
// --- commands
commands_1.CommandsRegistry.registerCommand('_workbench.open', function (accessor, args) {
    const editorService = accessor.get(editorService_1.IEditorService);
    const editorGroupService = accessor.get(editorGroupsService_1.IEditorGroupsService);
    const openerService = accessor.get(opener_1.IOpenerService);
    const [resource, options, position, label] = args;
    if (options || typeof position === 'number') {
        // use editor options or editor view column as a hint to use the editor service for opening
        return editorService.openEditor({ resource, options, label }, editor_2.viewColumnToEditorGroup(editorGroupService, position)).then(_ => undefined);
    }
    if (resource && resource.scheme === 'command') {
        // do not allow to execute commands from here
        return Promise.resolve(undefined);
    }
    // finally, delegate to opener service
    return openerService.open(resource).then(_ => undefined);
});
commands_1.CommandsRegistry.registerCommand('_workbench.diff', function (accessor, args) {
    const editorService = accessor.get(editorService_1.IEditorService);
    const editorGroupService = accessor.get(editorGroupsService_1.IEditorGroupsService);
    let [leftResource, rightResource, label, description, options, position] = args;
    if (!options || typeof options !== 'object') {
        options = {
            preserveFocus: false
        };
    }
    if (!label) {
        label = nls_1.localize('diffLeftRightLabel', "{0} ⟷ {1}", leftResource.toString(true), rightResource.toString(true));
    }
    return editorService.openEditor({ leftResource, rightResource, label, description, options }, editor_2.viewColumnToEditorGroup(editorGroupService, position)).then(() => undefined);
});
//# sourceMappingURL=mainThreadEditors.js.map