"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const DOM = require("vs/base/browser/dom");
const lifecycle_1 = require("vs/base/common/lifecycle");
const network_1 = require("vs/base/common/network");
require("vs/css!./media/resourceviewer");
const nls = require("vs/nls");
const themeService_1 = require("vs/platform/theme/common/themeService");
const theme_1 = require("vs/workbench/common/theme");
class BinarySize {
    static formatSize(size) {
        if (size < BinarySize.KB) {
            return nls.localize('sizeB', "{0}B", size);
        }
        if (size < BinarySize.MB) {
            return nls.localize('sizeKB', "{0}KB", (size / BinarySize.KB).toFixed(2));
        }
        if (size < BinarySize.GB) {
            return nls.localize('sizeMB', "{0}MB", (size / BinarySize.MB).toFixed(2));
        }
        if (size < BinarySize.TB) {
            return nls.localize('sizeGB', "{0}GB", (size / BinarySize.GB).toFixed(2));
        }
        return nls.localize('sizeTB', "{0}TB", (size / BinarySize.TB).toFixed(2));
    }
}
BinarySize.KB = 1024;
BinarySize.MB = BinarySize.KB * BinarySize.KB;
BinarySize.GB = BinarySize.MB * BinarySize.KB;
BinarySize.TB = BinarySize.GB * BinarySize.KB;
themeService_1.registerThemingParticipant((theme, collector) => {
    const borderColor = theme.getColor(theme_1.IMAGE_PREVIEW_BORDER);
    collector.addRule(`.monaco-resource-viewer.image img { border : 1px solid ${borderColor ? borderColor.toString() : ''}; }`);
});
/**
 * Helper to actually render the given resource into the provided container. Will adjust scrollbar (if provided) automatically based on loading
 * progress of the binary resource.
 */
class ResourceViewer {
    static show(descriptor, fileService, container, scrollbar, delegate, instantiationService) {
        // Ensure CSS class
        container.className = 'monaco-resource-viewer';
        // Large Files
        if (descriptor.size > ResourceViewer.MAX_OPEN_INTERNAL_SIZE) {
            return FileTooLargeFileView.create(container, descriptor, scrollbar, delegate);
        }
        // Seemingly Binary Files
        else {
            return FileSeemsBinaryFileView.create(container, descriptor, scrollbar, delegate);
        }
    }
}
exports.ResourceViewer = ResourceViewer;
ResourceViewer.MAX_OPEN_INTERNAL_SIZE = BinarySize.MB * 200; // max size until we offer an action to open internally
class FileTooLargeFileView {
    static create(container, descriptor, scrollbar, delegate) {
        const size = BinarySize.formatSize(descriptor.size);
        delegate.metadataClb(size);
        DOM.clearNode(container);
        const label = document.createElement('span');
        label.textContent = nls.localize('nativeFileTooLargeError', "The file is not displayed in the editor because it is too large ({0}).", size);
        container.appendChild(label);
        scrollbar.scanDomNode();
        return lifecycle_1.Disposable.None;
    }
}
class FileSeemsBinaryFileView {
    static create(container, descriptor, scrollbar, delegate) {
        delegate.metadataClb(typeof descriptor.size === 'number' ? BinarySize.formatSize(descriptor.size) : '');
        DOM.clearNode(container);
        const disposables = new lifecycle_1.DisposableStore();
        const label = document.createElement('p');
        label.textContent = nls.localize('nativeBinaryError', "The file is not displayed in the editor because it is either binary or uses an unsupported text encoding.");
        container.appendChild(label);
        if (descriptor.resource.scheme !== network_1.Schemas.data) {
            const link = DOM.append(label, DOM.$('a.embedded-link'));
            link.setAttribute('role', 'button');
            link.textContent = nls.localize('openAsText', "Do you want to open it anyway?");
            disposables.add(DOM.addDisposableListener(link, DOM.EventType.CLICK, () => delegate.openInternalClb(descriptor.resource)));
        }
        scrollbar.scanDomNode();
        return disposables;
    }
}
//# sourceMappingURL=resourceViewer.js.map