"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const performance_1 = require("vs/base/common/performance");
const dom_1 = require("vs/base/browser/dom");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const log_1 = require("vs/platform/log/common/log");
const lifecycle_1 = require("vs/base/common/lifecycle");
const environmentService_1 = require("vs/workbench/services/environment/browser/environmentService");
const workbench_1 = require("vs/workbench/browser/workbench");
const remoteAgentFileSystemChannel_1 = require("vs/platform/remote/common/remoteAgentFileSystemChannel");
const environmentService_2 = require("vs/workbench/services/environment/common/environmentService");
const productService_1 = require("vs/platform/product/common/productService");
const product_1 = require("vs/platform/product/common/product");
const remoteAgentServiceImpl_1 = require("vs/workbench/services/remote/browser/remoteAgentServiceImpl");
const remoteAuthorityResolverService_1 = require("vs/platform/remote/browser/remoteAuthorityResolverService");
const remoteAuthorityResolver_1 = require("vs/platform/remote/common/remoteAuthorityResolver");
const remoteAgentService_1 = require("vs/workbench/services/remote/common/remoteAgentService");
const files_1 = require("vs/platform/files/common/files");
const fileService_1 = require("vs/platform/files/common/fileService");
const network_1 = require("vs/base/common/network");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const errors_1 = require("vs/base/common/errors");
const uri_1 = require("vs/base/common/uri");
const configurationService_1 = require("vs/workbench/services/configuration/browser/configurationService");
const configurationCache_1 = require("vs/workbench/services/configuration/browser/configurationCache");
const sign_1 = require("vs/platform/sign/common/sign");
const signService_1 = require("vs/platform/sign/browser/signService");
const hash_1 = require("vs/base/common/hash");
const fileUserDataProvider_1 = require("vs/workbench/services/userData/common/fileUserDataProvider");
const environment_1 = require("vs/platform/environment/common/environment");
const resources_1 = require("vs/base/common/resources");
const storageService_1 = require("vs/platform/storage/browser/storageService");
const storage_1 = require("vs/platform/storage/common/storage");
const themeService_1 = require("vs/platform/theme/common/themeService");
const inMemoryUserDataProvider_1 = require("vs/workbench/services/userData/common/inMemoryUserDataProvider");
const driver_1 = require("vs/platform/driver/browser/driver");
const bufferLog_1 = require("vs/platform/log/common/bufferLog");
const fileLogService_1 = require("vs/platform/log/common/fileLogService");
const date_1 = require("vs/base/common/date");
const indexedDBLogProvider_1 = require("vs/workbench/services/log/browser/indexedDBLogProvider");
const inMemoryLogProvider_1 = require("vs/workbench/services/log/common/inMemoryLogProvider");
class CodeRendererMain extends lifecycle_1.Disposable {
    constructor(domElement, configuration) {
        super();
        this.domElement = domElement;
        this.configuration = configuration;
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            const services = yield this.initServices();
            yield dom_1.domContentLoaded();
            performance_1.mark('willStartWorkbench');
            // Base Theme
            this.restoreBaseTheme();
            // Create Workbench
            const workbench = new workbench_1.Workbench(this.domElement, services.serviceCollection, services.logService);
            // Layout
            this._register(dom_1.addDisposableListener(window, dom_1.EventType.RESIZE, () => workbench.layout()));
            // Prevent the back/forward gestures in macOS
            this._register(dom_1.addDisposableListener(this.domElement, dom_1.EventType.WHEEL, (e) => {
                e.preventDefault();
            }, { passive: false }));
            // Workbench Lifecycle
            this._register(workbench.onBeforeShutdown(event => {
                if (services.storageService.hasPendingUpdate) {
                    console.warn('Unload prevented: pending storage update');
                    event.veto(true); // prevent data loss from pending storage update
                }
            }));
            this._register(workbench.onWillShutdown(() => {
                services.storageService.close();
                this.saveBaseTheme();
            }));
            this._register(workbench.onShutdown(() => this.dispose()));
            // Driver
            if (this.configuration.driver) {
                (() => __awaiter(this, void 0, void 0, function* () { return this._register(yield driver_1.registerWindowDriver()); }))();
            }
            // Startup
            workbench.startup();
        });
    }
    restoreBaseTheme() {
        dom_1.addClass(this.domElement, window.localStorage.getItem('baseTheme') || themeService_1.getThemeTypeSelector(themeService_1.DARK));
    }
    saveBaseTheme() {
        const classes = this.domElement.className;
        const baseThemes = [themeService_1.DARK, themeService_1.LIGHT, themeService_1.HIGH_CONTRAST].map(baseTheme => themeService_1.getThemeTypeSelector(baseTheme));
        for (const baseTheme of baseThemes) {
            if (classes.indexOf(baseTheme) >= 0) {
                window.localStorage.setItem('baseTheme', baseTheme);
                break;
            }
        }
    }
    initServices() {
        return __awaiter(this, void 0, void 0, function* () {
            const serviceCollection = new serviceCollection_1.ServiceCollection();
            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // NOTE: DO NOT ADD ANY OTHER SERVICE INTO THE COLLECTION HERE.
            // CONTRIBUTE IT VIA WORKBENCH.WEB.MAIN.TS AND registerSingleton().
            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // Log
            const logsPath = uri_1.URI.file(date_1.toLocalISOString(new Date()).replace(/-|:|\.\d+Z$/g, '')).with({ scheme: 'vscode-log' });
            const logService = new bufferLog_1.BufferLogService(this.configuration.logLevel);
            serviceCollection.set(log_1.ILogService, logService);
            const payload = this.resolveWorkspaceInitializationPayload();
            // Environment
            const environmentService = new environmentService_1.BrowserWorkbenchEnvironmentService(Object.assign({ workspaceId: payload.id, logsPath }, this.configuration));
            serviceCollection.set(environmentService_2.IWorkbenchEnvironmentService, environmentService);
            // Product
            const productService = Object.assign({ _serviceBrand: undefined }, Object.assign(Object.assign({}, product_1.default), { urlProtocol: '' } // web related overrides from us
            ));
            serviceCollection.set(productService_1.IProductService, productService);
            // Remote
            const remoteAuthorityResolverService = new remoteAuthorityResolverService_1.RemoteAuthorityResolverService(this.configuration.resourceUriProvider);
            serviceCollection.set(remoteAuthorityResolver_1.IRemoteAuthorityResolverService, remoteAuthorityResolverService);
            // Signing
            const signService = new signService_1.SignService(environmentService.configuration.connectionToken);
            serviceCollection.set(sign_1.ISignService, signService);
            // Remote Agent
            const remoteAgentService = this._register(new remoteAgentServiceImpl_1.RemoteAgentService(this.configuration.webSocketFactory, environmentService, productService, remoteAuthorityResolverService, signService, logService));
            serviceCollection.set(remoteAgentService_1.IRemoteAgentService, remoteAgentService);
            // Files
            const fileService = this._register(new fileService_1.FileService(logService));
            serviceCollection.set(files_1.IFileService, fileService);
            this.registerFileSystemProviders(environmentService, fileService, remoteAgentService, logService, logsPath);
            // Long running services (workspace, config, storage)
            const services = yield Promise.all([
                this.createWorkspaceService(payload, environmentService, fileService, remoteAgentService, logService).then(service => {
                    // Workspace
                    serviceCollection.set(workspace_1.IWorkspaceContextService, service);
                    // Configuration
                    serviceCollection.set(configuration_1.IConfigurationService, service);
                    return service;
                }),
                this.createStorageService(payload, environmentService, fileService, logService).then(service => {
                    // Storage
                    serviceCollection.set(storage_1.IStorageService, service);
                    return service;
                })
            ]);
            return { serviceCollection, logService, storageService: services[1] };
        });
    }
    registerFileSystemProviders(environmentService, fileService, remoteAgentService, logService, logsPath) {
        // Logger
        const indexedDBLogProvider = new indexedDBLogProvider_1.IndexedDBLogProvider(logsPath.scheme);
        (() => __awaiter(this, void 0, void 0, function* () {
            try {
                yield indexedDBLogProvider.database;
                fileService.registerProvider(logsPath.scheme, indexedDBLogProvider);
            }
            catch (error) {
                logService.info('Error while creating indexedDB log provider. Falling back to in-memory log provider.');
                logService.error(error);
                fileService.registerProvider(logsPath.scheme, new inMemoryLogProvider_1.InMemoryLogProvider(logsPath.scheme));
            }
            const consoleLogService = new log_1.ConsoleLogService(logService.getLevel());
            const fileLogService = new fileLogService_1.FileLogService('window', environmentService.logFile, logService.getLevel(), fileService);
            logService.logger = new log_1.MultiplexLogService([consoleLogService, fileLogService]);
        }))();
        const connection = remoteAgentService.getConnection();
        if (connection) {
            // Remote file system
            const channel = connection.getChannel(remoteAgentFileSystemChannel_1.REMOTE_FILE_SYSTEM_CHANNEL_NAME);
            const remoteFileSystemProvider = this._register(new remoteAgentFileSystemChannel_1.RemoteExtensionsFileSystemProvider(channel, remoteAgentService.getEnvironment()));
            fileService.registerProvider(network_1.Schemas.vscodeRemote, remoteFileSystemProvider);
            if (!this.configuration.userDataProvider) {
                const remoteUserDataUri = this.getRemoteUserDataUri();
                if (remoteUserDataUri) {
                    this.configuration.userDataProvider = this._register(new fileUserDataProvider_1.FileUserDataProvider(remoteUserDataUri, resources_1.joinPath(remoteUserDataUri, environment_1.BACKUPS), remoteFileSystemProvider, environmentService));
                }
            }
        }
        // User data
        if (!this.configuration.userDataProvider) {
            this.configuration.userDataProvider = this._register(new inMemoryUserDataProvider_1.InMemoryFileSystemProvider());
        }
        fileService.registerProvider(network_1.Schemas.userData, this.configuration.userDataProvider);
    }
    createStorageService(payload, environmentService, fileService, logService) {
        return __awaiter(this, void 0, void 0, function* () {
            const storageService = new storageService_1.BrowserStorageService(environmentService, fileService);
            try {
                yield storageService.initialize(payload);
                return storageService;
            }
            catch (error) {
                errors_1.onUnexpectedError(error);
                logService.error(error);
                return storageService;
            }
        });
    }
    createWorkspaceService(payload, environmentService, fileService, remoteAgentService, logService) {
        return __awaiter(this, void 0, void 0, function* () {
            const workspaceService = new configurationService_1.WorkspaceService({ remoteAuthority: this.configuration.remoteAuthority, configurationCache: new configurationCache_1.ConfigurationCache() }, environmentService, fileService, remoteAgentService);
            try {
                yield workspaceService.initialize(payload);
                return workspaceService;
            }
            catch (error) {
                errors_1.onUnexpectedError(error);
                logService.error(error);
                return workspaceService;
            }
        });
    }
    resolveWorkspaceInitializationPayload() {
        // Multi-root workspace
        if (this.configuration.workspaceUri) {
            return { id: hash_1.hash(this.configuration.workspaceUri.toString()).toString(16), configPath: this.configuration.workspaceUri };
        }
        // Single-folder workspace
        if (this.configuration.folderUri) {
            return { id: hash_1.hash(this.configuration.folderUri.toString()).toString(16), folder: this.configuration.folderUri };
        }
        return { id: 'empty-window' };
    }
    getRemoteUserDataUri() {
        const element = document.getElementById('vscode-remote-user-data-uri');
        if (element) {
            const remoteUserDataPath = element.getAttribute('data-settings');
            if (remoteUserDataPath) {
                return resources_1.joinPath(uri_1.URI.revive(JSON.parse(remoteUserDataPath)), 'User');
            }
        }
        return null;
    }
}
function main(domElement, options) {
    const renderer = new CodeRendererMain(domElement, options);
    return renderer.open();
}
exports.main = main;
//# sourceMappingURL=web.main.js.map