"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const arrays_1 = require("vs/base/common/arrays");
const glob = require("vs/base/common/glob");
const lifecycle_1 = require("vs/base/common/lifecycle");
const network_1 = require("vs/base/common/network");
const resources_1 = require("vs/base/common/resources");
const types_1 = require("vs/base/common/types");
const uuid_1 = require("vs/base/common/uuid");
const nls = require("vs/nls");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const quickInput_1 = require("vs/platform/quickinput/common/quickInput");
const editor_1 = require("vs/workbench/common/editor");
const diffEditorInput_1 = require("vs/workbench/common/editor/diffEditorInput");
const extensionPoint_1 = require("vs/workbench/contrib/customEditor/browser/extensionPoint");
const customEditor_1 = require("vs/workbench/contrib/customEditor/common/customEditor");
const fileEditorInput_1 = require("vs/workbench/contrib/files/common/editors/fileEditorInput");
const webview_1 = require("vs/workbench/contrib/webview/browser/webview");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const customEditorInput_1 = require("./customEditorInput");
const defaultEditorId = 'default';
const defaultEditorInfo = {
    id: defaultEditorId,
    displayName: nls.localize('promptOpenWith.defaultEditor', "Default built-in editor"),
    selector: [
        { filenamePattern: '*' }
    ],
    discretion: "default" /* default */,
};
class CustomEditorStore {
    constructor() {
        this.contributedEditors = new Map();
    }
    clear() {
        this.contributedEditors.clear();
    }
    get(viewType) {
        return viewType === defaultEditorId
            ? defaultEditorInfo
            : this.contributedEditors.get(viewType);
    }
    add(info) {
        if (info.id === defaultEditorId || this.contributedEditors.has(info.id)) {
            console.log(`Custom editor with id '${info.id}' already registered`);
            return;
        }
        this.contributedEditors.set(info.id, info);
    }
    getContributedEditors(resource) {
        return Array.from(this.contributedEditors.values()).filter(customEditor => customEditor.selector.some(selector => matches(selector, resource)));
    }
}
exports.CustomEditorStore = CustomEditorStore;
let CustomEditorService = class CustomEditorService {
    constructor(configurationService, editorService, instantiationService, quickInputService, webviewService) {
        this.configurationService = configurationService;
        this.editorService = editorService;
        this.instantiationService = instantiationService;
        this.quickInputService = quickInputService;
        this.webviewService = webviewService;
        this.editors = new CustomEditorStore();
        extensionPoint_1.webviewEditorsExtensionPoint.setHandler(extensions => {
            this.editors.clear();
            for (const extension of extensions) {
                for (const webviewEditorContribution of extension.value) {
                    this.editors.add({
                        id: webviewEditorContribution.viewType,
                        displayName: webviewEditorContribution.displayName,
                        selector: webviewEditorContribution.selector || [],
                        discretion: webviewEditorContribution.discretion || "default" /* default */,
                    });
                }
            }
        });
    }
    getContributedCustomEditors(resource) {
        return this.editors.getContributedEditors(resource);
    }
    getUserConfiguredCustomEditors(resource) {
        const rawAssociations = this.configurationService.getValue(exports.customEditorsAssociationsKey) || [];
        return arrays_1.coalesce(rawAssociations
            .filter(association => matches(association, resource))
            .map(association => this.editors.get(association.viewType)));
    }
    promptOpenWith(resource, options, group) {
        return __awaiter(this, void 0, void 0, function* () {
            const customEditors = arrays_1.distinct([
                defaultEditorInfo,
                ...this.getUserConfiguredCustomEditors(resource),
                ...this.getContributedCustomEditors(resource),
            ], editor => editor.id);
            const pick = yield this.quickInputService.pick(customEditors.map((editorDescriptor) => ({
                label: editorDescriptor.displayName,
                id: editorDescriptor.id,
            })), {
                placeHolder: nls.localize('promptOpenWith.placeHolder', "Select editor to use for '{0}'...", resources_1.basename(resource)),
            });
            if (!pick || !pick.id) {
                return;
            }
            return this.openWith(resource, pick.id, options, group);
        });
    }
    openWith(resource, viewType, options, group) {
        if (viewType === defaultEditorId) {
            const fileInput = this.instantiationService.createInstance(fileEditorInput_1.FileEditorInput, resource, undefined, undefined);
            return this.openEditorForResource(resource, fileInput, Object.assign(Object.assign({}, options), { ignoreOverrides: true }), group);
        }
        if (!this.editors.get(viewType)) {
            return this.promptOpenWith(resource, options, group);
        }
        const input = this.createInput(resource, viewType, group);
        return this.openEditorForResource(resource, input, options, group);
    }
    createInput(resource, viewType, group) {
        const id = uuid_1.generateUuid();
        const webview = this.webviewService.createWebviewEditorOverlay(id, {}, {});
        const input = this.instantiationService.createInstance(customEditorInput_1.CustomFileEditorInput, resource, viewType, id, new lifecycle_1.UnownedDisposable(webview));
        if (group) {
            input.updateGroup(group.id);
        }
        return input;
    }
    openEditorForResource(resource, input, options, group) {
        return __awaiter(this, void 0, void 0, function* () {
            if (group) {
                const existingEditors = group.editors.filter(editor => editor.getResource() && editor.getResource().toString() === resource.toString());
                if (existingEditors.length) {
                    yield this.editorService.replaceEditors([{
                            editor: existingEditors[0],
                            replacement: input,
                            options: options ? editor_1.EditorOptions.create(options) : undefined,
                        }], group);
                }
            }
            return this.editorService.openEditor(input, options, group);
        });
    }
};
CustomEditorService = __decorate([
    __param(0, configuration_1.IConfigurationService),
    __param(1, editorService_1.IEditorService),
    __param(2, instantiation_1.IInstantiationService),
    __param(3, quickInput_1.IQuickInputService),
    __param(4, webview_1.IWebviewService)
], CustomEditorService);
exports.CustomEditorService = CustomEditorService;
exports.customEditorsAssociationsKey = 'workbench.experimental.editorAssociations';
let CustomEditorContribution = class CustomEditorContribution {
    constructor(editorService, customEditorService) {
        this.editorService = editorService;
        this.customEditorService = customEditorService;
        this.editorService.overrideOpenEditor((editor, options, group) => this.onEditorOpening(editor, options, group));
    }
    onEditorOpening(editor, options, group) {
        if (editor instanceof customEditorInput_1.CustomFileEditorInput) {
            return;
        }
        if (editor instanceof diffEditorInput_1.DiffEditorInput) {
            const getCustomEditorOverrideForSubInput = (subInput) => {
                if (subInput instanceof customEditorInput_1.CustomFileEditorInput) {
                    return undefined;
                }
                const resource = subInput.getResource();
                if (!resource) {
                    return undefined;
                }
                const editors = arrays_1.distinct([
                    ...this.customEditorService.getUserConfiguredCustomEditors(resource),
                    ...this.customEditorService.getContributedCustomEditors(resource),
                ], editor => editor.id);
                // Always prefer the first editor in the diff editor case
                return editors.length
                    ? this.customEditorService.createInput(resource, editors[0].id, group)
                    : undefined;
            };
            const modifiedOverride = getCustomEditorOverrideForSubInput(editor.modifiedInput);
            const originalOverride = getCustomEditorOverrideForSubInput(editor.originalInput);
            if (modifiedOverride || originalOverride) {
                return {
                    override: (() => __awaiter(this, void 0, void 0, function* () {
                        const input = new diffEditorInput_1.DiffEditorInput(editor.getName(), editor.getDescription(), originalOverride || editor.originalInput, modifiedOverride || editor.modifiedInput);
                        return this.editorService.openEditor(input, Object.assign(Object.assign({}, options), { ignoreOverrides: true }), group);
                    }))(),
                };
            }
            return undefined;
        }
        const resource = editor.getResource();
        if (!resource) {
            return;
        }
        const userConfiguredEditors = this.customEditorService.getUserConfiguredCustomEditors(resource);
        const contributedEditors = this.customEditorService.getContributedCustomEditors(resource);
        if (!userConfiguredEditors.length) {
            if (!contributedEditors.length) {
                return;
            }
            const defaultEditors = contributedEditors.filter(editor => editor.discretion === "default" /* default */);
            if (defaultEditors.length === 1) {
                return {
                    override: this.customEditorService.openWith(resource, defaultEditors[0].id, options, group),
                };
            }
        }
        for (const input of group.editors) {
            if (input instanceof customEditorInput_1.CustomFileEditorInput && input.getResource().toString() === resource.toString()) {
                return {
                    override: group.openEditor(input, options).then(types_1.withNullAsUndefined)
                };
            }
        }
        if (userConfiguredEditors.length) {
            return {
                override: this.customEditorService.openWith(resource, userConfiguredEditors[0].id, options, group),
            };
        }
        // Open default editor but prompt user to see if they wish to use a custom one instead
        return {
            override: (() => __awaiter(this, void 0, void 0, function* () {
                const standardEditor = yield this.editorService.openEditor(editor, Object.assign(Object.assign({}, options), { ignoreOverrides: true }), group);
                const selectedEditor = yield this.customEditorService.promptOpenWith(resource, options, group);
                if (selectedEditor && selectedEditor.input) {
                    yield group.replaceEditors([{
                            editor,
                            replacement: selectedEditor.input
                        }]);
                    return selectedEditor;
                }
                return standardEditor;
            }))()
        };
    }
};
CustomEditorContribution = __decorate([
    __param(0, editorService_1.IEditorService),
    __param(1, customEditor_1.ICustomEditorService)
], CustomEditorContribution);
exports.CustomEditorContribution = CustomEditorContribution;
function matches(selector, resource) {
    if (resource.scheme === network_1.Schemas.data) {
        const metadata = resources_1.DataUri.parseMetaData(resource);
        const mime = metadata.get(resources_1.DataUri.META_DATA_MIME);
        if (!selector.mime || !mime) {
            return false;
        }
        return glob.match(selector.mime, mime.toLowerCase());
    }
    if (!selector.filenamePattern && !selector.scheme) {
        return false;
    }
    if (selector.filenamePattern) {
        if (!glob.match(selector.filenamePattern.toLowerCase(), resources_1.basename(resource).toLowerCase())) {
            return false;
        }
    }
    if (selector.scheme) {
        if (resource.scheme !== selector.scheme) {
            return false;
        }
    }
    return true;
}
//# sourceMappingURL=customEditors.js.map