"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const uri_1 = require("vs/base/common/uri");
const path_1 = require("vs/base/common/path");
const resources = require("vs/base/common/resources");
const debug_1 = require("vs/workbench/contrib/debug/common/debug");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const network_1 = require("vs/base/common/network");
const debugUtils_1 = require("vs/workbench/contrib/debug/common/debugUtils");
exports.UNKNOWN_SOURCE_LABEL = nls.localize('unknownSource', "Unknown Source");
/**
 * Debug URI format
 *
 * a debug URI represents a Source object and the debug session where the Source comes from.
 *
 *       debug:arbitrary_path?session=123e4567-e89b-12d3-a456-426655440000&ref=1016
 *       \___/ \____________/ \__________________________________________/ \______/
 *         |          |                             |                          |
 *      scheme   source.path                    session id            source.reference
 *
 *
 */
class Source {
    constructor(raw_, sessionId) {
        let path;
        if (raw_) {
            this.raw = raw_;
            path = this.raw.path || this.raw.name || '';
            this.available = true;
        }
        else {
            this.raw = { name: exports.UNKNOWN_SOURCE_LABEL };
            this.available = false;
            path = `${debug_1.DEBUG_SCHEME}:${exports.UNKNOWN_SOURCE_LABEL}`;
        }
        if (typeof this.raw.sourceReference === 'number' && this.raw.sourceReference > 0) {
            this.uri = uri_1.URI.from({
                scheme: debug_1.DEBUG_SCHEME,
                path,
                query: `session=${sessionId}&ref=${this.raw.sourceReference}`
            });
        }
        else {
            if (debugUtils_1.isUri(path)) { // path looks like a uri
                this.uri = uri_1.URI.parse(path);
            }
            else {
                // assume a filesystem path
                if (path_1.isAbsolute(path)) {
                    this.uri = uri_1.URI.file(path);
                }
                else {
                    // path is relative: since VS Code cannot deal with this by itself
                    // create a debug url that will result in a DAP 'source' request when the url is resolved.
                    this.uri = uri_1.URI.from({
                        scheme: debug_1.DEBUG_SCHEME,
                        path,
                        query: `session=${sessionId}`
                    });
                }
            }
        }
    }
    get name() {
        return this.raw.name || resources.basenameOrAuthority(this.uri);
    }
    get origin() {
        return this.raw.origin;
    }
    get presentationHint() {
        return this.raw.presentationHint;
    }
    get reference() {
        return this.raw.sourceReference;
    }
    get inMemory() {
        return this.uri.scheme === debug_1.DEBUG_SCHEME;
    }
    openInEditor(editorService, selection, preserveFocus, sideBySide, pinned) {
        return !this.available ? Promise.resolve(undefined) : editorService.openEditor({
            resource: this.uri,
            description: this.origin,
            options: {
                preserveFocus,
                selection,
                revealIfOpened: true,
                revealInCenterIfOutsideViewport: true,
                pinned: pinned || (!preserveFocus && !this.inMemory)
            }
        }, sideBySide ? editorService_1.SIDE_GROUP : editorService_1.ACTIVE_GROUP);
    }
    static getEncodedDebugData(modelUri) {
        let path;
        let sourceReference;
        let sessionId;
        switch (modelUri.scheme) {
            case network_1.Schemas.file:
                path = path_1.normalize(modelUri.fsPath);
                break;
            case debug_1.DEBUG_SCHEME:
                path = modelUri.path;
                if (modelUri.query) {
                    const keyvalues = modelUri.query.split('&');
                    for (let keyvalue of keyvalues) {
                        const pair = keyvalue.split('=');
                        if (pair.length === 2) {
                            switch (pair[0]) {
                                case 'session':
                                    sessionId = pair[1];
                                    break;
                                case 'ref':
                                    sourceReference = parseInt(pair[1]);
                                    break;
                            }
                        }
                    }
                }
                break;
            default:
                path = modelUri.toString();
                break;
        }
        return {
            name: resources.basenameOrAuthority(modelUri),
            path,
            sourceReference,
            sessionId
        };
    }
}
exports.Source = Source;
//# sourceMappingURL=debugSource.js.map