"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const async_1 = require("vs/base/common/async");
const cancellation_1 = require("vs/base/common/cancellation");
const errors = require("vs/base/common/errors");
const event_1 = require("vs/base/common/event");
const labels_1 = require("vs/base/common/labels");
const lifecycle_1 = require("vs/base/common/lifecycle");
const map_1 = require("vs/base/common/map");
const objects = require("vs/base/common/objects");
const strings_1 = require("vs/base/common/strings");
const uri_1 = require("vs/base/common/uri");
const range_1 = require("vs/editor/common/core/range");
const model_1 = require("vs/editor/common/model");
const textModel_1 = require("vs/editor/common/model/textModel");
const modelService_1 = require("vs/editor/common/services/modelService");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const replace_1 = require("vs/workbench/services/search/common/replace");
const search_1 = require("vs/workbench/services/search/common/search");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const themeService_1 = require("vs/platform/theme/common/themeService");
const replace_2 = require("vs/workbench/contrib/search/common/replace");
const searchHelpers_1 = require("vs/workbench/services/search/common/searchHelpers");
const types_1 = require("vs/base/common/types");
const decorators_1 = require("vs/base/common/decorators");
class Match {
    constructor(_parent, _fullPreviewLines, _fullPreviewRange, _documentRange) {
        this._parent = _parent;
        this._fullPreviewLines = _fullPreviewLines;
        this._oneLinePreviewText = _fullPreviewLines[_fullPreviewRange.startLineNumber];
        const adjustedEndCol = _fullPreviewRange.startLineNumber === _fullPreviewRange.endLineNumber ?
            _fullPreviewRange.endColumn :
            this._oneLinePreviewText.length;
        this._rangeInPreviewText = new search_1.OneLineRange(1, _fullPreviewRange.startColumn + 1, adjustedEndCol + 1);
        this._range = new range_1.Range(_documentRange.startLineNumber + 1, _documentRange.startColumn + 1, _documentRange.endLineNumber + 1, _documentRange.endColumn + 1);
        this._fullPreviewRange = _fullPreviewRange;
        this._id = this._parent.id() + '>' + this._range + this.getMatchString();
    }
    id() {
        return this._id;
    }
    parent() {
        return this._parent;
    }
    text() {
        return this._oneLinePreviewText;
    }
    range() {
        return this._range;
    }
    preview() {
        let before = this._oneLinePreviewText.substring(0, this._rangeInPreviewText.startColumn - 1), inside = this.getMatchString(), after = this._oneLinePreviewText.substring(this._rangeInPreviewText.endColumn - 1);
        before = strings_1.lcut(before, 26);
        before = before.trimLeft();
        let charsRemaining = Match.MAX_PREVIEW_CHARS - before.length;
        inside = inside.substr(0, charsRemaining);
        charsRemaining -= inside.length;
        after = after.substr(0, charsRemaining);
        return {
            before,
            inside,
            after,
        };
    }
    get replaceString() {
        const searchModel = this.parent().parent().searchModel;
        if (!searchModel.replacePattern) {
            throw new Error('searchModel.replacePattern must be set before accessing replaceString');
        }
        const fullMatchText = this.fullMatchText();
        let replaceString = searchModel.replacePattern.getReplaceString(fullMatchText, searchModel.preserveCase);
        // If match string is not matching then regex pattern has a lookahead expression
        if (replaceString === null) {
            const fullMatchTextWithTrailingContent = this.fullMatchText(true);
            replaceString = searchModel.replacePattern.getReplaceString(fullMatchTextWithTrailingContent, searchModel.preserveCase);
            // Search/find normalize line endings - check whether \r prevents regex from matching
            if (replaceString === null) {
                const fullMatchTextWithoutCR = fullMatchTextWithTrailingContent.replace(/\r\n/g, '\n');
                replaceString = searchModel.replacePattern.getReplaceString(fullMatchTextWithoutCR, searchModel.preserveCase);
            }
        }
        // Match string is still not matching. Could be unsupported matches (multi-line).
        if (replaceString === null) {
            replaceString = searchModel.replacePattern.pattern;
        }
        return replaceString;
    }
    fullMatchText(includeTrailing = false) {
        let thisMatchPreviewLines;
        if (includeTrailing) {
            thisMatchPreviewLines = this._fullPreviewLines.slice(this._fullPreviewRange.startLineNumber);
        }
        else {
            thisMatchPreviewLines = this._fullPreviewLines.slice(this._fullPreviewRange.startLineNumber, this._fullPreviewRange.endLineNumber + 1);
            thisMatchPreviewLines[thisMatchPreviewLines.length - 1] = thisMatchPreviewLines[thisMatchPreviewLines.length - 1].slice(0, this._fullPreviewRange.endColumn);
        }
        thisMatchPreviewLines[0] = thisMatchPreviewLines[0].slice(this._fullPreviewRange.startColumn);
        return thisMatchPreviewLines.join('\n');
    }
    fullPreviewLines() {
        return this._fullPreviewLines.slice(this._fullPreviewRange.startLineNumber, this._fullPreviewRange.endLineNumber + 1);
    }
    getMatchString() {
        return this._oneLinePreviewText.substring(this._rangeInPreviewText.startColumn - 1, this._rangeInPreviewText.endColumn - 1);
    }
}
Match.MAX_PREVIEW_CHARS = 250;
__decorate([
    decorators_1.memoize
], Match.prototype, "preview", null);
exports.Match = Match;
let FileMatch = class FileMatch extends lifecycle_1.Disposable {
    constructor(_query, _previewOptions, _maxResults, _parent, rawMatch, modelService, replaceService) {
        super();
        this._query = _query;
        this._previewOptions = _previewOptions;
        this._maxResults = _maxResults;
        this._parent = _parent;
        this.rawMatch = rawMatch;
        this.modelService = modelService;
        this.replaceService = replaceService;
        this._onChange = this._register(new event_1.Emitter());
        this.onChange = this._onChange.event;
        this._onDispose = this._register(new event_1.Emitter());
        this.onDispose = this._onDispose.event;
        this._model = null;
        this._modelListener = null;
        this._selectedMatch = null;
        this._modelDecorations = [];
        this._resource = this.rawMatch.resource;
        this._matches = new Map();
        this._removedMatches = new Set();
        this._updateScheduler = new async_1.RunOnceScheduler(this.updateMatchesForModel.bind(this), 250);
        this.createMatches();
    }
    static getDecorationOption(selected) {
        return (selected ? FileMatch._CURRENT_FIND_MATCH : FileMatch._FIND_MATCH);
    }
    createMatches() {
        const model = this.modelService.getModel(this._resource);
        if (model) {
            this.bindModel(model);
            this.updateMatchesForModel();
        }
        else {
            this.rawMatch.results
                .filter(search_1.resultIsMatch)
                .forEach(rawMatch => {
                textSearchResultToMatches(rawMatch, this)
                    .forEach(m => this.add(m));
            });
        }
    }
    bindModel(model) {
        this._model = model;
        this._modelListener = this._model.onDidChangeContent(() => {
            this._updateScheduler.schedule();
        });
        this._model.onWillDispose(() => this.onModelWillDispose());
        this.updateHighlights();
    }
    onModelWillDispose() {
        // Update matches because model might have some dirty changes
        this.updateMatchesForModel();
        this.unbindModel();
    }
    unbindModel() {
        if (this._model) {
            this._updateScheduler.cancel();
            this._model.deltaDecorations(this._modelDecorations, []);
            this._model = null;
            this._modelListener.dispose();
        }
    }
    updateMatchesForModel() {
        // this is called from a timeout and might fire
        // after the model has been disposed
        if (!this._model) {
            return;
        }
        this._matches = new Map();
        const wordSeparators = this._query.isWordMatch && this._query.wordSeparators ? this._query.wordSeparators : null;
        const matches = this._model
            .findMatches(this._query.pattern, this._model.getFullModelRange(), !!this._query.isRegExp, !!this._query.isCaseSensitive, wordSeparators, false, this._maxResults);
        this.updateMatches(matches, true);
    }
    updatesMatchesForLineAfterReplace(lineNumber, modelChange) {
        if (!this._model) {
            return;
        }
        const range = {
            startLineNumber: lineNumber,
            startColumn: this._model.getLineMinColumn(lineNumber),
            endLineNumber: lineNumber,
            endColumn: this._model.getLineMaxColumn(lineNumber)
        };
        const oldMatches = map_1.values(this._matches).filter(match => match.range().startLineNumber === lineNumber);
        oldMatches.forEach(match => this._matches.delete(match.id()));
        const wordSeparators = this._query.isWordMatch && this._query.wordSeparators ? this._query.wordSeparators : null;
        const matches = this._model.findMatches(this._query.pattern, range, !!this._query.isRegExp, !!this._query.isCaseSensitive, wordSeparators, false, this._maxResults);
        this.updateMatches(matches, modelChange);
    }
    updateMatches(matches, modelChange) {
        if (!this._model) {
            return;
        }
        const textSearchResults = searchHelpers_1.editorMatchesToTextSearchResults(matches, this._model, this._previewOptions);
        textSearchResults.forEach(textSearchResult => {
            textSearchResultToMatches(textSearchResult, this).forEach(match => {
                if (!this._removedMatches.has(match.id())) {
                    this.add(match);
                    if (this.isMatchSelected(match)) {
                        this._selectedMatch = match;
                    }
                }
            });
        });
        this._onChange.fire(modelChange);
        this.updateHighlights();
    }
    updateHighlights() {
        if (!this._model) {
            return;
        }
        if (this.parent().showHighlights) {
            this._modelDecorations = this._model.deltaDecorations(this._modelDecorations, this.matches().map(match => ({
                range: match.range(),
                options: FileMatch.getDecorationOption(this.isMatchSelected(match))
            })));
        }
        else {
            this._modelDecorations = this._model.deltaDecorations(this._modelDecorations, []);
        }
    }
    id() {
        return this.resource.toString();
    }
    parent() {
        return this._parent;
    }
    matches() {
        return map_1.values(this._matches);
    }
    remove(match) {
        this.removeMatch(match);
        this._removedMatches.add(match.id());
        this._onChange.fire(false);
    }
    replace(toReplace) {
        return this.replaceService.replace(toReplace)
            .then(() => this.updatesMatchesForLineAfterReplace(toReplace.range().startLineNumber, false));
    }
    setSelectedMatch(match) {
        if (match) {
            if (!this._matches.has(match.id())) {
                return;
            }
            if (this.isMatchSelected(match)) {
                return;
            }
        }
        this._selectedMatch = match;
        this.updateHighlights();
    }
    getSelectedMatch() {
        return this._selectedMatch;
    }
    isMatchSelected(match) {
        return !!this._selectedMatch && this._selectedMatch.id() === match.id();
    }
    count() {
        return this.matches().length;
    }
    get resource() {
        return this._resource;
    }
    name() {
        return labels_1.getBaseLabel(this.resource);
    }
    add(match, trigger) {
        this._matches.set(match.id(), match);
        if (trigger) {
            this._onChange.fire(true);
        }
    }
    removeMatch(match) {
        this._matches.delete(match.id());
        if (this.isMatchSelected(match)) {
            this.setSelectedMatch(null);
        }
        else {
            this.updateHighlights();
        }
    }
    dispose() {
        this.setSelectedMatch(null);
        this.unbindModel();
        this._onDispose.fire();
        super.dispose();
    }
};
FileMatch._CURRENT_FIND_MATCH = textModel_1.ModelDecorationOptions.register({
    stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
    zIndex: 13,
    className: 'currentFindMatch',
    overviewRuler: {
        color: themeService_1.themeColorFromId(colorRegistry_1.overviewRulerFindMatchForeground),
        position: model_1.OverviewRulerLane.Center
    },
    minimap: {
        color: themeService_1.themeColorFromId(colorRegistry_1.minimapFindMatch),
        position: model_1.MinimapPosition.Inline
    }
});
FileMatch._FIND_MATCH = textModel_1.ModelDecorationOptions.register({
    stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
    className: 'findMatch',
    overviewRuler: {
        color: themeService_1.themeColorFromId(colorRegistry_1.overviewRulerFindMatchForeground),
        position: model_1.OverviewRulerLane.Center
    },
    minimap: {
        color: themeService_1.themeColorFromId(colorRegistry_1.minimapFindMatch),
        position: model_1.MinimapPosition.Inline
    }
});
FileMatch = __decorate([
    __param(5, modelService_1.IModelService), __param(6, replace_2.IReplaceService)
], FileMatch);
exports.FileMatch = FileMatch;
let FolderMatch = class FolderMatch extends lifecycle_1.Disposable {
    constructor(_resource, _id, _index, _query, _parent, _searchModel, replaceService, instantiationService) {
        super();
        this._resource = _resource;
        this._id = _id;
        this._index = _index;
        this._query = _query;
        this._parent = _parent;
        this._searchModel = _searchModel;
        this.replaceService = replaceService;
        this.instantiationService = instantiationService;
        this._onChange = this._register(new event_1.Emitter());
        this.onChange = this._onChange.event;
        this._onDispose = this._register(new event_1.Emitter());
        this.onDispose = this._onDispose.event;
        this._replacingAll = false;
        this._fileMatches = new map_1.ResourceMap();
        this._unDisposedFileMatches = new map_1.ResourceMap();
    }
    get searchModel() {
        return this._searchModel;
    }
    get showHighlights() {
        return this._parent.showHighlights;
    }
    set replacingAll(b) {
        this._replacingAll = b;
    }
    id() {
        return this._id;
    }
    get resource() {
        return this._resource;
    }
    index() {
        return this._index;
    }
    name() {
        return labels_1.getBaseLabel(types_1.withNullAsUndefined(this.resource)) || '';
    }
    parent() {
        return this._parent;
    }
    bindModel(model) {
        const fileMatch = this._fileMatches.get(model.uri);
        if (fileMatch) {
            fileMatch.bindModel(model);
        }
    }
    add(raw, silent) {
        const added = [];
        const updated = [];
        raw.forEach(rawFileMatch => {
            const existingFileMatch = this._fileMatches.get(rawFileMatch.resource);
            if (existingFileMatch) {
                rawFileMatch
                    .results
                    .filter(search_1.resultIsMatch)
                    .forEach(m => {
                    textSearchResultToMatches(m, existingFileMatch)
                        .forEach(m => existingFileMatch.add(m));
                });
                updated.push(existingFileMatch);
            }
            else {
                const fileMatch = this.instantiationService.createInstance(FileMatch, this._query.contentPattern, this._query.previewOptions, this._query.maxResults, this, rawFileMatch);
                this.doAdd(fileMatch);
                added.push(fileMatch);
                const disposable = fileMatch.onChange(() => this.onFileChange(fileMatch));
                fileMatch.onDispose(() => disposable.dispose());
            }
        });
        const elements = [...added, ...updated];
        if (!silent && elements.length) {
            this._onChange.fire({ elements, added: !!added.length });
        }
    }
    clear() {
        const changed = this.matches();
        this.disposeMatches();
        this._onChange.fire({ elements: changed, removed: true });
    }
    remove(matches) {
        this.doRemove(matches);
    }
    replace(match) {
        return this.replaceService.replace([match]).then(() => {
            this.doRemove(match, false, true);
        });
    }
    replaceAll() {
        const matches = this.matches();
        return this.replaceService.replace(matches).then(() => {
            matches.forEach(match => this.doRemove(match, false, true));
        });
    }
    matches() {
        return this._fileMatches.values();
    }
    isEmpty() {
        return this.fileCount() === 0;
    }
    fileCount() {
        return this._fileMatches.size;
    }
    count() {
        return this.matches().reduce((prev, match) => prev + match.count(), 0);
    }
    onFileChange(fileMatch) {
        let added = false;
        let removed = false;
        if (!this._fileMatches.has(fileMatch.resource)) {
            this.doAdd(fileMatch);
            added = true;
        }
        if (fileMatch.count() === 0) {
            this.doRemove(fileMatch, false, false);
            added = false;
            removed = true;
        }
        if (!this._replacingAll) {
            this._onChange.fire({ elements: [fileMatch], added: added, removed: removed });
        }
    }
    doAdd(fileMatch) {
        this._fileMatches.set(fileMatch.resource, fileMatch);
        if (this._unDisposedFileMatches.has(fileMatch.resource)) {
            this._unDisposedFileMatches.delete(fileMatch.resource);
        }
    }
    doRemove(fileMatches, dispose = true, trigger = true) {
        if (!Array.isArray(fileMatches)) {
            fileMatches = [fileMatches];
        }
        for (let match of fileMatches) {
            this._fileMatches.delete(match.resource);
            if (dispose) {
                match.dispose();
            }
            else {
                this._unDisposedFileMatches.set(match.resource, match);
            }
        }
        if (trigger) {
            this._onChange.fire({ elements: fileMatches, removed: true });
        }
    }
    disposeMatches() {
        this._fileMatches.values().forEach((fileMatch) => fileMatch.dispose());
        this._unDisposedFileMatches.values().forEach((fileMatch) => fileMatch.dispose());
        this._fileMatches.clear();
        this._unDisposedFileMatches.clear();
    }
    dispose() {
        this.disposeMatches();
        this._onDispose.fire();
        super.dispose();
    }
};
FolderMatch = __decorate([
    __param(6, replace_2.IReplaceService),
    __param(7, instantiation_1.IInstantiationService)
], FolderMatch);
exports.FolderMatch = FolderMatch;
/**
 * BaseFolderMatch => optional resource ("other files" node)
 * FolderMatch => required resource (normal folder node)
 */
let FolderMatchWithResource = class FolderMatchWithResource extends FolderMatch {
    constructor(_resource, _id, _index, _query, _parent, _searchModel, replaceService, instantiationService) {
        super(_resource, _id, _index, _query, _parent, _searchModel, replaceService, instantiationService);
    }
    get resource() {
        return this._resource;
    }
};
FolderMatchWithResource = __decorate([
    __param(6, replace_2.IReplaceService),
    __param(7, instantiation_1.IInstantiationService)
], FolderMatchWithResource);
exports.FolderMatchWithResource = FolderMatchWithResource;
/**
 * Compares instances of the same match type. Different match types should not be siblings
 * and their sort order is undefined.
 */
function searchMatchComparer(elementA, elementB) {
    if (elementA instanceof FolderMatch && elementB instanceof FolderMatch) {
        return elementA.index() - elementB.index();
    }
    if (elementA instanceof FileMatch && elementB instanceof FileMatch) {
        return elementA.resource.fsPath.localeCompare(elementB.resource.fsPath) || elementA.name().localeCompare(elementB.name());
    }
    if (elementA instanceof Match && elementB instanceof Match) {
        return range_1.Range.compareRangesUsingStarts(elementA.range(), elementB.range());
    }
    return 0;
}
exports.searchMatchComparer = searchMatchComparer;
let SearchResult = class SearchResult extends lifecycle_1.Disposable {
    constructor(_searchModel, replaceService, telemetryService, instantiationService, modelService) {
        super();
        this._searchModel = _searchModel;
        this.replaceService = replaceService;
        this.telemetryService = telemetryService;
        this.instantiationService = instantiationService;
        this.modelService = modelService;
        this._onChange = this._register(new event_1.Emitter());
        this.onChange = this._onChange.event;
        this._folderMatches = [];
        this._otherFilesMatch = null;
        this._folderMatchesMap = map_1.TernarySearchTree.forPaths();
        this._showHighlights = false;
        this._query = null;
        this._rangeHighlightDecorations = this.instantiationService.createInstance(RangeHighlightDecorations);
        this._register(this.modelService.onModelAdded(model => this.onModelAdded(model)));
    }
    get query() {
        return this._query;
    }
    set query(query) {
        // When updating the query we could change the roots, so ensure we clean up the old roots first.
        this.clear();
        if (!query) {
            return;
        }
        this._folderMatches = (query && query.folderQueries || [])
            .map(fq => fq.folder)
            .map((resource, index) => this.createFolderMatchWithResource(resource, resource.toString(), index, query));
        this._folderMatches.forEach(fm => this._folderMatchesMap.set(fm.resource.toString(), fm));
        this._otherFilesMatch = this.createOtherFilesFolderMatch('otherFiles', this._folderMatches.length + 1, query);
        this._query = query;
    }
    onModelAdded(model) {
        const folderMatch = this._folderMatchesMap.findSubstr(model.uri.toString());
        if (folderMatch) {
            folderMatch.bindModel(model);
        }
    }
    createFolderMatchWithResource(resource, id, index, query) {
        return this._createBaseFolderMatch(FolderMatchWithResource, resource, id, index, query);
    }
    createOtherFilesFolderMatch(id, index, query) {
        return this._createBaseFolderMatch(FolderMatch, null, id, index, query);
    }
    _createBaseFolderMatch(folderMatchClass, resource, id, index, query) {
        const folderMatch = this.instantiationService.createInstance(folderMatchClass, resource, id, index, query, this, this._searchModel);
        const disposable = folderMatch.onChange((event) => this._onChange.fire(event));
        folderMatch.onDispose(() => disposable.dispose());
        return folderMatch;
    }
    get searchModel() {
        return this._searchModel;
    }
    add(allRaw, silent = false) {
        // Split up raw into a list per folder so we can do a batch add per folder.
        const { byFolder, other } = this.groupFilesByFolder(allRaw);
        byFolder.forEach(raw => {
            if (!raw.length) {
                return;
            }
            const folderMatch = this.getFolderMatch(raw[0].resource);
            if (folderMatch) {
                folderMatch.add(raw, silent);
            }
        });
        this._otherFilesMatch.add(other, silent);
    }
    clear() {
        this.folderMatches().forEach((folderMatch) => folderMatch.clear());
        this.disposeMatches();
        this._folderMatches = [];
        this._otherFilesMatch = null;
    }
    remove(matches) {
        if (!Array.isArray(matches)) {
            matches = [matches];
        }
        matches.forEach(m => {
            if (m instanceof FolderMatch) {
                m.clear();
            }
        });
        const fileMatches = matches.filter(m => m instanceof FileMatch);
        const { byFolder, other } = this.groupFilesByFolder(fileMatches);
        byFolder.forEach(matches => {
            if (!matches.length) {
                return;
            }
            this.getFolderMatch(matches[0].resource).remove(matches);
        });
        if (other.length) {
            this.getFolderMatch(other[0].resource).remove(other);
        }
    }
    replace(match) {
        return this.getFolderMatch(match.resource).replace(match);
    }
    replaceAll(progress) {
        this.replacingAll = true;
        const promise = this.replaceService.replace(this.matches(), progress);
        const onDone = event_1.Event.stopwatch(event_1.Event.fromPromise(promise));
        /* __GDPR__
            "replaceAll.started" : {
                "duration" : { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth", "isMeasurement": true }
            }
        */
        onDone(duration => this.telemetryService.publicLog('replaceAll.started', { duration }));
        return promise.then(() => {
            this.replacingAll = false;
            this.clear();
        }, () => {
            this.replacingAll = false;
        });
    }
    folderMatches() {
        return this._otherFilesMatch ?
            [
                ...this._folderMatches,
                this._otherFilesMatch
            ] :
            [
                ...this._folderMatches
            ];
    }
    matches() {
        const matches = [];
        this.folderMatches().forEach(folderMatch => {
            matches.push(folderMatch.matches());
        });
        return [].concat(...matches);
    }
    isEmpty() {
        return this.folderMatches().every((folderMatch) => folderMatch.isEmpty());
    }
    fileCount() {
        return this.folderMatches().reduce((prev, match) => prev + match.fileCount(), 0);
    }
    count() {
        return this.matches().reduce((prev, match) => prev + match.count(), 0);
    }
    get showHighlights() {
        return this._showHighlights;
    }
    toggleHighlights(value) {
        if (this._showHighlights === value) {
            return;
        }
        this._showHighlights = value;
        let selectedMatch = null;
        this.matches().forEach((fileMatch) => {
            fileMatch.updateHighlights();
            if (!selectedMatch) {
                selectedMatch = fileMatch.getSelectedMatch();
            }
        });
        if (this._showHighlights && selectedMatch) {
            // TS?
            this._rangeHighlightDecorations.highlightRange(selectedMatch.parent().resource, selectedMatch.range());
        }
        else {
            this._rangeHighlightDecorations.removeHighlightRange();
        }
    }
    get rangeHighlightDecorations() {
        return this._rangeHighlightDecorations;
    }
    getFolderMatch(resource) {
        const folderMatch = this._folderMatchesMap.findSubstr(resource.toString());
        return folderMatch ? folderMatch : this._otherFilesMatch;
    }
    set replacingAll(running) {
        this.folderMatches().forEach((folderMatch) => {
            folderMatch.replacingAll = running;
        });
    }
    groupFilesByFolder(fileMatches) {
        const rawPerFolder = new map_1.ResourceMap();
        const otherFileMatches = [];
        this._folderMatches.forEach(fm => rawPerFolder.set(fm.resource, []));
        fileMatches.forEach(rawFileMatch => {
            const folderMatch = this.getFolderMatch(rawFileMatch.resource);
            if (!folderMatch) {
                // foldermatch was previously removed by user or disposed for some reason
                return;
            }
            const resource = folderMatch.resource;
            if (resource) {
                rawPerFolder.get(resource).push(rawFileMatch);
            }
            else {
                otherFileMatches.push(rawFileMatch);
            }
        });
        return {
            byFolder: rawPerFolder,
            other: otherFileMatches
        };
    }
    disposeMatches() {
        this.folderMatches().forEach(folderMatch => folderMatch.dispose());
        this._folderMatches = [];
        this._folderMatchesMap = map_1.TernarySearchTree.forPaths();
        this._rangeHighlightDecorations.removeHighlightRange();
    }
    dispose() {
        this.disposeMatches();
        this._rangeHighlightDecorations.dispose();
        super.dispose();
    }
};
SearchResult = __decorate([
    __param(1, replace_2.IReplaceService),
    __param(2, telemetry_1.ITelemetryService),
    __param(3, instantiation_1.IInstantiationService),
    __param(4, modelService_1.IModelService)
], SearchResult);
exports.SearchResult = SearchResult;
let SearchModel = class SearchModel extends lifecycle_1.Disposable {
    constructor(searchService, telemetryService, instantiationService) {
        super();
        this.searchService = searchService;
        this.telemetryService = telemetryService;
        this.instantiationService = instantiationService;
        this._searchQuery = null;
        this._replaceActive = false;
        this._replaceString = null;
        this._replacePattern = null;
        this._preserveCase = false;
        this._onReplaceTermChanged = this._register(new event_1.Emitter());
        this.onReplaceTermChanged = this._onReplaceTermChanged.event;
        this.currentCancelTokenSource = null;
        this._searchResult = this.instantiationService.createInstance(SearchResult, this);
    }
    isReplaceActive() {
        return this._replaceActive;
    }
    set replaceActive(replaceActive) {
        this._replaceActive = replaceActive;
    }
    get replacePattern() {
        return this._replacePattern;
    }
    get replaceString() {
        return this._replaceString || '';
    }
    set preserveCase(value) {
        this._preserveCase = value;
    }
    get preserveCase() {
        return this._preserveCase;
    }
    set replaceString(replaceString) {
        this._replaceString = replaceString;
        if (this._searchQuery) {
            this._replacePattern = new replace_1.ReplacePattern(replaceString, this._searchQuery.contentPattern);
        }
        this._onReplaceTermChanged.fire();
    }
    get searchResult() {
        return this._searchResult;
    }
    search(query, onProgress) {
        this.cancelSearch();
        this._searchQuery = query;
        this.searchResult.clear();
        this._searchResult.query = this._searchQuery;
        const progressEmitter = new event_1.Emitter();
        this._replacePattern = new replace_1.ReplacePattern(this.replaceString, this._searchQuery.contentPattern);
        const tokenSource = this.currentCancelTokenSource = new cancellation_1.CancellationTokenSource();
        const currentRequest = this.searchService.textSearch(this._searchQuery, this.currentCancelTokenSource.token, p => {
            progressEmitter.fire();
            this.onSearchProgress(p);
            if (onProgress) {
                onProgress(p);
            }
        });
        const dispose = () => tokenSource.dispose();
        currentRequest.then(dispose, dispose);
        const onDone = event_1.Event.fromPromise(currentRequest);
        const onFirstRender = event_1.Event.any(onDone, progressEmitter.event);
        const onFirstRenderStopwatch = event_1.Event.stopwatch(onFirstRender);
        /* __GDPR__
            "searchResultsFirstRender" : {
                "duration" : { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth", "isMeasurement": true }
            }
        */
        onFirstRenderStopwatch(duration => this.telemetryService.publicLog('searchResultsFirstRender', { duration }));
        const start = Date.now();
        currentRequest.then(value => this.onSearchCompleted(value, Date.now() - start), e => this.onSearchError(e, Date.now() - start));
        return currentRequest.finally(() => {
            /* __GDPR__
                "searchResultsFinished" : {
                    "duration" : { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth", "isMeasurement": true }
                }
            */
            this.telemetryService.publicLog('searchResultsFinished', { duration: Date.now() - start });
        });
    }
    onSearchCompleted(completed, duration) {
        if (!this._searchQuery) {
            throw new Error('onSearchCompleted must be called after a search is started');
        }
        const options = objects.assign({}, this._searchQuery.contentPattern);
        delete options.pattern;
        const stats = completed && completed.stats;
        const fileSchemeOnly = this._searchQuery.folderQueries.every(fq => fq.folder.scheme === 'file');
        const otherSchemeOnly = this._searchQuery.folderQueries.every(fq => fq.folder.scheme !== 'file');
        const scheme = fileSchemeOnly ? 'file' :
            otherSchemeOnly ? 'other' :
                'mixed';
        /* __GDPR__
            "searchResultsShown" : {
                "count" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
                "fileCount": { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
                "options": { "${inline}": [ "${IPatternInfo}" ] },
                "duration": { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth", "isMeasurement": true },
                "type" : { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth" },
                "scheme" : { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth" }
            }
        */
        this.telemetryService.publicLog('searchResultsShown', {
            count: this._searchResult.count(),
            fileCount: this._searchResult.fileCount(),
            options,
            duration,
            type: stats && stats.type,
            scheme
        });
        return completed;
    }
    onSearchError(e, duration) {
        if (errors.isPromiseCanceledError(e)) {
            this.onSearchCompleted(null, duration);
        }
    }
    onSearchProgress(p) {
        if (p.resource) {
            this._searchResult.add([p], true);
        }
    }
    cancelSearch() {
        if (this.currentCancelTokenSource) {
            this.currentCancelTokenSource.cancel();
            return true;
        }
        return false;
    }
    dispose() {
        this.cancelSearch();
        this.searchResult.dispose();
        super.dispose();
    }
};
SearchModel = __decorate([
    __param(0, search_1.ISearchService),
    __param(1, telemetry_1.ITelemetryService),
    __param(2, instantiation_1.IInstantiationService)
], SearchModel);
exports.SearchModel = SearchModel;
let SearchWorkbenchService = class SearchWorkbenchService {
    constructor(instantiationService) {
        this.instantiationService = instantiationService;
        this._searchModel = null;
    }
    get searchModel() {
        if (!this._searchModel) {
            this._searchModel = this.instantiationService.createInstance(SearchModel);
        }
        return this._searchModel;
    }
};
SearchWorkbenchService = __decorate([
    __param(0, instantiation_1.IInstantiationService)
], SearchWorkbenchService);
exports.SearchWorkbenchService = SearchWorkbenchService;
exports.ISearchWorkbenchService = instantiation_1.createDecorator('searchWorkbenchService');
/**
 * Can add a range highlight decoration to a model.
 * It will automatically remove it when the model has its decorations changed.
 */
let RangeHighlightDecorations = class RangeHighlightDecorations {
    constructor(_modelService) {
        this._modelService = _modelService;
        this._decorationId = null;
        this._model = null;
        this._modelDisposables = new lifecycle_1.DisposableStore();
    }
    removeHighlightRange() {
        if (this._model && this._decorationId) {
            this._model.deltaDecorations([this._decorationId], []);
        }
        this._decorationId = null;
    }
    highlightRange(resource, range, ownerId = 0) {
        let model;
        if (uri_1.URI.isUri(resource)) {
            model = this._modelService.getModel(resource);
        }
        else {
            model = resource;
        }
        if (model) {
            this.doHighlightRange(model, range);
        }
    }
    doHighlightRange(model, range) {
        this.removeHighlightRange();
        this._decorationId = model.deltaDecorations([], [{ range: range, options: RangeHighlightDecorations._RANGE_HIGHLIGHT_DECORATION }])[0];
        this.setModel(model);
    }
    setModel(model) {
        if (this._model !== model) {
            this.clearModelListeners();
            this._model = model;
            this._modelDisposables.add(this._model.onDidChangeDecorations((e) => {
                this.clearModelListeners();
                this.removeHighlightRange();
                this._model = null;
            }));
            this._modelDisposables.add(this._model.onWillDispose(() => {
                this.clearModelListeners();
                this.removeHighlightRange();
                this._model = null;
            }));
        }
    }
    clearModelListeners() {
        this._modelDisposables.clear();
    }
    dispose() {
        if (this._model) {
            this.removeHighlightRange();
            this._modelDisposables.dispose();
            this._model = null;
        }
    }
};
RangeHighlightDecorations._RANGE_HIGHLIGHT_DECORATION = textModel_1.ModelDecorationOptions.register({
    stickiness: 1 /* NeverGrowsWhenTypingAtEdges */,
    className: 'rangeHighlight',
    isWholeLine: true
});
RangeHighlightDecorations = __decorate([
    __param(0, modelService_1.IModelService)
], RangeHighlightDecorations);
exports.RangeHighlightDecorations = RangeHighlightDecorations;
function textSearchResultToMatches(rawMatch, fileMatch) {
    const previewLines = rawMatch.preview.text.split('\n');
    if (Array.isArray(rawMatch.ranges)) {
        return rawMatch.ranges.map((r, i) => {
            const previewRange = rawMatch.preview.matches[i];
            return new Match(fileMatch, previewLines, previewRange, r);
        });
    }
    else {
        const previewRange = rawMatch.preview.matches;
        const match = new Match(fileMatch, previewLines, previewRange, rawMatch.ranges);
        return [match];
    }
}
//# sourceMappingURL=searchModel.js.map