"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const errors = require("vs/base/common/errors");
const objects = require("vs/base/common/objects");
const openFileHandler_1 = require("vs/workbench/contrib/search/browser/openFileHandler");
const utils_1 = require("vs/base/test/common/utils");
suite('CacheState', () => {
    test('reuse old cacheKey until new cache is loaded', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = new MockCache();
            const first = createCacheState(cache);
            const firstKey = first.cacheKey;
            assert.strictEqual(first.isLoaded, false);
            assert.strictEqual(first.isUpdating, false);
            first.load();
            assert.strictEqual(first.isLoaded, false);
            assert.strictEqual(first.isUpdating, true);
            yield cache.loading[firstKey].complete(null);
            assert.strictEqual(first.isLoaded, true);
            assert.strictEqual(first.isUpdating, false);
            const second = createCacheState(cache, first);
            second.load();
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, true);
            yield cache.awaitDisposal(0);
            assert.strictEqual(second.cacheKey, firstKey); // still using old cacheKey
            const secondKey = cache.cacheKeys[1];
            yield cache.loading[secondKey].complete(null);
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(1);
            assert.strictEqual(second.cacheKey, secondKey);
        });
    });
    test('do not spawn additional load if previous is still loading', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = new MockCache();
            const first = createCacheState(cache);
            const firstKey = first.cacheKey;
            first.load();
            assert.strictEqual(first.isLoaded, false);
            assert.strictEqual(first.isUpdating, true);
            assert.strictEqual(Object.keys(cache.loading).length, 1);
            const second = createCacheState(cache, first);
            second.load();
            assert.strictEqual(second.isLoaded, false);
            assert.strictEqual(second.isUpdating, true);
            assert.strictEqual(cache.cacheKeys.length, 2);
            assert.strictEqual(Object.keys(cache.loading).length, 1); // still only one loading
            assert.strictEqual(second.cacheKey, firstKey);
            yield cache.loading[firstKey].complete(null);
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(0);
        });
    });
    test('do not use previous cacheKey if query changed', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = new MockCache();
            const first = createCacheState(cache);
            const firstKey = first.cacheKey;
            first.load();
            yield cache.loading[firstKey].complete(null);
            assert.strictEqual(first.isLoaded, true);
            assert.strictEqual(first.isUpdating, false);
            yield cache.awaitDisposal(0);
            cache.baseQuery.excludePattern = { '**/node_modules': true };
            const second = createCacheState(cache, first);
            assert.strictEqual(second.isLoaded, false);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(1);
            second.load();
            assert.strictEqual(second.isLoaded, false);
            assert.strictEqual(second.isUpdating, true);
            assert.notStrictEqual(second.cacheKey, firstKey); // not using old cacheKey
            const secondKey = cache.cacheKeys[1];
            assert.strictEqual(second.cacheKey, secondKey);
            yield cache.loading[secondKey].complete(null);
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(1);
        });
    });
    test('dispose propagates', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = new MockCache();
            const first = createCacheState(cache);
            const firstKey = first.cacheKey;
            first.load();
            yield cache.loading[firstKey].complete(null);
            const second = createCacheState(cache, first);
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(0);
            second.dispose();
            assert.strictEqual(second.isLoaded, false);
            assert.strictEqual(second.isUpdating, false);
            yield cache.awaitDisposal(1);
            assert.ok(cache.disposing[firstKey]);
        });
    });
    test('keep using old cacheKey when loading fails', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const cache = new MockCache();
            const first = createCacheState(cache);
            const firstKey = first.cacheKey;
            first.load();
            yield cache.loading[firstKey].complete(null);
            const second = createCacheState(cache, first);
            second.load();
            const secondKey = cache.cacheKeys[1];
            const origErrorHandler = errors.errorHandler.getUnexpectedErrorHandler();
            try {
                errors.setUnexpectedErrorHandler(() => null);
                yield cache.loading[secondKey].error('loading failed');
            }
            finally {
                errors.setUnexpectedErrorHandler(origErrorHandler);
            }
            assert.strictEqual(second.isLoaded, true);
            assert.strictEqual(second.isUpdating, false);
            assert.strictEqual(Object.keys(cache.loading).length, 2);
            yield cache.awaitDisposal(0);
            assert.strictEqual(second.cacheKey, firstKey); // keep using old cacheKey
            const third = createCacheState(cache, second);
            third.load();
            assert.strictEqual(third.isLoaded, true);
            assert.strictEqual(third.isUpdating, true);
            assert.strictEqual(Object.keys(cache.loading).length, 3);
            yield cache.awaitDisposal(0);
            assert.strictEqual(third.cacheKey, firstKey);
            const thirdKey = cache.cacheKeys[2];
            yield cache.loading[thirdKey].complete(null);
            assert.strictEqual(third.isLoaded, true);
            assert.strictEqual(third.isUpdating, false);
            assert.strictEqual(Object.keys(cache.loading).length, 3);
            yield cache.awaitDisposal(2);
            assert.strictEqual(third.cacheKey, thirdKey); // recover with next successful load
        });
    });
    function createCacheState(cache, previous) {
        return new openFileHandler_1.CacheState(cacheKey => cache.query(cacheKey), query => cache.load(query), cacheKey => cache.dispose(cacheKey), previous);
    }
    class MockCache {
        constructor() {
            this.cacheKeys = [];
            this.loading = {};
            this.disposing = {};
            this._awaitDisposal = [];
            this.baseQuery = {
                type: 1 /* File */,
                folderQueries: []
            };
        }
        query(cacheKey) {
            this.cacheKeys.push(cacheKey);
            return objects.assign({ cacheKey: cacheKey }, this.baseQuery);
        }
        load(query) {
            const promise = new utils_1.DeferredPromise();
            this.loading[query.cacheKey] = promise;
            return promise.p;
        }
        dispose(cacheKey) {
            const promise = new utils_1.DeferredPromise();
            this.disposing[cacheKey] = promise;
            const n = Object.keys(this.disposing).length;
            for (const done of this._awaitDisposal[n] || []) {
                done();
            }
            delete this._awaitDisposal[n];
            return promise.p;
        }
        awaitDisposal(n) {
            return new Promise(resolve => {
                if (n === Object.keys(this.disposing).length) {
                    resolve();
                }
                else {
                    (this._awaitDisposal[n] || (this._awaitDisposal[n] = [])).push(resolve);
                }
            });
        }
    }
});
//# sourceMappingURL=openFileHandler.test.js.map