"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const snippets_contribution_1 = require("./snippets.contribution");
const snippetsService_1 = require("./snippetsService");
const strings_1 = require("vs/base/common/strings");
const lifecycle_1 = require("vs/base/common/lifecycle");
const range_1 = require("vs/editor/common/core/range");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const snippetController2_1 = require("vs/editor/contrib/snippet/snippetController2");
const suggest_1 = require("vs/editor/contrib/suggest/suggest");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const snippetCompletionProvider_1 = require("./snippetCompletionProvider");
let TabCompletionController = class TabCompletionController {
    constructor(_editor, _snippetService, contextKeyService) {
        this._editor = _editor;
        this._snippetService = _snippetService;
        this._activeSnippets = [];
        this._hasSnippets = TabCompletionController.ContextKey.bindTo(contextKeyService);
        this._configListener = this._editor.onDidChangeConfiguration(e => {
            if (e.hasChanged(88 /* tabCompletion */)) {
                this._update();
            }
        });
        this._update();
    }
    static get(editor) {
        return editor.getContribution(TabCompletionController.ID);
    }
    getId() {
        return TabCompletionController.ID;
    }
    dispose() {
        lifecycle_1.dispose(this._configListener);
        lifecycle_1.dispose(this._selectionListener);
    }
    _update() {
        const enabled = this._editor.getOption(88 /* tabCompletion */) === 'onlySnippets';
        if (this._enabled !== enabled) {
            this._enabled = enabled;
            if (!this._enabled) {
                lifecycle_1.dispose(this._selectionListener);
            }
            else {
                this._selectionListener = this._editor.onDidChangeCursorSelection(e => this._updateSnippets());
                if (this._editor.getModel()) {
                    this._updateSnippets();
                }
            }
        }
    }
    _updateSnippets() {
        // reset first
        this._activeSnippets = [];
        if (!this._editor.hasModel()) {
            return;
        }
        // lots of dance for getting the
        const selection = this._editor.getSelection();
        const model = this._editor.getModel();
        model.tokenizeIfCheap(selection.positionLineNumber);
        const id = model.getLanguageIdAtPosition(selection.positionLineNumber, selection.positionColumn);
        const snippets = this._snippetService.getSnippetsSync(id);
        if (!snippets) {
            // nothing for this language
            this._hasSnippets.set(false);
            return;
        }
        if (range_1.Range.isEmpty(selection)) {
            // empty selection -> real text (no whitespace) left of cursor
            const prefix = snippetsService_1.getNonWhitespacePrefix(model, selection.getPosition());
            if (prefix) {
                for (const snippet of snippets) {
                    if (strings_1.endsWith(prefix, snippet.prefix)) {
                        this._activeSnippets.push(snippet);
                    }
                }
            }
        }
        else if (!range_1.Range.spansMultipleLines(selection) && model.getValueLengthInRange(selection) <= 100) {
            // actual selection -> snippet must be a full match
            const selected = model.getValueInRange(selection);
            if (selected) {
                for (const snippet of snippets) {
                    if (selected === snippet.prefix) {
                        this._activeSnippets.push(snippet);
                    }
                }
            }
        }
        this._hasSnippets.set(this._activeSnippets.length > 0);
    }
    performSnippetCompletions() {
        if (!this._editor.hasModel()) {
            return;
        }
        if (this._activeSnippets.length === 1) {
            // one -> just insert
            const [snippet] = this._activeSnippets;
            snippetController2_1.SnippetController2.get(this._editor).insert(snippet.codeSnippet, { overwriteBefore: snippet.prefix.length, overwriteAfter: 0 });
        }
        else if (this._activeSnippets.length > 1) {
            // two or more -> show IntelliSense box
            const position = this._editor.getPosition();
            suggest_1.showSimpleSuggestions(this._editor, this._activeSnippets.map(snippet => {
                const range = range_1.Range.fromPositions(position.delta(0, -snippet.prefix.length), position);
                return new snippetCompletionProvider_1.SnippetCompletion(snippet, range);
            }));
        }
    }
};
TabCompletionController.ID = 'editor.tabCompletionController';
TabCompletionController.ContextKey = new contextkey_1.RawContextKey('hasSnippetCompletions', undefined);
TabCompletionController = __decorate([
    __param(1, snippets_contribution_1.ISnippetsService),
    __param(2, contextkey_1.IContextKeyService)
], TabCompletionController);
exports.TabCompletionController = TabCompletionController;
editorExtensions_1.registerEditorContribution(TabCompletionController);
const TabCompletionCommand = editorExtensions_1.EditorCommand.bindToContribution(TabCompletionController.get);
editorExtensions_1.registerEditorCommand(new TabCompletionCommand({
    id: 'insertSnippet',
    precondition: TabCompletionController.ContextKey,
    handler: x => x.performSnippetCompletions(),
    kbOpts: {
        weight: 100 /* EditorContrib */,
        kbExpr: contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.editorTextFocus, editorContextKeys_1.EditorContextKeys.tabDoesNotMoveFocus, snippetController2_1.SnippetController2.InSnippetMode.toNegated()),
        primary: 2 /* Tab */
    }
}));
//# sourceMappingURL=tabCompletion.js.map