"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const gracefulFs = require("graceful-fs");
const crypto_1 = require("crypto");
const performance_1 = require("vs/base/common/performance");
const workbench_1 = require("vs/workbench/browser/workbench");
const window_1 = require("vs/workbench/electron-browser/window");
const browser_1 = require("vs/base/browser/browser");
const dom_1 = require("vs/base/browser/dom");
const errors_1 = require("vs/base/common/errors");
const platform_1 = require("vs/base/common/platform");
const uri_1 = require("vs/base/common/uri");
const configurationService_1 = require("vs/workbench/services/configuration/browser/configurationService");
const environmentService_1 = require("vs/workbench/services/environment/node/environmentService");
const environmentService_2 = require("vs/workbench/services/environment/common/environmentService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const pfs_1 = require("vs/base/node/pfs");
const nativeKeymapService_1 = require("vs/workbench/services/keybinding/electron-browser/nativeKeymapService");
const electron_1 = require("electron");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const log_1 = require("vs/platform/log/common/log");
const storageService_1 = require("vs/platform/storage/node/storageService");
const logIpc_1 = require("vs/platform/log/common/logIpc");
const network_1 = require("vs/base/common/network");
const extpath_1 = require("vs/base/common/extpath");
const storageIpc_1 = require("vs/platform/storage/node/storageIpc");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const storage_1 = require("vs/platform/storage/common/storage");
const lifecycle_1 = require("vs/base/common/lifecycle");
const driver_1 = require("vs/platform/driver/electron-browser/driver");
const mainProcessService_1 = require("vs/platform/ipc/electron-browser/mainProcessService");
const remoteAuthorityResolverService_1 = require("vs/platform/remote/electron-browser/remoteAuthorityResolverService");
const remoteAuthorityResolver_1 = require("vs/platform/remote/common/remoteAuthorityResolver");
const remoteAgentServiceImpl_1 = require("vs/workbench/services/remote/electron-browser/remoteAgentServiceImpl");
const remoteAgentService_1 = require("vs/workbench/services/remote/common/remoteAgentService");
const fileService_1 = require("vs/platform/files/common/fileService");
const files_1 = require("vs/platform/files/common/files");
const diskFileSystemProvider_1 = require("vs/platform/files/electron-browser/diskFileSystemProvider");
const remoteAgentFileSystemChannel_1 = require("vs/platform/remote/common/remoteAgentFileSystemChannel");
const configurationExportHelper_1 = require("vs/workbench/services/configuration/node/configurationExportHelper");
const configurationCache_1 = require("vs/workbench/services/configuration/node/configurationCache");
const spdlogService_1 = require("vs/platform/log/node/spdlogService");
const signService_1 = require("vs/platform/sign/node/signService");
const sign_1 = require("vs/platform/sign/common/sign");
const fileUserDataProvider_1 = require("vs/workbench/services/userData/common/fileUserDataProvider");
const resources_1 = require("vs/base/common/resources");
const productService_1 = require("vs/platform/product/common/productService");
const product_1 = require("vs/platform/product/common/product");
class CodeRendererMain extends lifecycle_1.Disposable {
    constructor(configuration) {
        super();
        this.environmentService = new environmentService_1.WorkbenchEnvironmentService(configuration, configuration.execPath);
        this.init();
    }
    init() {
        // Enable gracefulFs
        gracefulFs.gracefulify(fs);
        // Massage configuration file URIs
        this.reviveUris();
        // Setup perf
        performance_1.importEntries(this.environmentService.configuration.perfEntries);
        // Browser config
        browser_1.setZoomFactor(electron_1.webFrame.getZoomFactor()); // Ensure others can listen to zoom level changes
        browser_1.setZoomLevel(electron_1.webFrame.getZoomLevel(), true /* isTrusted */); // Can be trusted because we are not setting it ourselves (https://github.com/Microsoft/vscode/issues/26151)
        browser_1.setFullscreen(!!this.environmentService.configuration.fullscreen);
        // Keyboard support
        nativeKeymapService_1.KeyboardMapperFactory.INSTANCE._onKeyboardLayoutChanged();
    }
    reviveUris() {
        if (this.environmentService.configuration.folderUri) {
            this.environmentService.configuration.folderUri = uri_1.URI.revive(this.environmentService.configuration.folderUri);
        }
        if (this.environmentService.configuration.workspace) {
            this.environmentService.configuration.workspace = workspaces_1.reviveWorkspaceIdentifier(this.environmentService.configuration.workspace);
        }
        const filesToWait = this.environmentService.configuration.filesToWait;
        const filesToWaitPaths = filesToWait && filesToWait.paths;
        [filesToWaitPaths, this.environmentService.configuration.filesToOpenOrCreate, this.environmentService.configuration.filesToDiff].forEach(paths => {
            if (Array.isArray(paths)) {
                paths.forEach(path => {
                    if (path.fileUri) {
                        path.fileUri = uri_1.URI.revive(path.fileUri);
                    }
                });
            }
        });
        if (filesToWait) {
            filesToWait.waitMarkerFileUri = uri_1.URI.revive(filesToWait.waitMarkerFileUri);
        }
    }
    open() {
        return __awaiter(this, void 0, void 0, function* () {
            const services = yield this.initServices();
            yield dom_1.domContentLoaded();
            performance_1.mark('willStartWorkbench');
            // Create Workbench
            const workbench = new workbench_1.Workbench(document.body, services.serviceCollection, services.logService);
            // Layout
            this._register(dom_1.addDisposableListener(window, dom_1.EventType.RESIZE, e => this.onWindowResize(e, true, workbench)));
            // Workbench Lifecycle
            this._register(workbench.onShutdown(() => this.dispose()));
            this._register(workbench.onWillShutdown(event => event.join(services.storageService.close())));
            // Startup
            const instantiationService = workbench.startup();
            // Window
            this._register(instantiationService.createInstance(window_1.ElectronWindow));
            // Driver
            if (this.environmentService.configuration.driver) {
                instantiationService.invokeFunction((accessor) => __awaiter(this, void 0, void 0, function* () { return this._register(yield driver_1.registerWindowDriver(accessor)); }));
            }
            // Config Exporter
            if (this.environmentService.configuration['export-default-configuration']) {
                instantiationService.createInstance(configurationExportHelper_1.DefaultConfigurationExportHelper);
            }
            // Logging
            services.logService.trace('workbench configuration', JSON.stringify(this.environmentService.configuration));
        });
    }
    onWindowResize(e, retry, workbench) {
        if (e.target === window) {
            if (window.document && window.document.body && window.document.body.clientWidth === 0) {
                // TODO@Ben this is an electron issue on macOS when simple fullscreen is enabled
                // where for some reason the window clientWidth is reported as 0 when switching
                // between simple fullscreen and normal screen. In that case we schedule the layout
                // call at the next animation frame once, in the hope that the dimensions are
                // proper then.
                if (retry) {
                    dom_1.scheduleAtNextAnimationFrame(() => this.onWindowResize(e, false, workbench));
                }
                return;
            }
            workbench.layout();
        }
    }
    initServices() {
        return __awaiter(this, void 0, void 0, function* () {
            const serviceCollection = new serviceCollection_1.ServiceCollection();
            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // NOTE: DO NOT ADD ANY OTHER SERVICE INTO THE COLLECTION HERE.
            // CONTRIBUTE IT VIA WORKBENCH.DESKTOP.MAIN.TS AND registerSingleton().
            // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // Main Process
            const mainProcessService = this._register(new mainProcessService_1.MainProcessService(this.environmentService.configuration.windowId));
            serviceCollection.set(mainProcessService_1.IMainProcessService, mainProcessService);
            // Environment
            serviceCollection.set(environmentService_2.IWorkbenchEnvironmentService, this.environmentService);
            // Product
            serviceCollection.set(productService_1.IProductService, Object.assign({ _serviceBrand: undefined }, product_1.default));
            // Log
            const logService = this._register(this.createLogService(mainProcessService, this.environmentService));
            serviceCollection.set(log_1.ILogService, logService);
            // Remote
            const remoteAuthorityResolverService = new remoteAuthorityResolverService_1.RemoteAuthorityResolverService();
            serviceCollection.set(remoteAuthorityResolver_1.IRemoteAuthorityResolverService, remoteAuthorityResolverService);
            // Sign
            const signService = new signService_1.SignService();
            serviceCollection.set(sign_1.ISignService, signService);
            const remoteAgentService = this._register(new remoteAgentServiceImpl_1.RemoteAgentService(this.environmentService.configuration, this.environmentService, remoteAuthorityResolverService, signService, logService));
            serviceCollection.set(remoteAgentService_1.IRemoteAgentService, remoteAgentService);
            // Files
            const fileService = this._register(new fileService_1.FileService(logService));
            serviceCollection.set(files_1.IFileService, fileService);
            const diskFileSystemProvider = this._register(new diskFileSystemProvider_1.DiskFileSystemProvider(logService));
            fileService.registerProvider(network_1.Schemas.file, diskFileSystemProvider);
            // User Data Provider
            fileService.registerProvider(network_1.Schemas.userData, new fileUserDataProvider_1.FileUserDataProvider(this.environmentService.appSettingsHome, this.environmentService.backupHome, diskFileSystemProvider, this.environmentService));
            const connection = remoteAgentService.getConnection();
            if (connection) {
                const channel = connection.getChannel(remoteAgentFileSystemChannel_1.REMOTE_FILE_SYSTEM_CHANNEL_NAME);
                const remoteFileSystemProvider = this._register(new remoteAgentFileSystemChannel_1.RemoteExtensionsFileSystemProvider(channel, remoteAgentService.getEnvironment()));
                fileService.registerProvider(network_1.Schemas.vscodeRemote, remoteFileSystemProvider);
            }
            const payload = yield this.resolveWorkspaceInitializationPayload();
            const services = yield Promise.all([
                this.createWorkspaceService(payload, fileService, remoteAgentService, logService).then(service => {
                    // Workspace
                    serviceCollection.set(workspace_1.IWorkspaceContextService, service);
                    // Configuration
                    serviceCollection.set(configuration_1.IConfigurationService, service);
                    return service;
                }),
                this.createStorageService(payload, logService, mainProcessService).then(service => {
                    // Storage
                    serviceCollection.set(storage_1.IStorageService, service);
                    return service;
                })
            ]);
            return { serviceCollection, logService, storageService: services[1] };
        });
    }
    resolveWorkspaceInitializationPayload() {
        return __awaiter(this, void 0, void 0, function* () {
            // Multi-root workspace
            if (this.environmentService.configuration.workspace) {
                return this.environmentService.configuration.workspace;
            }
            // Single-folder workspace
            let workspaceInitializationPayload;
            if (this.environmentService.configuration.folderUri) {
                workspaceInitializationPayload = yield this.resolveSingleFolderWorkspaceInitializationPayload(this.environmentService.configuration.folderUri);
            }
            // Fallback to empty workspace if we have no payload yet.
            if (!workspaceInitializationPayload) {
                let id;
                if (this.environmentService.configuration.backupWorkspaceResource) {
                    id = resources_1.basename(this.environmentService.configuration.backupWorkspaceResource); // we know the backupPath must be a unique path so we leverage its name as workspace ID
                }
                else if (this.environmentService.isExtensionDevelopment) {
                    id = 'ext-dev'; // extension development window never stores backups and is a singleton
                }
                else {
                    throw new Error('Unexpected window configuration without backupPath');
                }
                workspaceInitializationPayload = { id };
            }
            return workspaceInitializationPayload;
        });
    }
    resolveSingleFolderWorkspaceInitializationPayload(folderUri) {
        return __awaiter(this, void 0, void 0, function* () {
            // Return early the folder is not local
            if (folderUri.scheme !== network_1.Schemas.file) {
                return { id: crypto_1.createHash('md5').update(folderUri.toString()).digest('hex'), folder: folderUri };
            }
            function computeLocalDiskFolderId(folder, stat) {
                let ctime;
                if (platform_1.isLinux) {
                    ctime = stat.ino; // Linux: birthtime is ctime, so we cannot use it! We use the ino instead!
                }
                else if (platform_1.isMacintosh) {
                    ctime = stat.birthtime.getTime(); // macOS: birthtime is fine to use as is
                }
                else if (platform_1.isWindows) {
                    if (typeof stat.birthtimeMs === 'number') {
                        ctime = Math.floor(stat.birthtimeMs); // Windows: fix precision issue in node.js 8.x to get 7.x results (see https://github.com/nodejs/node/issues/19897)
                    }
                    else {
                        ctime = stat.birthtime.getTime();
                    }
                }
                // we use the ctime as extra salt to the ID so that we catch the case of a folder getting
                // deleted and recreated. in that case we do not want to carry over previous state
                return crypto_1.createHash('md5').update(folder.fsPath).update(ctime ? String(ctime) : '').digest('hex');
            }
            // For local: ensure path is absolute and exists
            try {
                const sanitizedFolderPath = extpath_1.sanitizeFilePath(folderUri.fsPath, process.env['VSCODE_CWD'] || process.cwd());
                const fileStat = yield pfs_1.stat(sanitizedFolderPath);
                const sanitizedFolderUri = uri_1.URI.file(sanitizedFolderPath);
                return {
                    id: computeLocalDiskFolderId(sanitizedFolderUri, fileStat),
                    folder: sanitizedFolderUri
                };
            }
            catch (error) {
                errors_1.onUnexpectedError(error);
            }
            return;
        });
    }
    createWorkspaceService(payload, fileService, remoteAgentService, logService) {
        return __awaiter(this, void 0, void 0, function* () {
            const workspaceService = new configurationService_1.WorkspaceService({ remoteAuthority: this.environmentService.configuration.remoteAuthority, configurationCache: new configurationCache_1.ConfigurationCache(this.environmentService) }, this.environmentService, fileService, remoteAgentService);
            try {
                yield workspaceService.initialize(payload);
                return workspaceService;
            }
            catch (error) {
                errors_1.onUnexpectedError(error);
                logService.error(error);
                return workspaceService;
            }
        });
    }
    createStorageService(payload, logService, mainProcessService) {
        return __awaiter(this, void 0, void 0, function* () {
            const globalStorageDatabase = new storageIpc_1.GlobalStorageDatabaseChannelClient(mainProcessService.getChannel('storage'));
            const storageService = new storageService_1.StorageService(globalStorageDatabase, logService, this.environmentService);
            try {
                yield storageService.initialize(payload);
                return storageService;
            }
            catch (error) {
                errors_1.onUnexpectedError(error);
                logService.error(error);
                return storageService;
            }
        });
    }
    createLogService(mainProcessService, environmentService) {
        const loggerClient = new logIpc_1.LoggerChannelClient(mainProcessService.getChannel('logger'));
        // Extension development test CLI: forward everything to main side
        const loggers = [];
        if (environmentService.isExtensionDevelopment && !!environmentService.extensionTestsLocationURI) {
            loggers.push(new log_1.ConsoleLogInMainService(loggerClient, this.environmentService.configuration.logLevel));
        }
        // Normal logger: spdylog and console
        else {
            loggers.push(new log_1.ConsoleLogService(this.environmentService.configuration.logLevel), new spdlogService_1.SpdLogService(`renderer${this.environmentService.configuration.windowId}`, environmentService.logsPath, this.environmentService.configuration.logLevel));
        }
        return new logIpc_1.FollowerLogService(loggerClient, new log_1.MultiplexLogService(loggers));
    }
}
function main(configuration) {
    const renderer = new CodeRendererMain(configuration);
    return renderer.open();
}
exports.main = main;
//# sourceMappingURL=desktop.main.js.map