"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const paths = require("vs/base/common/path");
const process = require("vs/base/common/process");
const types = require("vs/base/common/types");
const objects = require("vs/base/common/objects");
const platform_1 = require("vs/base/common/platform");
const labels_1 = require("vs/base/common/labels");
const nls_1 = require("vs/nls");
class AbstractVariableResolverService {
    constructor(_context, _envVariables) {
        this._context = _context;
        this._envVariables = _envVariables;
        this._contributedVariables = new Map();
        if (platform_1.isWindows && _envVariables) {
            this._envVariables = Object.create(null);
            Object.keys(_envVariables).forEach(key => {
                this._envVariables[key.toLowerCase()] = _envVariables[key];
            });
        }
    }
    resolve(root, value) {
        return this.recursiveResolve(root ? root.uri : undefined, value);
    }
    resolveAnyBase(workspaceFolder, config, commandValueMapping, resolvedVariables) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = objects.deepClone(config);
            // hoist platform specific attributes to top level
            if (platform_1.isWindows && result.windows) {
                Object.keys(result.windows).forEach(key => result[key] = result.windows[key]);
            }
            else if (platform_1.isMacintosh && result.osx) {
                Object.keys(result.osx).forEach(key => result[key] = result.osx[key]);
            }
            else if (platform_1.isLinux && result.linux) {
                Object.keys(result.linux).forEach(key => result[key] = result.linux[key]);
            }
            // delete all platform specific sections
            delete result.windows;
            delete result.osx;
            delete result.linux;
            // substitute all variables recursively in string values
            return this.recursiveResolvePromise(workspaceFolder ? workspaceFolder.uri : undefined, result, commandValueMapping, resolvedVariables);
        });
    }
    resolveAny(workspaceFolder, config, commandValueMapping) {
        return this.resolveAnyBase(workspaceFolder, config, commandValueMapping);
    }
    resolveAnyMap(workspaceFolder, config, commandValueMapping) {
        return __awaiter(this, void 0, void 0, function* () {
            const resolvedVariables = new Map();
            const newConfig = yield this.resolveAnyBase(workspaceFolder, config, commandValueMapping, resolvedVariables);
            return { newConfig, resolvedVariables };
        });
    }
    resolveWithInteractionReplace(folder, config, section, variables) {
        throw new Error('resolveWithInteractionReplace not implemented.');
    }
    resolveWithInteraction(folder, config, section, variables) {
        throw new Error('resolveWithInteraction not implemented.');
    }
    contributeVariable(variable, resolution) {
        if (this._contributedVariables.has(variable)) {
            throw new Error('Variable ' + variable + ' is contributed twice.');
        }
        else {
            this._contributedVariables.set(variable, resolution);
        }
    }
    recursiveResolve(folderUri, value, commandValueMapping, resolvedVariables) {
        if (types.isString(value)) {
            return this.resolveString(folderUri, value, commandValueMapping, resolvedVariables);
        }
        else if (types.isArray(value)) {
            return value.map(s => this.recursiveResolve(folderUri, s, commandValueMapping, resolvedVariables));
        }
        else if (types.isObject(value)) {
            let result = Object.create(null);
            Object.keys(value).forEach(key => {
                const replaced = this.resolveString(folderUri, key, commandValueMapping, resolvedVariables);
                result[replaced] = this.recursiveResolve(folderUri, value[key], commandValueMapping, resolvedVariables);
            });
            return result;
        }
        return value;
    }
    recursiveResolvePromise(folderUri, value, commandValueMapping, resolvedVariables) {
        return __awaiter(this, void 0, void 0, function* () {
            if (types.isString(value)) {
                return this.resolveStringPromise(folderUri, value, commandValueMapping, resolvedVariables);
            }
            else if (types.isArray(value)) {
                return Promise.all(value.map(s => this.recursiveResolvePromise(folderUri, s, commandValueMapping, resolvedVariables)));
            }
            else if (types.isObject(value)) {
                let result = Object.create(null);
                const keys = Object.keys(value);
                for (let key of keys) {
                    const replaced = yield this.resolveStringPromise(folderUri, key, commandValueMapping, resolvedVariables);
                    result[replaced] = yield this.recursiveResolvePromise(folderUri, value[key], commandValueMapping, resolvedVariables);
                }
                return result;
            }
            return value;
        });
    }
    resolveString(folderUri, value, commandValueMapping, resolvedVariables) {
        // loop through all variables occurrences in 'value'
        const replaced = value.replace(AbstractVariableResolverService.VARIABLE_REGEXP, (match, variable) => {
            let resolvedValue = this.evaluateSingleVariable(match, variable, folderUri, commandValueMapping);
            if (resolvedVariables) {
                resolvedVariables.set(variable, resolvedValue);
            }
            return resolvedValue;
        });
        return replaced;
    }
    resolveStringPromise(folderUri, value, commandValueMapping, resolvedVariables) {
        return __awaiter(this, void 0, void 0, function* () {
            // loop through all variables occurrences in 'value'
            const matches = value.match(AbstractVariableResolverService.VARIABLE_REGEXP);
            const replaces = new Map();
            if (!matches) {
                return value;
            }
            for (const match of matches) {
                const evaluate = yield this.evaluateSingleContributedVariable(match, match.match(AbstractVariableResolverService.VARIABLE_REGEXP_SINGLE)[1]);
                if (evaluate !== match) {
                    replaces.set(match, evaluate);
                }
            }
            const replaced = value.replace(AbstractVariableResolverService.VARIABLE_REGEXP, (match, variable) => {
                let resolvedValue = this.evaluateSingleVariable(match, variable, folderUri, commandValueMapping);
                if ((resolvedValue === match) && (replaces.has(match))) {
                    resolvedValue = replaces.get(match);
                }
                if (resolvedVariables) {
                    resolvedVariables.set(variable, resolvedValue);
                }
                return resolvedValue;
            });
            return replaced;
        });
    }
    evaluateSingleVariable(match, variable, folderUri, commandValueMapping) {
        // try to separate variable arguments from variable name
        let argument;
        const parts = variable.split(':');
        if (parts.length > 1) {
            variable = parts[0];
            argument = parts[1];
        }
        // common error handling for all variables that require an open editor
        const getFilePath = () => {
            const filePath = this._context.getFilePath();
            if (filePath) {
                return filePath;
            }
            throw new Error(nls_1.localize('canNotResolveFile', "'{0}' can not be resolved. Please open an editor.", match));
        };
        // common error handling for all variables that require an open folder and accept a folder name argument
        const getFolderUri = (withArg = true) => {
            if (withArg && argument) {
                const folder = this._context.getFolderUri(argument);
                if (folder) {
                    return folder;
                }
                throw new Error(nls_1.localize('canNotFindFolder', "'{0}' can not be resolved. No such folder '{1}'.", match, argument));
            }
            if (folderUri) {
                return folderUri;
            }
            if (this._context.getWorkspaceFolderCount() > 1) {
                throw new Error(nls_1.localize('canNotResolveWorkspaceFolderMultiRoot', "'{0}' can not be resolved in a multi folder workspace. Scope this variable using ':' and a workspace folder name.", match));
            }
            throw new Error(nls_1.localize('canNotResolveWorkspaceFolder', "'{0}' can not be resolved. Please open a folder.", match));
        };
        switch (variable) {
            case 'env':
                if (argument) {
                    if (platform_1.isWindows) {
                        argument = argument.toLowerCase();
                    }
                    const env = this._envVariables[argument];
                    if (types.isString(env)) {
                        return env;
                    }
                    // For `env` we should do the same as a normal shell does - evaluates missing envs to an empty string #46436
                    return '';
                }
                throw new Error(nls_1.localize('missingEnvVarName', "'{0}' can not be resolved because no environment variable name is given.", match));
            case 'config':
                if (argument) {
                    const config = this._context.getConfigurationValue(getFolderUri(false), argument);
                    if (types.isUndefinedOrNull(config)) {
                        throw new Error(nls_1.localize('configNotFound', "'{0}' can not be resolved because setting '{1}' not found.", match, argument));
                    }
                    if (types.isObject(config)) {
                        throw new Error(nls_1.localize('configNoString', "'{0}' can not be resolved because '{1}' is a structured value.", match, argument));
                    }
                    return config;
                }
                throw new Error(nls_1.localize('missingConfigName', "'{0}' can not be resolved because no settings name is given.", match));
            case 'command':
                return this.resolveFromMap(match, argument, commandValueMapping, 'command');
            case 'input':
                return this.resolveFromMap(match, argument, commandValueMapping, 'input');
            default: {
                switch (variable) {
                    case 'workspaceRoot':
                    case 'workspaceFolder':
                        return labels_1.normalizeDriveLetter(getFolderUri().fsPath);
                    case 'cwd':
                        return (folderUri ? labels_1.normalizeDriveLetter(getFolderUri().fsPath) : process.cwd());
                    case 'workspaceRootFolderName':
                    case 'workspaceFolderBasename':
                        return paths.basename(getFolderUri().fsPath);
                    case 'lineNumber':
                        const lineNumber = this._context.getLineNumber();
                        if (lineNumber) {
                            return lineNumber;
                        }
                        throw new Error(nls_1.localize('canNotResolveLineNumber', "'{0}' can not be resolved. Make sure to have a line selected in the active editor.", match));
                    case 'selectedText':
                        const selectedText = this._context.getSelectedText();
                        if (selectedText) {
                            return selectedText;
                        }
                        throw new Error(nls_1.localize('canNotResolveSelectedText', "'{0}' can not be resolved. Make sure to have some text selected in the active editor.", match));
                    case 'file':
                        return getFilePath();
                    case 'relativeFile':
                        if (folderUri) {
                            return paths.normalize(paths.relative(getFolderUri().fsPath, getFilePath()));
                        }
                        return getFilePath();
                    case 'relativeFileDirname':
                        let dirname = paths.dirname(getFilePath());
                        if (folderUri) {
                            return paths.normalize(paths.relative(getFolderUri().fsPath, dirname));
                        }
                        return dirname;
                    case 'fileDirname':
                        return paths.dirname(getFilePath());
                    case 'fileExtname':
                        return paths.extname(getFilePath());
                    case 'fileBasename':
                        return paths.basename(getFilePath());
                    case 'fileBasenameNoExtension':
                        const basename = paths.basename(getFilePath());
                        return (basename.slice(0, basename.length - paths.extname(basename).length));
                    case 'execPath':
                        const ep = this._context.getExecPath();
                        if (ep) {
                            return ep;
                        }
                        return match;
                    default:
                        return match;
                }
            }
        }
    }
    evaluateSingleContributedVariable(match, variable) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._contributedVariables.has(variable)) {
                const contributedValue = yield this._contributedVariables.get(variable)();
                if (contributedValue !== undefined) {
                    return contributedValue;
                }
            }
            return match;
        });
    }
    resolveFromMap(match, argument, commandValueMapping, prefix) {
        if (argument && commandValueMapping) {
            const v = commandValueMapping[prefix + ':' + argument];
            if (typeof v === 'string') {
                return v;
            }
            throw new Error(nls_1.localize('noValueForCommand', "'{0}' can not be resolved because the command has no value.", match));
        }
        return match;
    }
}
exports.AbstractVariableResolverService = AbstractVariableResolverService;
AbstractVariableResolverService.VARIABLE_REGEXP = /\$\{(.*?)\}/g;
AbstractVariableResolverService.VARIABLE_REGEXP_SINGLE = /\$\{(.*?)\}/;
//# sourceMappingURL=variableResolver.js.map