"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const fs = require("fs");
const os = require("os");
const path = require("vs/base/common/path");
const json = require("vs/base/common/json");
const keyCodes_1 = require("vs/base/common/keyCodes");
const platform_1 = require("vs/base/common/platform");
const uuid = require("vs/base/common/uuid");
const pfs_1 = require("vs/base/node/pfs");
const modeService_1 = require("vs/editor/common/services/modeService");
const modeServiceImpl_1 = require("vs/editor/common/services/modeServiceImpl");
const modelService_1 = require("vs/editor/common/services/modelService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const resolverService_1 = require("vs/editor/common/services/resolverService");
const resourceConfiguration_1 = require("vs/editor/common/services/resourceConfiguration");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const configurationService_1 = require("vs/platform/configuration/node/configurationService");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const environment_1 = require("vs/platform/environment/common/environment");
const files_1 = require("vs/platform/files/common/files");
const instantiationServiceMock_1 = require("vs/platform/instantiation/test/common/instantiationServiceMock");
const resolvedKeybindingItem_1 = require("vs/platform/keybinding/common/resolvedKeybindingItem");
const usLayoutResolvedKeybinding_1 = require("vs/platform/keybinding/common/usLayoutResolvedKeybinding");
const mockKeybindingService_1 = require("vs/platform/keybinding/test/common/mockKeybindingService");
const lifecycle_1 = require("vs/platform/lifecycle/common/lifecycle");
const log_1 = require("vs/platform/log/common/log");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const backup_1 = require("vs/workbench/services/backup/common/backup");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
const keybindingEditing_1 = require("vs/workbench/services/keybinding/common/keybindingEditing");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const textModelResolverService_1 = require("vs/workbench/services/textmodelResolver/common/textModelResolverService");
const untitledEditorService_1 = require("vs/workbench/services/untitled/common/untitledEditorService");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const fileService_1 = require("vs/platform/files/common/fileService");
const network_1 = require("vs/base/common/network");
const diskFileSystemProvider_1 = require("vs/platform/files/node/diskFileSystemProvider");
const uri_1 = require("vs/base/common/uri");
const fileUserDataProvider_1 = require("vs/workbench/services/userData/common/fileUserDataProvider");
const argv_1 = require("vs/platform/environment/node/argv");
const environmentService_1 = require("vs/workbench/services/environment/node/environmentService");
class TestEnvironmentService extends environmentService_1.WorkbenchEnvironmentService {
    constructor(_appSettingsHome) {
        super(argv_1.parseArgs(process.argv, argv_1.OPTIONS), process.execPath);
        this._appSettingsHome = _appSettingsHome;
    }
    get appSettingsHome() { return this._appSettingsHome; }
}
suite('KeybindingsEditing', () => {
    let instantiationService;
    let testObject;
    let testDir;
    let keybindingsFile;
    setup(() => {
        return setUpWorkspace().then(() => {
            keybindingsFile = path.join(testDir, 'keybindings.json');
            instantiationService = new instantiationServiceMock_1.TestInstantiationService();
            const environmentService = new TestEnvironmentService(uri_1.URI.file(testDir));
            instantiationService.stub(environment_1.IEnvironmentService, environmentService);
            instantiationService.stub(configuration_1.IConfigurationService, configurationService_1.ConfigurationService);
            instantiationService.stub(configuration_1.IConfigurationService, 'getValue', { 'eol': '\n' });
            instantiationService.stub(configuration_1.IConfigurationService, 'onDidUpdateConfiguration', () => { });
            instantiationService.stub(configuration_1.IConfigurationService, 'onDidChangeConfiguration', () => { });
            instantiationService.stub(workspace_1.IWorkspaceContextService, new workbenchTestServices_1.TestContextService());
            const lifecycleService = new workbenchTestServices_1.TestLifecycleService();
            instantiationService.stub(lifecycle_1.ILifecycleService, lifecycleService);
            instantiationService.stub(contextkey_1.IContextKeyService, instantiationService.createInstance(mockKeybindingService_1.MockContextKeyService));
            instantiationService.stub(editorGroupsService_1.IEditorGroupsService, new workbenchTestServices_1.TestEditorGroupsService());
            instantiationService.stub(editorService_1.IEditorService, new workbenchTestServices_1.TestEditorService());
            instantiationService.stub(telemetry_1.ITelemetryService, telemetryUtils_1.NullTelemetryService);
            instantiationService.stub(modeService_1.IModeService, modeServiceImpl_1.ModeServiceImpl);
            instantiationService.stub(log_1.ILogService, new log_1.NullLogService());
            instantiationService.stub(resourceConfiguration_1.ITextResourcePropertiesService, new workbenchTestServices_1.TestTextResourcePropertiesService(instantiationService.get(configuration_1.IConfigurationService)));
            instantiationService.stub(modelService_1.IModelService, instantiationService.createInstance(modelServiceImpl_1.ModelServiceImpl));
            const fileService = new fileService_1.FileService(new log_1.NullLogService());
            const diskFileSystemProvider = new diskFileSystemProvider_1.DiskFileSystemProvider(new log_1.NullLogService());
            fileService.registerProvider(network_1.Schemas.file, diskFileSystemProvider);
            fileService.registerProvider(network_1.Schemas.userData, new fileUserDataProvider_1.FileUserDataProvider(environmentService.appSettingsHome, environmentService.backupHome, diskFileSystemProvider, environmentService));
            instantiationService.stub(files_1.IFileService, fileService);
            instantiationService.stub(untitledEditorService_1.IUntitledEditorService, instantiationService.createInstance(untitledEditorService_1.UntitledEditorService));
            instantiationService.stub(textfiles_1.ITextFileService, instantiationService.createInstance(workbenchTestServices_1.TestTextFileService));
            instantiationService.stub(resolverService_1.ITextModelService, instantiationService.createInstance(textModelResolverService_1.TextModelResolverService));
            instantiationService.stub(backup_1.IBackupFileService, new workbenchTestServices_1.TestBackupFileService());
            testObject = instantiationService.createInstance(keybindingEditing_1.KeybindingsEditingService);
        });
    });
    function setUpWorkspace() {
        return __awaiter(this, void 0, void 0, function* () {
            testDir = path.join(os.tmpdir(), 'vsctests', uuid.generateUuid());
            return yield pfs_1.mkdirp(testDir, 493);
        });
    }
    teardown(() => {
        return new Promise((c) => {
            if (testDir) {
                pfs_1.rimraf(testDir, pfs_1.RimRafMode.MOVE).then(c, c);
            }
            else {
                c(undefined);
            }
        }).then(() => testDir = null);
    });
    test('errors cases - parse errors', () => {
        fs.writeFileSync(keybindingsFile, ',,,,,,,,,,,,,,');
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ } }), 'alt+c', undefined)
            .then(() => assert.fail('Should fail with parse errors'), error => assert.equal(error.message, 'Unable to write to the keybindings configuration file. Please open it to correct errors/warnings in the file and try again.'));
    });
    test('errors cases - parse errors 2', () => {
        fs.writeFileSync(keybindingsFile, '[{"key": }]');
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ } }), 'alt+c', undefined)
            .then(() => assert.fail('Should fail with parse errors'), error => assert.equal(error.message, 'Unable to write to the keybindings configuration file. Please open it to correct errors/warnings in the file and try again.'));
    });
    test('errors cases - dirty', () => {
        instantiationService.stub(textfiles_1.ITextFileService, 'isDirty', true);
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ } }), 'alt+c', undefined)
            .then(() => assert.fail('Should fail with dirty error'), error => assert.equal(error.message, 'Unable to write because the keybindings configuration file is dirty. Please save it first and then try again.'));
    });
    test('errors cases - did not find an array', () => {
        fs.writeFileSync(keybindingsFile, '{"key": "alt+c", "command": "hello"}');
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ } }), 'alt+c', undefined)
            .then(() => assert.fail('Should fail with dirty error'), error => assert.equal(error.message, 'Unable to write to the keybindings configuration file. It has an object which is not of type Array. Please open the file to clean up and try again.'));
    });
    test('edit a default keybinding to an empty file', () => {
        fs.writeFileSync(keybindingsFile, '');
        const expected = [{ key: 'alt+c', command: 'a' }, { key: 'escape', command: '-a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ }, command: 'a' }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('edit a default keybinding to an empty array', () => {
        writeToKeybindingsFile();
        const expected = [{ key: 'alt+c', command: 'a' }, { key: 'escape', command: '-a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ }, command: 'a' }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('edit a default keybinding in an existing array', () => {
        writeToKeybindingsFile({ command: 'b', key: 'shift+c' });
        const expected = [{ key: 'shift+c', command: 'b' }, { key: 'alt+c', command: 'a' }, { key: 'escape', command: '-a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ }, command: 'a' }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('add a new default keybinding', () => {
        const expected = [{ key: 'alt+c', command: 'a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a' }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('edit an user keybinding', () => {
        writeToKeybindingsFile({ key: 'escape', command: 'b' });
        const expected = [{ key: 'alt+c', command: 'b' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ }, command: 'b', isDefault: false }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('edit an user keybinding with more than one element', () => {
        writeToKeybindingsFile({ key: 'escape', command: 'b' }, { key: 'alt+shift+g', command: 'c' });
        const expected = [{ key: 'alt+c', command: 'b' }, { key: 'alt+shift+g', command: 'c' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ firstPart: { keyCode: 9 /* Escape */ }, command: 'b', isDefault: false }), 'alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('remove a default keybinding', () => {
        const expected = [{ key: 'alt+c', command: '-a' }];
        return testObject.removeKeybinding(aResolvedKeybindingItem({ command: 'a', firstPart: { keyCode: 33 /* KEY_C */, modifiers: { altKey: true } } }))
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('remove a user keybinding', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: 'b' });
        return testObject.removeKeybinding(aResolvedKeybindingItem({ command: 'b', firstPart: { keyCode: 33 /* KEY_C */, modifiers: { altKey: true } }, isDefault: false }))
            .then(() => assert.deepEqual(getUserKeybindings(), []));
    });
    test('reset an edited keybinding', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: 'b' });
        return testObject.resetKeybinding(aResolvedKeybindingItem({ command: 'b', firstPart: { keyCode: 33 /* KEY_C */, modifiers: { altKey: true } }, isDefault: false }))
            .then(() => assert.deepEqual(getUserKeybindings(), []));
    });
    test('reset a removed keybinding', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-b' });
        return testObject.resetKeybinding(aResolvedKeybindingItem({ command: 'b', isDefault: false }))
            .then(() => assert.deepEqual(getUserKeybindings(), []));
    });
    test('reset multiple removed keybindings', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-b' });
        writeToKeybindingsFile({ key: 'alt+shift+c', command: '-b' });
        writeToKeybindingsFile({ key: 'escape', command: '-b' });
        return testObject.resetKeybinding(aResolvedKeybindingItem({ command: 'b', isDefault: false }))
            .then(() => assert.deepEqual(getUserKeybindings(), []));
    });
    test('add a new keybinding to unassigned keybinding', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-a' });
        const expected = [{ key: 'alt+c', command: '-a' }, { key: 'shift+alt+c', command: 'a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a', isDefault: false }), 'shift+alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('add when expression', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-a' });
        const expected = [{ key: 'alt+c', command: '-a' }, { key: 'shift+alt+c', command: 'a', when: 'editorTextFocus' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a', isDefault: false }), 'shift+alt+c', 'editorTextFocus')
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('update command and when expression', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' });
        const expected = [{ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' }, { key: 'shift+alt+c', command: 'a', when: 'editorTextFocus' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a', isDefault: false }), 'shift+alt+c', 'editorTextFocus')
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('update when expression', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' }, { key: 'shift+alt+c', command: 'a', when: 'editorTextFocus && !editorReadonly' });
        const expected = [{ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' }, { key: 'shift+alt+c', command: 'a', when: 'editorTextFocus' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a', isDefault: false, when: 'editorTextFocus && !editorReadonly' }), 'shift+alt+c', 'editorTextFocus')
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    test('remove when expression', () => {
        writeToKeybindingsFile({ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' });
        const expected = [{ key: 'alt+c', command: '-a', when: 'editorTextFocus && !editorReadonly' }, { key: 'shift+alt+c', command: 'a' }];
        return testObject.editKeybinding(aResolvedKeybindingItem({ command: 'a', isDefault: false }), 'shift+alt+c', undefined)
            .then(() => assert.deepEqual(getUserKeybindings(), expected));
    });
    function writeToKeybindingsFile(...keybindings) {
        fs.writeFileSync(keybindingsFile, JSON.stringify(keybindings || []));
    }
    function getUserKeybindings() {
        return json.parse(fs.readFileSync(keybindingsFile).toString('utf8'));
    }
    function aResolvedKeybindingItem({ command, when, isDefault, firstPart, chordPart }) {
        const aSimpleKeybinding = function (part) {
            const { ctrlKey, shiftKey, altKey, metaKey } = part.modifiers || { ctrlKey: false, shiftKey: false, altKey: false, metaKey: false };
            return new keyCodes_1.SimpleKeybinding(ctrlKey, shiftKey, altKey, metaKey, part.keyCode);
        };
        let parts = [];
        if (firstPart) {
            parts.push(aSimpleKeybinding(firstPart));
            if (chordPart) {
                parts.push(aSimpleKeybinding(chordPart));
            }
        }
        const keybinding = parts.length > 0 ? new usLayoutResolvedKeybinding_1.USLayoutResolvedKeybinding(new keyCodes_1.ChordKeybinding(parts), platform_1.OS) : undefined;
        return new resolvedKeybindingItem_1.ResolvedKeybindingItem(keybinding, command || 'some command', null, when ? contextkey_1.ContextKeyExpr.deserialize(when) : undefined, isDefault === undefined ? true : isDefault);
    }
});
//# sourceMappingURL=keybindingEditing.test.js.map