"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls_1 = require("vs/nls");
const uri_1 = require("vs/base/common/uri");
const paths = require("vs/base/common/path");
const event_1 = require("vs/base/common/event");
const contributions_1 = require("vs/workbench/common/contributions");
const platform_1 = require("vs/platform/registry/common/platform");
const environment_1 = require("vs/platform/environment/common/environment");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const resources_1 = require("vs/base/common/resources");
const platform_2 = require("vs/base/common/platform");
const labels_1 = require("vs/base/common/labels");
const strings_1 = require("vs/base/common/strings");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const label_1 = require("vs/platform/label/common/label");
const extensionsRegistry_1 = require("vs/workbench/services/extensions/common/extensionsRegistry");
const glob_1 = require("vs/base/common/glob");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const resourceLabelFormattersExtPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'resourceLabelFormatters',
    jsonSchema: {
        description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters', 'Contributes resource label formatting rules.'),
        type: 'array',
        items: {
            type: 'object',
            required: ['scheme', 'formatting'],
            properties: {
                scheme: {
                    type: 'string',
                    description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.scheme', 'URI scheme on which to match the formatter on. For example "file". Simple glob patterns are supported.'),
                },
                authority: {
                    type: 'string',
                    description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.authority', 'URI authority on which to match the formatter on. Simple glob patterns are supported.'),
                },
                formatting: {
                    description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.formatting', "Rules for formatting uri resource labels."),
                    type: 'object',
                    properties: {
                        label: {
                            type: 'string',
                            description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.label', "Label rules to display. For example: myLabel:/${path}. ${path}, ${scheme} and ${authority} are supported as variables.")
                        },
                        separator: {
                            type: 'string',
                            description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.separator', "Separator to be used in the uri label display. '/' or '\' as an example.")
                        },
                        tildify: {
                            type: 'boolean',
                            description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.tildify', "Controls if the start of the uri label should be tildified when possible.")
                        },
                        workspaceSuffix: {
                            type: 'string',
                            description: nls_1.localize('vscode.extension.contributes.resourceLabelFormatters.formatting.workspaceSuffix', "Suffix appended to the workspace label.")
                        }
                    }
                }
            }
        }
    }
});
const sepRegexp = /\//g;
const labelMatchingRegexp = /\$\{(scheme|authority|path|(query)\.(.+?))\}/g;
function hasDriveLetter(path) {
    return !!(platform_2.isWindows && path && path[2] === ':');
}
let ResourceLabelFormattersHandler = class ResourceLabelFormattersHandler {
    constructor(labelService) {
        this.formattersDisposables = new Map();
        resourceLabelFormattersExtPoint.setHandler((extensions, delta) => {
            delta.added.forEach(added => added.value.forEach(formatter => {
                this.formattersDisposables.set(formatter, labelService.registerFormatter(formatter));
            }));
            delta.removed.forEach(removed => removed.value.forEach(formatter => {
                this.formattersDisposables.get(formatter).dispose();
            }));
        });
    }
};
ResourceLabelFormattersHandler = __decorate([
    __param(0, label_1.ILabelService)
], ResourceLabelFormattersHandler);
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(ResourceLabelFormattersHandler, 3 /* Restored */);
let LabelService = class LabelService {
    constructor(environmentService, contextService) {
        this.environmentService = environmentService;
        this.contextService = contextService;
        this.formatters = [];
        this._onDidChangeFormatters = new event_1.Emitter();
    }
    get onDidChangeFormatters() {
        return this._onDidChangeFormatters.event;
    }
    findFormatting(resource) {
        let bestResult;
        this.formatters.forEach(formatter => {
            if (formatter.scheme === resource.scheme) {
                if (!bestResult && !formatter.authority) {
                    bestResult = formatter;
                    return;
                }
                if (!formatter.authority) {
                    return;
                }
                if (glob_1.match(formatter.authority, resource.authority) && (!bestResult || !bestResult.authority || formatter.authority.length > bestResult.authority.length || ((formatter.authority.length === bestResult.authority.length) && formatter.priority))) {
                    bestResult = formatter;
                }
            }
        });
        return bestResult ? bestResult.formatting : undefined;
    }
    getUriLabel(resource, options = {}) {
        return this.doGetUriLabel(resource, this.findFormatting(resource), options);
    }
    doGetUriLabel(resource, formatting, options = {}) {
        if (!formatting) {
            return labels_1.getPathLabel(resource.path, this.environmentService, options.relative ? this.contextService : undefined);
        }
        let label;
        const baseResource = this.contextService && this.contextService.getWorkspaceFolder(resource);
        if (options.relative && baseResource) {
            let relativeLabel;
            if (resources_1.isEqual(baseResource.uri, resource)) {
                relativeLabel = ''; // no label if resources are identical
            }
            else {
                const baseResourceLabel = this.formatUri(baseResource.uri, formatting, options.noPrefix);
                relativeLabel = strings_1.ltrim(this.formatUri(resource, formatting, options.noPrefix).substring(baseResourceLabel.length), formatting.separator);
            }
            const hasMultipleRoots = this.contextService.getWorkspace().folders.length > 1;
            if (hasMultipleRoots && !options.noPrefix) {
                const rootName = (baseResource && baseResource.name) ? baseResource.name : resources_1.basenameOrAuthority(baseResource.uri);
                relativeLabel = relativeLabel ? (rootName + ' • ' + relativeLabel) : rootName; // always show root basename if there are multiple
            }
            label = relativeLabel;
        }
        else {
            label = this.formatUri(resource, formatting, options.noPrefix);
        }
        return options.endWithSeparator ? this.appendSeparatorIfMissing(label, formatting) : label;
    }
    getUriBasenameLabel(resource) {
        const formatting = this.findFormatting(resource);
        const label = this.doGetUriLabel(resource, formatting);
        if (formatting) {
            switch (formatting.separator) {
                case paths.win32.sep: return paths.win32.basename(label);
                case paths.posix.sep: return paths.posix.basename(label);
            }
        }
        return paths.basename(label);
    }
    getWorkspaceLabel(workspace, options) {
        if (workspace_1.IWorkspace.isIWorkspace(workspace)) {
            const identifier = workspaces_1.toWorkspaceIdentifier(workspace);
            if (!identifier) {
                return '';
            }
            workspace = identifier;
        }
        // Workspace: Single Folder
        if (workspaces_1.isSingleFolderWorkspaceIdentifier(workspace)) {
            // Folder on disk
            const label = options && options.verbose ? this.getUriLabel(workspace) : resources_1.basename(workspace) || '/';
            return this.appendWorkspaceSuffix(label, workspace);
        }
        if (workspaces_1.isWorkspaceIdentifier(workspace)) {
            // Workspace: Untitled
            if (resources_1.isEqualOrParent(workspace.configPath, this.environmentService.untitledWorkspacesHome)) {
                return nls_1.localize('untitledWorkspace', "Untitled (Workspace)");
            }
            // Workspace: Saved
            let filename = resources_1.basename(workspace.configPath);
            if (strings_1.endsWith(filename, workspaces_1.WORKSPACE_EXTENSION)) {
                filename = filename.substr(0, filename.length - workspaces_1.WORKSPACE_EXTENSION.length - 1);
            }
            let label;
            if (options && options.verbose) {
                label = nls_1.localize('workspaceNameVerbose', "{0} (Workspace)", this.getUriLabel(resources_1.joinPath(resources_1.dirname(workspace.configPath), filename)));
            }
            else {
                label = nls_1.localize('workspaceName', "{0} (Workspace)", filename);
            }
            return this.appendWorkspaceSuffix(label, workspace.configPath);
        }
        return '';
    }
    getSeparator(scheme, authority) {
        const formatter = this.findFormatting(uri_1.URI.from({ scheme, authority }));
        return formatter && formatter.separator || '/';
    }
    getHostLabel(scheme, authority) {
        const formatter = this.findFormatting(uri_1.URI.from({ scheme, authority }));
        return formatter && formatter.workspaceSuffix || '';
    }
    registerFormatter(formatter) {
        this.formatters.push(formatter);
        this._onDidChangeFormatters.fire();
        return {
            dispose: () => {
                this.formatters = this.formatters.filter(f => f !== formatter);
                this._onDidChangeFormatters.fire();
            }
        };
    }
    formatUri(resource, formatting, forceNoTildify) {
        let label = formatting.label.replace(labelMatchingRegexp, (match, token, qsToken, qsValue) => {
            switch (token) {
                case 'scheme': return resource.scheme;
                case 'authority': return resource.authority;
                case 'path': return resource.path;
                default: {
                    if (qsToken === 'query') {
                        const { query } = resource;
                        if (query && query[0] === '{' && query[query.length - 1] === '}') {
                            try {
                                return JSON.parse(query)[qsValue] || '';
                            }
                            catch (_a) { }
                        }
                    }
                    return '';
                }
            }
        });
        // convert \c:\something => C:\something
        if (formatting.normalizeDriveLetter && hasDriveLetter(label)) {
            label = label.charAt(1).toUpperCase() + label.substr(2);
        }
        if (formatting.tildify && !forceNoTildify) {
            label = labels_1.tildify(label, this.environmentService.userHome);
        }
        if (formatting.authorityPrefix && resource.authority) {
            label = formatting.authorityPrefix + label;
        }
        return label.replace(sepRegexp, formatting.separator);
    }
    appendSeparatorIfMissing(label, formatting) {
        let appendedLabel = label;
        if (!strings_1.endsWith(label, formatting.separator)) {
            appendedLabel += formatting.separator;
        }
        return appendedLabel;
    }
    appendWorkspaceSuffix(label, uri) {
        const formatting = this.findFormatting(uri);
        const suffix = formatting && (typeof formatting.workspaceSuffix === 'string') ? formatting.workspaceSuffix : undefined;
        return suffix ? `${label} [${suffix}]` : label;
    }
};
LabelService = __decorate([
    __param(0, environment_1.IEnvironmentService),
    __param(1, workspace_1.IWorkspaceContextService)
], LabelService);
exports.LabelService = LabelService;
extensions_1.registerSingleton(label_1.ILabelService, LabelService, true);
//# sourceMappingURL=labelService.js.map