"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const minimist = require("vscode-minimist");
const path = require("vs/base/common/path");
const cancellation_1 = require("vs/base/common/cancellation");
const uri_1 = require("vs/base/common/uri");
const modelService_1 = require("vs/editor/common/services/modelService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const resourceConfiguration_1 = require("vs/editor/common/services/resourceConfiguration");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const testConfigurationService_1 = require("vs/platform/configuration/test/common/testConfigurationService");
const environment_1 = require("vs/platform/environment/common/environment");
const descriptors_1 = require("vs/platform/instantiation/common/descriptors");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const platform_1 = require("vs/platform/registry/common/platform");
const search_1 = require("vs/workbench/services/search/common/search");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const testWorkspace_1 = require("vs/platform/workspace/test/common/testWorkspace");
const quickopen_1 = require("vs/workbench/browser/quickopen");
require("vs/workbench/contrib/search/browser/search.contribution"); // load contributions
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
const searchService_1 = require("vs/workbench/services/search/node/searchService");
const untitledEditorService_1 = require("vs/workbench/services/untitled/common/untitledEditorService");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
// Checkout sources to run against:
// git clone --separate-git-dir=testGit --no-checkout --single-branch https://chromium.googlesource.com/chromium/src testWorkspace
// cd testWorkspace; git checkout 39a7f93d67f7
// Run from repository root folder with (test.bat on Windows): ./scripts/test.sh --grep QuickOpen.performance --timeout 180000 --testWorkspace <path>
suite.skip('QuickOpen performance (integration)', () => {
    test('Measure', () => {
        if (process.env['VSCODE_PID']) {
            return undefined; // TODO@Christoph find out why test fails when run from within VS Code
        }
        const n = 3;
        const argv = minimist(process.argv);
        const testWorkspaceArg = argv['testWorkspace'];
        const verboseResults = argv['verboseResults'];
        const testWorkspacePath = testWorkspaceArg ? path.resolve(testWorkspaceArg) : __dirname;
        const telemetryService = new TestTelemetryService();
        const configurationService = new testConfigurationService_1.TestConfigurationService();
        const textResourcePropertiesService = new workbenchTestServices_1.TestTextResourcePropertiesService(configurationService);
        const instantiationService = new instantiationService_1.InstantiationService(new serviceCollection_1.ServiceCollection([telemetry_1.ITelemetryService, telemetryService], [configuration_1.IConfigurationService, configurationService], [resourceConfiguration_1.ITextResourcePropertiesService, textResourcePropertiesService], [modelService_1.IModelService, new modelServiceImpl_1.ModelServiceImpl(configurationService, textResourcePropertiesService)], [workspace_1.IWorkspaceContextService, new workbenchTestServices_1.TestContextService(testWorkspace_1.testWorkspace(uri_1.URI.file(testWorkspacePath)))], [editorService_1.IEditorService, new workbenchTestServices_1.TestEditorService()], [editorGroupsService_1.IEditorGroupsService, new workbenchTestServices_1.TestEditorGroupsService()], [environment_1.IEnvironmentService, workbenchTestServices_1.TestEnvironmentService], [untitledEditorService_1.IUntitledEditorService, descriptors_1.createSyncDescriptor(untitledEditorService_1.UntitledEditorService)], [search_1.ISearchService, descriptors_1.createSyncDescriptor(searchService_1.LocalSearchService)]));
        const registry = platform_1.Registry.as(quickopen_1.Extensions.Quickopen);
        const descriptor = registry.getDefaultQuickOpenHandler();
        assert.ok(descriptor);
        function measure() {
            const handler = descriptor.instantiate(instantiationService);
            handler.onOpen();
            return handler.getResults('a', cancellation_1.CancellationToken.None).then(result => {
                const uncachedEvent = popEvent();
                assert.strictEqual(uncachedEvent.data.symbols.fromCache, false, 'symbols.fromCache');
                assert.strictEqual(uncachedEvent.data.files.fromCache, true, 'files.fromCache');
                if (testWorkspaceArg) {
                    assert.ok(!!uncachedEvent.data.files.joined, 'files.joined');
                }
                return uncachedEvent;
            }).then(uncachedEvent => {
                return handler.getResults('ab', cancellation_1.CancellationToken.None).then(result => {
                    const cachedEvent = popEvent();
                    assert.strictEqual(uncachedEvent.data.symbols.fromCache, false, 'symbols.fromCache');
                    assert.ok(cachedEvent.data.files.fromCache, 'filesFromCache');
                    handler.onClose(false);
                    return [uncachedEvent, cachedEvent];
                });
            });
        }
        function popEvent() {
            const events = telemetryService.events
                .filter(event => event.name === 'openAnything');
            assert.strictEqual(events.length, 1);
            const event = events[0];
            telemetryService.events.length = 0;
            return event;
        }
        function printResult(data) {
            if (verboseResults) {
                console.log(JSON.stringify(data, null, '  ') + ',');
            }
            else {
                console.log(JSON.stringify({
                    filesfromCacheNotJoined: data.files.fromCache && !data.files.joined,
                    searchLength: data.searchLength,
                    sortedResultDuration: data.sortedResultDuration,
                    filesResultCount: data.files.resultCount,
                    errorCount: data.files.errors && data.files.errors.length || undefined
                }) + ',');
            }
        }
        return measure() // Warm-up first
            .then(() => {
            if (testWorkspaceArg || verboseResults) { // Don't measure by default
                const cachedEvents = [];
                let i = n;
                return (function iterate() {
                    if (!i--) {
                        return undefined;
                    }
                    return measure()
                        .then(([uncachedEvent, cachedEvent]) => {
                        printResult(uncachedEvent.data);
                        cachedEvents.push(cachedEvent);
                        return iterate();
                    });
                })().then(() => {
                    console.log();
                    cachedEvents.forEach(cachedEvent => {
                        printResult(cachedEvent.data);
                    });
                });
            }
            return undefined;
        });
    });
});
class TestTelemetryService {
    constructor() {
        this.isOptedIn = true;
        this.events = [];
    }
    setEnabled(value) {
    }
    publicLog(eventName, data) {
        this.events.push({ name: eventName, data: data });
        return Promise.resolve(undefined);
    }
    publicLog2(eventName, data) {
        return this.publicLog(eventName, data);
    }
    getTelemetryInfo() {
        return Promise.resolve({
            instanceId: 'someValue.instanceId',
            sessionId: 'someValue.sessionId',
            machineId: 'someValue.machineId'
        });
    }
}
//# sourceMappingURL=quickopen.perf.integrationTest.js.map